/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Portal - app.component.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {DOCUMENT} from "@angular/common";
import {
  ChangeDetectionStrategy,
  Component,
  Inject,
  Injector,
  Renderer2,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {Router} from "@angular/router";
import {UserDialog} from "@app/components/dialogs/user/user.dialog";
import {RoutesEnum} from "@app/shared/enums/routes.enum";
import {appActionNameSpace} from "@app/stores/app.action";
import {AppState} from "@app/stores/app.state";
import {AppCartArchiveState} from "@app/stores/cart/archive/app-cart-archive.state";
import {AppNotificationInboxState} from "@app/stores/notification-inbox/app-notification-inbox.state";
import {AppUserState} from "@app/stores/user/app-user.state";
import {Enums} from "@enums";
import {environment} from "@environments/environment";
import {HomeHelper} from "@home/helpers/home.helper";
import {User} from "@models";
import {TranslateService} from "@ngx-translate/core";
import {Store} from "@ngxs/store";
import {ApplicationRolePermissionEnum} from "@shared/enums/application-role-permission.enum";
import {LabelTranslateEnum} from "@shared/enums/label-translate.enum";
import {LocalStorageEnum} from "@shared/enums/local-storage.enum";
import {TourEnum} from "@shared/enums/tour.enum";
import {PermissionUtil} from "@shared/utils/permission.util";
import {TourService} from "ngx-ui-tour-md-menu";
import {
  combineLatest,
  Observable,
} from "rxjs";
import {
  filter,
  map,
  take,
  tap,
} from "rxjs/operators";
import {
  AbstractAppComponent,
  AppStatusService,
  BreakpointService,
  CookieConsentService,
  DialogUtil,
  FrontendVersion,
  GoogleAnalyticsService,
  isNotNullNorUndefined,
  LoggingService,
  MARK_AS_TRANSLATABLE,
  MemoizedUtil,
  NotificationService,
  ScrollService,
  SOLIDIFY_CONSTANTS,
  UrlQueryParamHelper,
} from "solidify-frontend";

const urlSeparator: string = SOLIDIFY_CONSTANTS.URL_SEPARATOR;

@Component({
  selector: "dlcm-root",
  templateUrl: "./app.component.html",
  styleUrls: ["./app.component.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AppComponent extends AbstractAppComponent {

  currentModule: string;
  isHomePage: boolean;

  userRolesObs: Observable<Enums.UserApplicationRole.UserApplicationRoleEnum[]> = MemoizedUtil.select(this._store, AppState, state => state.userRoles);
  currentPhotoUserObs: Observable<string> = MemoizedUtil.select(this._store, AppUserState, state => state.logo);

  numberPendingRequestNotificationInboxObs: Observable<number> = MemoizedUtil.total(this._store, AppNotificationInboxState);
  numberArchiveInCartObs: Observable<number> = MemoizedUtil.total(this._store, AppCartArchiveState);

  frontendVersionObs: Observable<FrontendVersion> = MemoizedUtil.select(this._store, AppState, state => state.frontendVersion);

  get localStorageEnum(): typeof LocalStorageEnum {
    return LocalStorageEnum;
  }

  get routesEnum(): typeof RoutesEnum {
    return RoutesEnum;
  }

  get tourEnum(): typeof TourEnum {
    return TourEnum;
  }

  get labelTranslateEnum(): typeof LabelTranslateEnum {
    return LabelTranslateEnum;
  }

  constructor(protected readonly _store: Store,
              protected readonly _router: Router,
              protected readonly _translate: TranslateService,
              protected readonly _renderer: Renderer2,
              @Inject(DOCUMENT) protected readonly _document: Document,
              protected readonly _injector: Injector,
              protected readonly _notificationService: NotificationService,
              protected readonly _loggingService: LoggingService,
              public readonly appStatusService: AppStatusService,
              public readonly breakpointService: BreakpointService,
              protected readonly _dialog: MatDialog,
              public readonly tourService: TourService,
              protected readonly _scrollService: ScrollService,
              protected readonly _googleAnalyticsService: GoogleAnalyticsService,
              protected readonly _cookieConsentService: CookieConsentService,
  ) {
    super(AppState,
      AppUserState as any,
      _store,
      _router,
      _translate,
      _renderer,
      _document,
      _injector,
      _notificationService,
      _loggingService,
      appStatusService,
      breakpointService,
      _dialog,
      _scrollService,
      _googleAnalyticsService,
      _cookieConsentService,
    );
    this._listenCurrentModule();
    this.subscribe(this._observeLoggedUserForCart());
    this._checkOrcidQueryParam();
  }

  private _observeLoggedUserForCart(): Observable<boolean> {
    return combineLatest([this.isLoggedObs, this.userRolesObs])
      .pipe(
        map(([isLogged, userRoles]) => this.computeDisplayCart(isLogged, userRoles)),
      );
  }

  private _checkOrcidQueryParam(): void {
    if (UrlQueryParamHelper.currentUrlContainsQueryParam(environment.orcidQueryParam)) {
      this.subscribe(this.currentUserObs.pipe(
        filter(user => isNotNullNorUndefined(user)),
        take(1),
        tap((user: User) => {
          DialogUtil.open(this._dialog, UserDialog, user, {
            width: "90%",
          });
          this._notificationService.showSuccess(MARK_AS_TRANSLATABLE("notification.orcid.associated"), {orcid: user?.person?.orcid});
        }),
      ));
    }
  }

  private _listenCurrentModule(): void {
    this.subscribe(this.urlStateObs
      .pipe(
        tap(urlState => {
          this.ignoreGrid = false;
          this.isHomePage = false;
          if (urlState) {
            const url = urlState.url;

            const urlHomeDetail = urlSeparator + RoutesEnum.homeDetail;
            if (url.startsWith(urlHomeDetail)) {
              this.currentModule = undefined;
              if (url.lastIndexOf(urlSeparator) === urlHomeDetail.length) {
                this.ignoreGrid = true;
              }
              return;
            }

            if (url.startsWith(urlSeparator + RoutesEnum.about)) {
              this.currentModule = undefined;
              this.isHomePage = false;
              this.ignoreGrid = true;
              return;
            }

            if (url.startsWith(urlSeparator + RoutesEnum.homeSearch)) {
              this.currentModule = undefined;
              this.isHomePage = true;
              this.ignoreGrid = true;
              return;
            }

            if (url.startsWith(urlSeparator + RoutesEnum.homePage)) {
              this.currentModule = RoutesEnum.homePage;
              this.isHomePage = true;
              this.ignoreGrid = true;
              return;
            }

            if (url.startsWith(urlSeparator + RoutesEnum.deposit)) {
              this.currentModule = RoutesEnum.deposit;
              return;
            }

            if (url.startsWith(urlSeparator + RoutesEnum.preservationSpace)) {
              this.currentModule = RoutesEnum.preservationSpace;
              return;
            }

            if (url.startsWith(urlSeparator + RoutesEnum.admin)) {
              this.currentModule = RoutesEnum.admin;
              return;
            }

            if (url.startsWith(urlSeparator + RoutesEnum.preservationPlanning)) {
              this.currentModule = RoutesEnum.preservationPlanning;
              return;
            }

            if (url.startsWith(urlSeparator + RoutesEnum.order)) {
              this.currentModule = RoutesEnum.order;
              return;
            }

            this.currentModule = null;
          }
        }),
      ),
    );
  }

  navigateToHome(): void {
    this.navigate(RoutesEnum.homePage);
  }

  computeDisplayCart(isLogged: boolean, userRoles: Enums.UserApplicationRole.UserApplicationRoleEnum[]): boolean {
    this.displayCart = PermissionUtil.isUserHavePermission(isLogged, ApplicationRolePermissionEnum.userPermission, userRoles);
    return this.displayCart;
  }

  search(searchTerm: string): void {
    HomeHelper.navigateToSearch(this._store, searchTerm);
  }

  openUserGuideSidebar(): void {
    this._store.dispatch(new appActionNameSpace.ChangeDisplaySidebarUserGuide(true));
  }

  initiateNewLogin(): void {
    this._store.dispatch(new appActionNameSpace.InitiateNewLogin());
  }
}
