/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Portal - user.dialog.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {
  ChangeDetectionStrategy,
  Component,
  Inject,
  OnInit,
  ViewChild,
} from "@angular/core";
import {
  MAT_DIALOG_DATA,
  MatDialogRef,
} from "@angular/material/dialog";
import {AppAuthorizedOrganizationalUnitState} from "@app/stores/authorized-organizational-unit/app-authorized-organizational-unit.state";
import {AppPersonAction} from "@app/stores/person/app-person.action";
import {AppPersonState} from "@app/stores/person/app-person.state";
import {AppPersonNotificationTypeAction} from "@app/stores/person/notification-type/app-person-notification-type.action";
import {
  AppPersonNotificationTypeState,
  defaultAppPersonNotificationTypeStateStateQueryParameters,
} from "@app/stores/person/notification-type/app-person-notification-type.state";
import {AppUserAction} from "@app/stores/user/app-user.action";
import {AppUserState} from "@app/stores/user/app-user.state";
import {
  NotificationTypeDlcm,
  Person,
  User,
} from "@models";
import {
  Actions,
  Select,
  Store,
} from "@ngxs/store";
import {SharedAbstractDialog} from "@shared/components/dialogs/shared-abstract/shared-abstract.dialog";
import {SharedPersonFormPresentational} from "@shared/components/presentationals/shared-person-form/shared-person-form.presentational";
import {SecurityService} from "@shared/services/security.service";
import {ResourceLogoNameSpace} from "@shared/stores/resource-logo/resource-logo-namespace.model";
import {Observable} from "rxjs";
import {tap} from "rxjs/operators";
import {
  isNonEmptyArray,
  isNotNullNorUndefined,
  isNullOrUndefined,
  MemoizedUtil,
  ModelFormControlEvent,
  ofSolidifyActionCompleted,
} from "solidify-frontend";

@Component({
  selector: "dlcm-user-dialog",
  templateUrl: "./user.dialog.html",
  styleUrls: ["./user.dialog.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class UserDialog extends SharedAbstractDialog<User, Person> implements OnInit {
  currentPersonObs: Observable<Person> = MemoizedUtil.current(this._store, AppPersonState);
  isLoadingPersonObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, AppPersonState);
  selectedNotificationTypeObs: Observable<NotificationTypeDlcm[]> = MemoizedUtil.selected(this._store, AppPersonNotificationTypeState);
  @Select(AppPersonState.isReadyToBeDisplayed) isReadyToBeDisplayedObs: Observable<boolean>;

  userAvatarActionNameSpace: ResourceLogoNameSpace = AppUserAction;
  userState: typeof AppUserState = AppUserState;

  validateEnable: boolean = false;

  isStewardOfAnyOrgUnit: boolean;
  isApproverOfAnyOrgUnit: boolean;

  @ViewChild("formPresentational")
  readonly formPresentational: SharedPersonFormPresentational;

  constructor(protected readonly _store: Store,
              protected readonly _dialogRef: MatDialogRef<UserDialog>,
              @Inject(MAT_DIALOG_DATA) public readonly user: User,
              protected readonly _actions$: Actions,
              protected readonly _securityService: SecurityService) {
    super(_dialogRef, user);
  }

  ngOnInit(): void {
    // Not necessary to refresh current user and cause ExpressionChangedAfterItHasBeenCheckedError when open profile info in deposit-edit.routable (cf DLCM-795)
    // this.store.dispatch(new AppPersonAction.GetById(this.user.person.resId));
    this._store.dispatch(new AppPersonNotificationTypeAction.GetAll(this.personResId, defaultAppPersonNotificationTypeStateStateQueryParameters()));

    const listAuthorizedOrgUnit = MemoizedUtil.listSnapshot(this._store, AppAuthorizedOrganizationalUnitState);
    if (isNonEmptyArray(listAuthorizedOrgUnit)) {
      this.isStewardOfAnyOrgUnit = this._securityService.isStewardOfAnyOrgUnit(listAuthorizedOrgUnit);
    }
    if (isNonEmptyArray(listAuthorizedOrgUnit)) {
      this.isApproverOfAnyOrgUnit = this._securityService.isApproverOfAnyOrgUnit(listAuthorizedOrgUnit);
    }
  }

  savePersonInfo($event: ModelFormControlEvent<Person>): void {
    this._store.dispatch(new AppPersonAction.Update($event));
  }

  getUser(): User {
    if (isNullOrUndefined(this.formPresentational)) {
      return this.user;
    }
    return this.formPresentational.form.value as User;
  }

  get personResId(): string {
    return this.user.person.resId;
  }

  isAvatarPresent(): boolean {
    return isNotNullNorUndefined(this.user.person.avatar);
  }

  activateButton($event: Event): void {
    this.validateEnable = true;
  }

  validate(): void {
    this.subscribe(this._actions$.pipe(
      ofSolidifyActionCompleted(AppPersonAction.UpdateSuccess),
      tap(result => {
        this.submit(result.action.model);
      }),
    ));
    this.formPresentational?.onSubmit();
  }
}
