/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Portal - admin-institution.state.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {
  AdminInstitutionAction,
  adminInstitutionActionNameSpace,
} from "@admin/institution/stores/admin-institution.action";
import {AdminInstitutionOrganizationalUnitAction} from "@admin/institution/stores/organizational-unit/admin-institution-organizational-unit.action";
import {
  AdminInstitutionOrganizationalUnitState,
  AdminInstitutionOrganizationalUnitStateModel,
} from "@admin/institution/stores/organizational-unit/admin-institution-organizational-unit.state";
import {HttpClient} from "@angular/common/http";
import {Injectable} from "@angular/core";
import {Institution} from "@models";
import {
  Action,
  Actions,
  Selector,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {ApiEnum} from "@shared/enums/api.enum";
import {
  RoutesEnum,
  urlSeparator,
} from "@shared/enums/routes.enum";
import {StateEnum} from "@shared/enums/state.enum";
import {ResourceLogoStateModel} from "@shared/stores/resource-logo/resource-logo-state.model";
import {
  defaultResourceLogoStateInitValue,
  ResourceLogoState,
  ResourceLogoStateModeEnum,
} from "@shared/stores/resource-logo/resource-logo.state";
import {Observable} from "rxjs";
import {
  map,
  tap,
} from "rxjs/operators";
import {
  ApiService,
  defaultAssociationStateInitValue,
  isNotNullNorUndefined,
  isNullOrUndefined,
  MARK_AS_TRANSLATABLE,
  NotificationService,
  ofSolidifyActionCompleted,
  OverrideDefaultAction,
  StoreUtil,
} from "solidify-frontend";

export interface AdminInstitutionStateModel extends ResourceLogoStateModel<Institution> {
  admin_institution_organizationalUnit: AdminInstitutionOrganizationalUnitStateModel;

}

@Injectable()
@State<AdminInstitutionStateModel>({
    name: StateEnum.admin_institution,
    defaults: {
      ...defaultResourceLogoStateInitValue(),
      admin_institution_organizationalUnit: {...defaultAssociationStateInitValue()},
    },
    children: [
      AdminInstitutionOrganizationalUnitState,
    ],
  },
)
export class AdminInstitutionState extends ResourceLogoState<AdminInstitutionStateModel, Institution> {
  constructor(protected readonly _apiService: ApiService,
              protected readonly _store: Store,
              protected readonly _notificationService: NotificationService,
              protected readonly _actions$: Actions,
              protected readonly _httpClient: HttpClient) {
    super(_apiService, _store, _notificationService, _actions$, {
      nameSpace: adminInstitutionActionNameSpace,
      routeRedirectUrlAfterSuccessCreateAction: (resId: string) => RoutesEnum.adminInstitutionDetail + urlSeparator + resId,
      routeRedirectUrlAfterSuccessUpdateAction: (resId: string) => RoutesEnum.adminInstitutionDetail + urlSeparator + resId,
      routeRedirectUrlAfterSuccessDeleteAction: RoutesEnum.adminInstitution,
      notificationResourceCreateSuccessTextToTranslate: MARK_AS_TRANSLATABLE("admin.institution.notification.resource.create"),
      notificationResourceDeleteSuccessTextToTranslate: MARK_AS_TRANSLATABLE("admin.institution.notification.resource.delete"),
      notificationResourceUpdateSuccessTextToTranslate: MARK_AS_TRANSLATABLE("admin.institution.notification.resource.update"),
    }, _httpClient, ResourceLogoStateModeEnum.logo);
  }

  protected get _urlResource(): string {
    return ApiEnum.adminInstitutions;
  }

  protected get _urlLogoResource(): string {
    return this._urlResource;
  }

  @Selector()
  static isLoading(state: AdminInstitutionStateModel): boolean {
    return StoreUtil.isLoadingState(state);
  }

  @Selector()
  static isLoadingWithDependency(state: AdminInstitutionStateModel): boolean {
    return this.isLoading(state)
      || StoreUtil.isLoadingState(state.admin_institution_organizationalUnit);
  }

  @Selector()
  static currentTitle(state: AdminInstitutionStateModel): string | undefined {
    if (isNullOrUndefined(state.current)) {
      return undefined;
    }
    return state.current.name;
  }

  @Selector()
  static isReadyToBeDisplayed(state: AdminInstitutionStateModel): boolean {
    return this.isReadyToBeDisplayedInCreateMode
      && !isNullOrUndefined(state.current)
      && !isNullOrUndefined(state.admin_institution_organizationalUnit.selected);
  }

  @Selector()
  static isReadyToBeDisplayedInCreateMode(state: AdminInstitutionStateModel): boolean {
    return true;
  }

  @OverrideDefaultAction()
  @Action(AdminInstitutionAction.GetByIdSuccess)
  getByIdSuccess(ctx: StateContext<AdminInstitutionStateModel>, action: AdminInstitutionAction.GetByIdSuccess): void {
    super.getByIdSuccess(ctx, action);
    if (isNotNullNorUndefined(action.model.logo)) {
      ctx.dispatch(new AdminInstitutionAction.GetPhoto(action.model.resId));
    }
  }

  @OverrideDefaultAction()
  @Action(AdminInstitutionAction.Create)
  create(ctx: StateContext<AdminInstitutionStateModel>, action: AdminInstitutionAction.Create): Observable<Institution> {
    return super._internalCreate(ctx, action)
      .pipe(
        tap(model => {
          this._updateSubResource(model, action, ctx)
            .subscribe(success => {
              if (success) {
                ctx.dispatch(new AdminInstitutionAction.CreateSuccess(action, model));
              } else {
                ctx.dispatch(new AdminInstitutionAction.CreateFail(action));
              }
            });
        }),
      );
  }

  @OverrideDefaultAction()
  @Action(AdminInstitutionAction.Update)
  update(ctx: StateContext<AdminInstitutionStateModel>, action: AdminInstitutionAction.Update): Observable<Institution> {
    return super._internalUpdate(ctx, action)
      .pipe(
        tap(model => {
          this.subscribe(
            this._updateSubResource(model, action, ctx).pipe(
              tap(success => {
                if (success) {
                  ctx.dispatch(new AdminInstitutionAction.UpdateSuccess(action, model));
                } else {
                  ctx.dispatch(new AdminInstitutionAction.UpdateFail(action));
                }
              }),
            ));
        }),
      );
  }

  private _updateSubResource(model: Institution, action: AdminInstitutionAction.Create | AdminInstitutionAction.Update, ctx: StateContext<AdminInstitutionStateModel>): Observable<boolean> {
    const institutionId = model.resId;

    const newOrgUnitIds = action.modelFormControlEvent.model.organizationalUnits.map(p => p.resId);

    return StoreUtil.dispatchSequentialActionAndWaitForSubActionsCompletion(ctx, [
      {
        action: new AdminInstitutionOrganizationalUnitAction.Update(institutionId, newOrgUnitIds),
        subActionCompletions: [
          this._actions$.pipe(ofSolidifyActionCompleted(AdminInstitutionOrganizationalUnitAction.UpdateSuccess)),
          this._actions$.pipe(ofSolidifyActionCompleted(AdminInstitutionOrganizationalUnitAction.UpdateFail)),
        ],
      },
    ]).pipe(
      map(result => result.success),
    );
  }

}
