/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Portal - admin-oai-set.state.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {
  AdminOaiSetAction,
  adminOaiSetActionNameSpace,
} from "@admin/oai-set/stores/admin-oai-set.action";
import {Injectable} from "@angular/core";
import {OaiSet} from "@models";
import {
  Action,
  Actions,
  Selector,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {ApiEnum} from "@shared/enums/api.enum";
import {
  RoutesEnum,
  urlSeparator,
} from "@shared/enums/routes.enum";
import {StateEnum} from "@shared/enums/state.enum";
import {SharedOrgUnitAction} from "@shared/stores/organizational-unit/shared-organizational-unit.action";
import {Observable} from "rxjs";
import {
  map,
  tap,
} from "rxjs/operators";
import {
  ActionSubActionCompletionsWrapper,
  ApiService,
  defaultResourceStateInitValue,
  isNullOrUndefined,
  MARK_AS_TRANSLATABLE,
  NotificationService,
  ofSolidifyActionCompleted,
  ResourceState,
  ResourceStateModel,
  SOLIDIFY_CONSTANTS,
  StoreUtil,
} from "solidify-frontend";

export interface AdminOaiSetStateModel extends ResourceStateModel<OaiSet> {
}

@Injectable()
@State<AdminOaiSetStateModel>({
  name: StateEnum.admin_oaiSet,
  defaults: {
    ...defaultResourceStateInitValue(),
  },
})
export class AdminOaiSetState extends ResourceState<AdminOaiSetStateModel, OaiSet> {
  constructor(protected readonly _apiService: ApiService,
              protected readonly _store: Store,
              protected readonly _notificationService: NotificationService,
              protected readonly _actions$: Actions) {
    super(_apiService, _store, _notificationService, _actions$, {
      nameSpace: adminOaiSetActionNameSpace,
      routeRedirectUrlAfterSuccessCreateAction: (resId: string) => RoutesEnum.adminOaiSetDetail + urlSeparator + resId,
      routeRedirectUrlAfterSuccessUpdateAction: (resId: string) => RoutesEnum.adminOaiSetDetail + urlSeparator + resId,
      routeRedirectUrlAfterSuccessDeleteAction: RoutesEnum.adminOaiSet,
      notificationResourceCreateSuccessTextToTranslate: MARK_AS_TRANSLATABLE("admin.oai-set.notification.resource.create"),
      notificationResourceDeleteSuccessTextToTranslate: MARK_AS_TRANSLATABLE("admin.oai-set.notification.resource.delete"),
      notificationResourceUpdateSuccessTextToTranslate: MARK_AS_TRANSLATABLE("admin.oai-set.notification.resource.update"),
    });
  }

  protected get _urlResource(): string {
    return ApiEnum.oaiInfoOaiSets;
  }

  @Selector()
  static isLoading(state: AdminOaiSetStateModel): boolean {
    return StoreUtil.isLoadingState(state);
  }

  @Selector()
  static isLoadingWithDependency(state: AdminOaiSetStateModel): boolean {
    return this.isLoading(state);
  }

  @Selector()
  static isReadyToBeDisplayed(state: AdminOaiSetStateModel): boolean {
    return this.isReadyToBeDisplayedInCreateMode
      && !isNullOrUndefined(state.current);
  }

  @Selector()
  static isReadyToBeDisplayedInCreateMode(state: AdminOaiSetStateModel): boolean {
    return true;
  }

  @Selector()
  static currentTitle(state: AdminOaiSetStateModel): string | undefined {
    if (isNullOrUndefined(state.current)) {
      return undefined;
    }
    return state.current.name;
  }

  @Action(AdminOaiSetAction.LoadResource)
  loadResource(ctx: StateContext<AdminOaiSetStateModel>, action: AdminOaiSetAction.LoadResource): Observable<boolean> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });

    return StoreUtil.dispatchParallelActionAndWaitForSubActionsCompletion(ctx, [
      {
        action: new SharedOrgUnitAction.GetAll(undefined, false, false),
        subActionCompletions: [
          this._actions$.pipe(ofSolidifyActionCompleted(SharedOrgUnitAction.GetAllSuccess)),
          this._actions$.pipe(ofSolidifyActionCompleted(SharedOrgUnitAction.GetAllFail)),
        ],
      },
    ]).pipe(
      map(result => {
        if (result.success) {
          ctx.dispatch(new AdminOaiSetAction.LoadResourceSuccess(action));
        } else {
          ctx.dispatch(new AdminOaiSetAction.LoadResourceFail(action));
        }
        return result.success;
      }),
    );
  }

  @Action(AdminOaiSetAction.BulkCreate)
  bulkCreate(ctx: StateContext<AdminOaiSetStateModel>, action: AdminOaiSetAction.BulkCreate): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });

    const listActionSubActionCompletionsWrapper: ActionSubActionCompletionsWrapper[] = [];
    const listOrgUnit = action.listOrgUnit;
    listOrgUnit.forEach(orgUnit => {
      const orgUnitName = orgUnit.name;
      const oaiSet = {
        spec: `orgunit:${orgUnitName.toLowerCase().replace(SOLIDIFY_CONSTANTS.SPACE, SOLIDIFY_CONSTANTS.DASH)}`,
        name: orgUnitName,
        description: `Set for ${orgUnitName}`,
        query: `aip-organizational-unit:${orgUnit.resId}`,
        enabled: false,
      } as OaiSet;
      listActionSubActionCompletionsWrapper.push({
        action: new AdminOaiSetAction.CreateDuringBulk({
          model: oaiSet,
        }),
        subActionCompletions: [
          this._actions$.pipe(ofSolidifyActionCompleted(AdminOaiSetAction.CreateDuringBulkSuccess)),
          this._actions$.pipe(ofSolidifyActionCompleted(AdminOaiSetAction.CreateFail)),
        ],
      });
    });
    this.subscribe(StoreUtil.dispatchSequentialActionAndWaitForSubActionsCompletion(ctx, listActionSubActionCompletionsWrapper).pipe(tap(result => {
      if (result.success) {
        ctx.dispatch(new AdminOaiSetAction.BulkCreateSuccess(action));
      } else {
        ctx.dispatch(new AdminOaiSetAction.BulkCreateFail(action));
      }
    })));
  }

  @Action(AdminOaiSetAction.BulkCreateSuccess)
  bulkCreateSuccess(ctx: StateContext<AdminOaiSetStateModel>, action: AdminOaiSetAction.BulkCreateSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });

    StoreUtil.navigateIfDefined(ctx, RoutesEnum.adminOaiSet, null);
    this._notificationService.showSuccess(MARK_AS_TRANSLATABLE("admin.oai-set.notification.bulkCreateSuccess"));
  }

  @Action(AdminOaiSetAction.BulkCreateFail)
  bulkCreateFail(ctx: StateContext<AdminOaiSetStateModel>, action: AdminOaiSetAction.BulkCreateFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
  }

  @Action(AdminOaiSetAction.CreateDuringBulk)
  createDuringBulk(ctx: StateContext<AdminOaiSetStateModel>, action: AdminOaiSetAction.CreateDuringBulk): Observable<OaiSet> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });

    return super._internalCreate(ctx, action as AdminOaiSetAction.Create).pipe(
      tap(model => {
        this._store.dispatch(new AdminOaiSetAction.CreateDuringBulkSuccess(action));
      }),
    );
  }
}
