/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Portal - admin-person-form.presentational.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {OrganizationalUnitRole} from "@admin/models/organizational-unit-role.model";
import {AdminPersonAction} from "@admin/person/stores/admin-person.action";
import {AdminPersonState} from "@admin/person/stores/admin-person.state";
import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  ElementRef,
  Injector,
  Input,
} from "@angular/core";
import {
  FormBuilder,
  Validators,
} from "@angular/forms";
import {sharedInstitutionActionNameSpace} from "@app/shared/stores/institution/shared-institution.action";
import {
  Institution,
  Person,
  Role,
  User,
} from "@models";
import {PersonOrgUnitRoleMode} from "@shared/components/containers/shared-table-person-orgunit-role/shared-table-person-orgunit-role.container";
import {SharedInstitutionOverlayPresentational} from "@shared/components/presentationals/shared-institution-overlay/shared-institution-overlay.presentational";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {LabelTranslateEnum} from "@shared/enums/label-translate.enum";
import {RoutesEnum} from "@shared/enums/routes.enum";
import {BaseFormDefinition} from "@shared/models/base-form-definition.model";
import {SharedInstitutionState} from "@shared/stores/institution/shared-institution.state";
import {ResourceLogoNameSpace} from "@shared/stores/resource-logo/resource-logo-namespace.model";
import {
  AbstractFormPresentational,
  isNotNullNorUndefined,
  isNullOrUndefined,
  OrderEnum,
  PropertyName,
  ResourceNameSpace,
  SolidifyValidator,
  Sort,
  Type,
} from "solidify-frontend";

@Component({
  selector: "dlcm-admin-person-form",
  templateUrl: "./admin-person-form.presentational.html",
  styleUrls: ["./admin-person-form.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AdminPersonFormPresentational extends AbstractFormPresentational<Person> {
  formDefinition: FormComponentFormDefinition = new FormComponentFormDefinition();

  personAvatarActionNameSpace: ResourceLogoNameSpace = AdminPersonAction;
  personState: typeof AdminPersonState = AdminPersonState;

  @Input()
  selectedOrgUnitRole: OrganizationalUnitRole[];

  @Input()
  selectedInstitutions: Institution[];

  @Input()
  listRole: Role[];

  sharedInstitutionSort: Sort<Institution> = {
    field: "name",
    order: OrderEnum.ascending,
  };
  sharedInstitutionActionNameSpace: ResourceNameSpace = sharedInstitutionActionNameSpace;
  sharedInstitutionState: typeof SharedInstitutionState = SharedInstitutionState;

  institutionOverlayComponent: Type<SharedInstitutionOverlayPresentational> = SharedInstitutionOverlayPresentational;

  get personOrgUnitRoleMode(): typeof PersonOrgUnitRoleMode {
    return PersonOrgUnitRoleMode;
  }

  get labelTranslateEnum(): typeof LabelTranslateEnum {
    return LabelTranslateEnum;
  }

  get iconNameEnum(): typeof IconNameEnum {
    return IconNameEnum;
  }

  getPersonResId(): string | undefined {
    if (isNullOrUndefined(this.model)) {
      return undefined;
    }
    return this.model.resId;
  }

  get isAvatarPresent(): boolean {
    return isNotNullNorUndefined(this.model?.avatar);
  }

  getUser(): User {
    if (isNullOrUndefined(this.form)) {
      return this.model;
    }
    return this.form.value as User;
  }

  constructor(protected readonly _changeDetectorRef: ChangeDetectorRef,
              protected readonly _elementRef: ElementRef,
              protected readonly _injector: Injector,
              private readonly _fb: FormBuilder) {
    super(_changeDetectorRef, _elementRef, _injector);
  }

  protected _bindFormTo(person: Person): void {
    this.form = this._fb.group({
      [this.formDefinition.firstName]: [person.firstName, [Validators.required, SolidifyValidator]],
      [this.formDefinition.lastName]: [person.lastName, [Validators.required, SolidifyValidator]],
      [this.formDefinition.orcid]: [person.orcid, [SolidifyValidator]],
      [this.formDefinition.orgUnitRole]: ["", [SolidifyValidator]],
      [this.formDefinition.institutions]: [this.selectedInstitutions.map(i => i.resId)],
    });
  }

  protected _initNewForm(): void {
    this.form = this._fb.group({
      [this.formDefinition.firstName]: ["", [Validators.required, SolidifyValidator]],
      [this.formDefinition.lastName]: ["", [Validators.required, SolidifyValidator]],
      [this.formDefinition.orcid]: ["", [SolidifyValidator]],
      [this.formDefinition.institutions]: [],
      [this.formDefinition.orgUnitRole]: ["", [SolidifyValidator]],
    });
  }

  protected _treatmentBeforeSubmit(person: Person): Person {
    person.institutions = [];
    const listInstitutions = this.form.get(this.formDefinition.institutions).value;
    if (!isNullOrUndefined(listInstitutions)) {
      listInstitutions.forEach(resId => {
        person.institutions.push({resId: resId});
      });
    }
    return person;
  }

  navigateToInstitution(institution: Institution): void {
    this.navigate([RoutesEnum.adminInstitutionDetail, institution.resId]);
  }

  navigateToContributor(): void {
    this.navigate([RoutesEnum.preservationSpaceContributorDetail, this.model.resId]);
  }
}

class FormComponentFormDefinition extends BaseFormDefinition {
  @PropertyName() firstName: string;
  @PropertyName() lastName: string;
  @PropertyName() orcid: string;
  @PropertyName() orgUnitRole: string;
  @PropertyName() institutions: string;
}
