/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Portal - admin-person.state.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {
  AdminPersonAction,
  adminPersonActionNameSpace,
} from "@admin/person/stores/admin-person.action";
import {AdminPersonInstitutionsAction} from "@admin/person/stores/institutions/admin-people-institutions.action";
import {
  AdminPersonInstitutionsState,
  AdminPersonInstitutionsStateModel,
} from "@admin/person/stores/institutions/admin-people-institutions.state";
import {AdminPersonOrgUnitRoleAction} from "@admin/person/stores/person-role/admin-person-orgunit-role.action";
import {
  AdminPersonOrgUnitRoleState,
  AdminPersonOrgUnitRoleStateModel,
  defaultAdminPersonOrgUnitRoleStateModel,
} from "@admin/person/stores/person-role/admin-person-orgunit-role.state";
import {HttpClient} from "@angular/common/http";
import {Injectable} from "@angular/core";
import {DepositStateModel} from "@deposit/stores/deposit.state";
import {Person} from "@models";
import {
  Action,
  Actions,
  Selector,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {ApiEnum} from "@shared/enums/api.enum";
import {
  RoutesEnum,
  urlSeparator,
} from "@shared/enums/routes.enum";
import {StateEnum} from "@shared/enums/state.enum";
import {ResourceLogoStateModel} from "@shared/stores/resource-logo/resource-logo-state.model";
import {
  defaultResourceLogoStateInitValue,
  ResourceLogoState,
  ResourceLogoStateModeEnum,
} from "@shared/stores/resource-logo/resource-logo.state";
import {SharedRoleAction} from "@shared/stores/role/shared-role.action";
import {Observable} from "rxjs";
import {
  map,
  tap,
} from "rxjs/operators";
import {
  ApiService,
  defaultAssociationStateInitValue,
  isArray,
  isNullOrUndefined,
  MARK_AS_TRANSLATABLE,
  NotificationService,
  ofSolidifyActionCompleted,
  OverrideDefaultAction,
  Relation3TiersForm,
  StoreUtil,
} from "solidify-frontend";

export interface AdminPersonStateModel extends ResourceLogoStateModel<Person> {
  admin_person_organizationalUnitRole?: AdminPersonOrgUnitRoleStateModel;
  admin_person_institutions?: AdminPersonInstitutionsStateModel;
}

@Injectable()
@State<AdminPersonStateModel>({
  name: StateEnum.admin_person,
  defaults: {
    ...defaultResourceLogoStateInitValue(),
    admin_person_organizationalUnitRole: {...defaultAdminPersonOrgUnitRoleStateModel()},
    admin_person_institutions: {...defaultAssociationStateInitValue()},
  },
  children: [
    AdminPersonOrgUnitRoleState,
    AdminPersonInstitutionsState,
  ],
})
export class AdminPersonState extends ResourceLogoState<AdminPersonStateModel, Person> {
  constructor(protected readonly _apiService: ApiService,
              protected readonly _store: Store,
              protected readonly _notificationService: NotificationService,
              protected readonly _actions$: Actions,
              protected readonly _httpClient: HttpClient) {
    super(_apiService, _store, _notificationService, _actions$, {
      nameSpace: adminPersonActionNameSpace,
      routeRedirectUrlAfterSuccessCreateAction: (resId: string) => RoutesEnum.adminPersonDetail + urlSeparator + resId,
      routeRedirectUrlAfterSuccessUpdateAction: (resId: string) => RoutesEnum.adminPersonDetail + urlSeparator + resId,
      routeRedirectUrlAfterSuccessDeleteAction: RoutesEnum.adminPerson,
      notificationResourceCreateSuccessTextToTranslate: MARK_AS_TRANSLATABLE("admin.person.notification.resource.create"),
      notificationResourceDeleteSuccessTextToTranslate: MARK_AS_TRANSLATABLE("admin.person.notification.resource.delete"),
      notificationResourceUpdateSuccessTextToTranslate: MARK_AS_TRANSLATABLE("admin.person.notification.resource.update"),
    }, _httpClient, ResourceLogoStateModeEnum.avatar);
  }

  protected get _urlResource(): string {
    return ApiEnum.adminPeople;
  }

  protected get _urlLogoResource(): string {
    return this._urlResource;
  }

  @Selector()
  static isLoading(state: AdminPersonStateModel): boolean {
    return StoreUtil.isLoadingState(state);
  }

  @Selector()
  static isLoadingWithDependency(state: AdminPersonStateModel): boolean {
    return this.isLoading(state)
      || StoreUtil.isLoadingState(state.admin_person_organizationalUnitRole)
      || StoreUtil.isLoadingState(state.admin_person_institutions);
  }

  @Selector()
  static currentTitle(state: AdminPersonStateModel): string | undefined {
    if (isNullOrUndefined(state.current)) {
      return undefined;
    }
    return state.current.fullName;
  }

  @Selector()
  static isReadyToBeDisplayed(state: AdminPersonStateModel): boolean {
    return this.isReadyToBeDisplayedInCreateMode
      && !isNullOrUndefined(state.current)
      && !isNullOrUndefined(state.admin_person_organizationalUnitRole.selected)
      && !isNullOrUndefined(state.admin_person_institutions.selected);
  }

  @Selector()
  static isReadyToBeDisplayedInCreateMode(state: AdminPersonStateModel): boolean {
    return true;
  }

  @Action(AdminPersonAction.LoadResource)
  loadResource(ctx: StateContext<DepositStateModel>, action: AdminPersonAction.LoadResource): Observable<boolean> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });

    return StoreUtil.dispatchParallelActionAndWaitForSubActionsCompletion(ctx, [
      {
        action: new SharedRoleAction.GetAll(undefined, false, false),
        subActionCompletions: [
          this._actions$.pipe(ofSolidifyActionCompleted(SharedRoleAction.GetAllSuccess)),
          this._actions$.pipe(ofSolidifyActionCompleted(SharedRoleAction.GetAllFail)),
        ],
      },
    ]).pipe(
      map(result => {
        if (result.success) {
          ctx.dispatch(new AdminPersonAction.LoadResourceSuccess(action));
        } else {
          ctx.dispatch(new AdminPersonAction.LoadResourceFail(action));
        }
        return result.success;
      }),
    );
  }

  @OverrideDefaultAction()
  @Action(AdminPersonAction.Create)
  create(ctx: StateContext<AdminPersonStateModel>, action: AdminPersonAction.Create): Observable<Person> {
    return super._internalCreate(ctx, action)
      .pipe(
        tap(model => {
          this._updateSubResource(model, action, ctx)
            .subscribe(success => {
              if (success) {
                ctx.dispatch(new AdminPersonAction.CreateSuccess(action, model));
              } else {
                ctx.dispatch(new AdminPersonAction.CreateFail(action));
              }
            });
        }),
      );
  }

  @OverrideDefaultAction()
  @Action(AdminPersonAction.Update)
  update(ctx: StateContext<AdminPersonStateModel>, action: AdminPersonAction.Update): Observable<Person> {
    return super._internalUpdate(ctx, action)
      .pipe(
        tap(model => {
          this._updateSubResource(model, action, ctx)
            .subscribe(success => {
              if (success) {
                ctx.dispatch(new AdminPersonAction.UpdateSuccess(action, model));
              } else {
                ctx.dispatch(new AdminPersonAction.UpdateFail(action));
              }
            });
        }),
      );
  }

  private _updateSubResource(model: Person, action: AdminPersonAction.Create | AdminPersonAction.Update, ctx: StateContext<AdminPersonStateModel>): Observable<boolean> {
    const personId = model.resId;
    const newOrgUnitRole = action.modelFormControlEvent.formControl.get("orgUnitRole").value as Relation3TiersForm[];
    newOrgUnitRole.forEach((elem) => {
      if (!isNullOrUndefined(elem.listId) && !isArray(elem.listId)) {
        elem.listId = [elem.listId];
      }
    });
    const newInstitutions = action.modelFormControlEvent.model.institutions.map(o => o.resId);

    return StoreUtil.dispatchParallelActionAndWaitForSubActionsCompletion(ctx, [
      {
        action: new AdminPersonOrgUnitRoleAction.Update(personId, newOrgUnitRole),
        subActionCompletions: [
          this._actions$.pipe(ofSolidifyActionCompleted(AdminPersonOrgUnitRoleAction.UpdateSuccess)),
          this._actions$.pipe(ofSolidifyActionCompleted(AdminPersonOrgUnitRoleAction.UpdateFail)),
        ],
      },
      {
        action: new AdminPersonInstitutionsAction.Update(personId, newInstitutions),
        subActionCompletions: [
          this._actions$.pipe(ofSolidifyActionCompleted(AdminPersonInstitutionsAction.UpdateSuccess)),
          this._actions$.pipe(ofSolidifyActionCompleted(AdminPersonInstitutionsAction.UpdateFail)),
        ],
      },
    ]).pipe(
      map(result => result.success),
    );
  }
}
