/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Portal - admin-research-domain-form.presentational.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  ElementRef,
  Injector,
  Input,
} from "@angular/core";
import {
  FormBuilder,
  Validators,
} from "@angular/forms";
import {
  Language,
  ResearchDomain,
} from "@models";
import {LabelTranslateEnum} from "@shared/enums/label-translate.enum";
import {BaseFormDefinition} from "@shared/models/base-form-definition.model";
import {ResearchDomainLabel} from "@shared/models/business/research-domain-label.model";
import {
  AbstractFormPresentational,
  isNullOrUndefined,
  PropertyName,
  SolidifyValidator,
} from "solidify-frontend";

@Component({
  selector: "dlcm-admin-research-domain-form",
  templateUrl: "./admin-research-domain-form.presentational.html",
  styleUrls: ["./admin-research-domain-form.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AdminResearchDomainFormPresentational extends AbstractFormPresentational<ResearchDomain> {
  get labelTranslateEnum(): typeof LabelTranslateEnum {
    return LabelTranslateEnum;
  }

  private _languages: Language[];

  @Input()
  set languages(value: Language[]) {
    this._languages = value;

    if (isNullOrUndefined(this.languages)) {
      return;
    }
    this.languages.forEach(language => {
      if (isNullOrUndefined(this.model) || isNullOrUndefined(this.model.labels)) {
        this._addControl(language.resId);
        return;
      }
      const existingValue = (this.model.labels as ResearchDomainLabel[]).find(label => label.language.resId === language.resId);
      if (isNullOrUndefined(existingValue)) {
        this._addControl(language.resId);
        return;
      }
      this._addControl(language.resId, existingValue.text);
    });
  }

  get languages(): Language[] {
    return this._languages;
  }

  private _addControl(languageResId: string, value: string | undefined = undefined): void {
    this.form.addControl(languageResId, this._fb.control(value));
  }

  public formDefinition: FormComponentFormDefinition = new FormComponentFormDefinition();

  constructor(protected readonly _changeDetectorRef: ChangeDetectorRef,
              protected readonly _elementRef: ElementRef,
              protected readonly _injector: Injector,
              private readonly _fb: FormBuilder) {
    super(_changeDetectorRef, _elementRef, _injector);
  }

  protected _initNewForm(): void {
    this.form = this._fb.group({
      [this.formDefinition.name]: ["", [Validators.required, SolidifyValidator]],
      [this.formDefinition.source]: ["", [Validators.required, SolidifyValidator]],
      [this.formDefinition.code]: ["", [Validators.required, SolidifyValidator]],
    });
  }

  protected _bindFormTo(researchDomains: ResearchDomain): void {
    this.form = this._fb.group({
      [this.formDefinition.name]: [researchDomains.name, [Validators.required, SolidifyValidator]],
      [this.formDefinition.source]: [researchDomains.source, [Validators.required, SolidifyValidator]],
      [this.formDefinition.code]: [researchDomains.code, [Validators.required, SolidifyValidator]],
    });
  }

  protected _treatmentBeforeSubmit(researchDomains: ResearchDomain): ResearchDomain {
    const newLabels: ResearchDomainLabel[] = [];
    this.languages.forEach(language => {
      newLabels.push({
        text: researchDomains[language.resId],
        language: language,
      });
    });
    return {
      code: researchDomains.code,
      name: researchDomains.name,
      source: researchDomains.source,
      labels: newLabels,
    };
  }
}

class FormComponentFormDefinition extends BaseFormDefinition {
  @PropertyName() name: string;
  @PropertyName() source: string;
  @PropertyName() code: string;
}
