/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Portal - admin-user.state.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {
  AdminUserAction,
  adminUserActionNameSpace,
} from "@admin/user/stores/admin-user.action";
import {HttpClient} from "@angular/common/http";
import {Injectable} from "@angular/core";
import {ApiActionNameEnum} from "@app/shared/enums/api-action-name.enum";
import {User} from "@models";
import {
  Action,
  Actions,
  Selector,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {ApiEnum} from "@shared/enums/api.enum";
import {
  RoutesEnum,
  urlSeparator,
} from "@shared/enums/routes.enum";
import {StateEnum} from "@shared/enums/state.enum";
import {SharedPersonAction} from "@shared/stores/person/shared-person.action";
import {ResourceLogoStateModel} from "@shared/stores/resource-logo/resource-logo-state.model";
import {
  defaultResourceLogoStateInitValue,
  ResourceLogoState,
  ResourceLogoStateModeEnum,
} from "@shared/stores/resource-logo/resource-logo.state";
import {Observable} from "rxjs";
import {
  catchError,
  map,
  tap,
} from "rxjs/operators";
import {
  ApiService,
  isNotNullNorUndefined,
  isNullOrUndefined,
  MARK_AS_TRANSLATABLE,
  NotificationService,
  ofSolidifyActionCompleted,
  OverrideDefaultAction,
  Result,
  ResultActionStatusEnum,
  SolidifyHttpErrorResponseModel,
  SolidifyStateError,
  StoreUtil,
} from "solidify-frontend";

export interface AdminUserStateModel extends ResourceLogoStateModel<User> {
}

@Injectable()
@State<AdminUserStateModel>({
  name: StateEnum.admin_user,
  defaults: {
    ...defaultResourceLogoStateInitValue(),
  },
})
export class AdminUserState extends ResourceLogoState<AdminUserStateModel, User> {
  constructor(protected readonly _apiService: ApiService,
              protected readonly _store: Store,
              protected readonly _notificationService: NotificationService,
              protected readonly _actions$: Actions,
              protected readonly _httpClient: HttpClient) {
    super(_apiService, _store, _notificationService, _actions$, {
      nameSpace: adminUserActionNameSpace,
      routeRedirectUrlAfterSuccessCreateAction: (resId: string) => RoutesEnum.adminUserDetail + urlSeparator + resId,
      routeRedirectUrlAfterSuccessUpdateAction: (resId: string) => RoutesEnum.adminUserDetail + urlSeparator + resId,
      routeRedirectUrlAfterSuccessDeleteAction: RoutesEnum.adminUser,
      notificationResourceCreateSuccessTextToTranslate: MARK_AS_TRANSLATABLE("admin.user.notification.resource.create"),
      notificationResourceDeleteSuccessTextToTranslate: MARK_AS_TRANSLATABLE("admin.user.notification.resource.delete"),
      notificationResourceUpdateSuccessTextToTranslate: MARK_AS_TRANSLATABLE("admin.user.notification.resource.update"),
    }, _httpClient, ResourceLogoStateModeEnum.avatar);
  }

  protected get _urlResource(): string {
    return ApiEnum.adminUsers;
  }

  protected get _urlLogoResource(): string {
    return ApiEnum.adminPeople;
  }

  @Selector()
  static isLoading(state: AdminUserStateModel): boolean {
    return StoreUtil.isLoadingState(state);
  }

  @Selector()
  static isLoadingWithDependency(state: AdminUserStateModel): boolean {
    return this.isLoading(state);
  }

  @Selector()
  static currentTitle(state: AdminUserStateModel): string | undefined {
    if (isNullOrUndefined(state.current)) {
      return undefined;
    }
    return state.current.externalUid;
  }

  @Selector()
  static isReadyToBeDisplayed(state: AdminUserStateModel): boolean {
    return this.isReadyToBeDisplayedInCreateMode
      && !isNullOrUndefined(state.current);
  }

  @Selector()
  static isReadyToBeDisplayedInCreateMode(state: AdminUserStateModel): boolean {
    return true;
  }

  @Action(AdminUserAction.LoadResource)
  loadResource(ctx: StateContext<AdminUserStateModel>, action: AdminUserAction.LoadResource): Observable<boolean> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });

    return StoreUtil.dispatchParallelActionAndWaitForSubActionsCompletion(ctx, [
      {
        action: new SharedPersonAction.GetAll(undefined, false, false),
        subActionCompletions: [
          this._actions$.pipe(ofSolidifyActionCompleted(SharedPersonAction.GetAllSuccess)),
          this._actions$.pipe(ofSolidifyActionCompleted(SharedPersonAction.GetAllFail)),
        ],
      },
    ]).pipe(
      map(result => {
        if (result.success) {
          ctx.dispatch(new AdminUserAction.LoadResourceSuccess(action));
        } else {
          ctx.dispatch(new AdminUserAction.LoadResourceFail(action));
        }
        return result.success;
      }),
    );
  }

  @OverrideDefaultAction()
  @Action(AdminUserAction.GetByIdSuccess)
  getByIdSuccess(ctx: StateContext<AdminUserStateModel>, action: AdminUserAction.GetByIdSuccess): void {
    super.getByIdSuccess(ctx, action);

    if (isNotNullNorUndefined(action.model.person?.avatar)) {
      ctx.dispatch(new AdminUserAction.GetPhoto(action.model.person.resId));
    }
  }

  @Action(AdminUserAction.Refresh)
  refresh(ctx: StateContext<AdminUserStateModel>, action: AdminUserAction.Refresh): Observable<Result> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });

    return this._apiService.post<any, Result>(this._urlResource + urlSeparator + ApiActionNameEnum.REFRESH).pipe(
      tap(result => {
        if (result?.status === ResultActionStatusEnum.EXECUTED) {
          ctx.dispatch(new AdminUserAction.RefreshSuccess(action, result));
        } else {
          ctx.dispatch(new AdminUserAction.RefreshFail(action));
        }
      }),
      catchError((error: SolidifyHttpErrorResponseModel) => {
        ctx.dispatch(new AdminUserAction.RefreshFail(action));
        throw new SolidifyStateError(this, error);
      }),
    );
  }

  @Action(AdminUserAction.RefreshSuccess)
  refreshSuccess(ctx: StateContext<AdminUserStateModel>, action: AdminUserAction.RefreshSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this._notificationService.showSuccess(action.result.message);
    ctx.dispatch(new AdminUserAction.GetAll());
  }

  @Action(AdminUserAction.RefreshFail)
  refreshFail(ctx: StateContext<AdminUserStateModel>, action: AdminUserAction.RefreshFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this._notificationService.showError(MARK_AS_TRANSLATABLE("admin.user.notification.refresh.fail"));
  }

  @Action(AdminUserAction.Synchronize)
  synchronize(ctx: StateContext<AdminUserStateModel>, action: AdminUserAction.Synchronize): Observable<Result> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });

    return this._apiService.post<undefined, Result>(this._urlResource + urlSeparator + ApiActionNameEnum.SYNCHRONIZE).pipe(
      tap(result => {
        if (result?.status === ResultActionStatusEnum.EXECUTED) {
          ctx.dispatch(new AdminUserAction.SynchronizeSuccess(action, result));
        } else {
          ctx.dispatch(new AdminUserAction.SynchronizeFail(action));
        }
      }),
      catchError((error: SolidifyHttpErrorResponseModel) => {
        ctx.dispatch(new AdminUserAction.SynchronizeFail(action));
        throw new SolidifyStateError(this, error);
      }),
    );
  }

  @Action(AdminUserAction.SynchronizeSuccess)
  synchronizeSuccess(ctx: StateContext<AdminUserStateModel>, action: AdminUserAction.SynchronizeSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this._notificationService.showSuccess(action.result.message);
    ctx.dispatch(new AdminUserAction.GetAll());
  }

  @Action(AdminUserAction.SynchronizeFail)
  synchronizeFail(ctx: StateContext<AdminUserStateModel>, action: AdminUserAction.SynchronizeFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this._notificationService.showError(MARK_AS_TRANSLATABLE("admin.user.notification.synchronize.fail"));
  }
}
