/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Portal - deposit-aip-upload.dialog.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {
  ChangeDetectionStrategy,
  Component,
  Inject,
  OnInit,
} from "@angular/core";
import {
  FormBuilder,
  FormGroup,
} from "@angular/forms";
import {
  MAT_DIALOG_DATA,
  MatDialogRef,
} from "@angular/material/dialog";
import {DepositAipAction} from "@deposit/stores/aip/deposit-aip.action";
import {DepositAipState} from "@deposit/stores/aip/deposit-aip.state";
import {Enums} from "@enums";
import {ArchiveDataFile} from "@home/models/archive-data-file.model";
import {Aip} from "@models";
import {Store} from "@ngxs/store";
import {SharedAbstractDialog} from "@shared/components/dialogs/shared-abstract/shared-abstract.dialog";
import {DataTableComponentEnum} from "@shared/enums/data-table-component.enum";
import {LabelTranslateEnum} from "@shared/enums/label-translate.enum";
import {AppRoutesEnum} from "@shared/enums/routes.enum";
import {DataTableComponentHelper} from "@shared/helpers/data-table-component.helper";
import {BaseFormDefinition} from "@shared/models/base-form-definition.model";
import {Observable} from "rxjs";
import {map} from "rxjs/operators";
import {
  DataTableColumns,
  DataTableFieldTypeEnum,
  isEmptyArray,
  isNotNullNorUndefinedNorWhiteString,
  isNullOrUndefined,
  MappingObject,
  MappingObjectUtil,
  MemoizedUtil,
  OrderEnum,
  PropertyName,
  QueryParameters,
} from "solidify-frontend";

@Component({
  selector: "dlcm-aip-upload-dialog",
  templateUrl: "./deposit-aip-upload.dialog.html",
  styleUrls: ["./deposit-aip-upload.dialog.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DepositAipUploadDialog extends SharedAbstractDialog<DepositAipUploadDialogData, string[]> implements OnInit {
  form: FormGroup;
  formDefinition: DepositAipUploadFormComponentFormDefinition = new DepositAipUploadFormComponentFormDefinition();

  listAipObs: Observable<Aip[]> = MemoizedUtil.list(this._store, DepositAipState).pipe(
    map(list => {
      const listIdAipAlreadyLinked = this.data.listIdAipAlreadyLinked;
      if (isNullOrUndefined(list) || isEmptyArray(list) || isNullOrUndefined(listIdAipAlreadyLinked) || isEmptyArray(listIdAipAlreadyLinked)) {
        return list;
      }
      return list.filter(aip => !listIdAipAlreadyLinked.includes(aip.resId));
    }),
  );
  queryParametersObs: Observable<QueryParameters> = MemoizedUtil.queryParameters(this._store, DepositAipState);
  isLoadingObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, DepositAipState);

  selectedAips: Aip[] = [];

  get selectedAipsId(): string[] {
    return this.selectedAips.map(a => a.resId);
  }

  searchObs: Observable<string>;

  columns: DataTableColumns<ArchiveDataFile>[] = [
    {
      field: "info.name",
      header: LabelTranslateEnum.title,
      type: DataTableFieldTypeEnum.string,
      order: OrderEnum.ascending,
      isSortable: false,
      isFilterable: false,
    },
    {
      field: "info.access",
      header: LabelTranslateEnum.accessLevel,
      type: DataTableFieldTypeEnum.singleSelect,
      order: OrderEnum.none,
      isSortable: false,
      isFilterable: false,
      translate: true,
      filterEnum: Enums.Deposit.AccessEnumTranslate,
      component: DataTableComponentHelper.get(DataTableComponentEnum.accessLevelWithEmbargo),
      width: "180px",
    },
    {
      field: "info.dataSensitivity",
      header: LabelTranslateEnum.sensitivity,
      type: DataTableFieldTypeEnum.singleSelect,
      order: OrderEnum.none,
      isSortable: false,
      isFilterable: false,
      translate: true,
      filterEnum: Enums.Deposit.DataSensitivityEnumTranslate,
      component: DataTableComponentHelper.get(DataTableComponentEnum.sensitivity),
      width: "60px",
    },
    {
      field: "info.complianceLevel",
      header: LabelTranslateEnum.complianceLevel,
      type: DataTableFieldTypeEnum.singleSelect,
      order: OrderEnum.none,
      sortableField: "dataFile.complianceLevel" as any,
      filterableField: "dataFile.complianceLevel" as any,
      isSortable: false,
      isFilterable: false,
      translate: true,
      width: "185px",
      component: DataTableComponentHelper.get(DataTableComponentEnum.conformityLevelStar),
      filterEnum: Enums.DataFile.ComplianceLevelEnumTranslate,
      alignment: "center",
    },
    {
      field: "smartSize",
      header: LabelTranslateEnum.size,
      type: DataTableFieldTypeEnum.string,
      order: OrderEnum.none,
      sortableField: "dataFile.fileSize" as any,
      filterableField: "dataFile.fileSize" as any,
      isSortable: false,
      isFilterable: false,
      translate: true,
      alignment: "right",
      width: "100px",
    },
  ];

  constructor(protected readonly _store: Store,
              protected readonly _dialogRef: MatDialogRef<DepositAipUploadDialog>,
              @Inject(MAT_DIALOG_DATA) public readonly data: DepositAipUploadDialogData,
              protected readonly _fb: FormBuilder) {
    super(_dialogRef, data);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.form = this._fb.group({
      [this.formDefinition.search]: [""],
    });

    this._store.dispatch(new DepositAipAction.Clean(false, undefined));
    this.search();
  }

  search(searchTerm: string = undefined): void {
    const searchItems: MappingObject<string> = {};
    if (isNotNullNorUndefinedNorWhiteString(searchTerm)) {
      MappingObjectUtil.set(searchItems, "info.name", searchTerm);
    }
    const orgUnitId = this._store.selectSnapshot(state => state.router.state.root.children[0].children[0].params[AppRoutesEnum.paramIdOrgUnitWithoutPrefixParam]);
    MappingObjectUtil.set(searchItems, "info.organizationalUnitId", orgUnitId);
    const newQueryParameters = new QueryParameters();
    newQueryParameters.search = {
      searchItems: searchItems,
    };
    this._store.dispatch(new DepositAipAction.ChangeQueryParameters(newQueryParameters, true));
  }

  onQueryParametersEvent(queryParameters: QueryParameters): void {
    this._store.dispatch(new DepositAipAction.ChangeQueryParameters(queryParameters));
  }

  onSubmit(): void {
    this.submit(this.selectedAipsId);
  }

  selectionChange(aip: Aip): void {
    const indexOf = this.selectedAips.findIndex(a => a.resId === aip.resId);
    if (indexOf !== -1) {
      this.removeAip(indexOf);
    } else {
      this.selectedAips.push(aip);
    }
  }

  removeAip(index: number): void {
    this.selectedAips.splice(index, 1);
  }
}

export class DepositAipUploadFormComponentFormDefinition extends BaseFormDefinition {
  @PropertyName() search: string;
}

export interface DepositAipUploadDialogData {
  listIdAipAlreadyLinked: string[];
}
