/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Portal - deposit-file-change-data-category.dialog.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {
  ChangeDetectionStrategy,
  Component,
  Inject,
  OnInit,
} from "@angular/core";
import {
  AbstractControl,
  FormBuilder,
  FormGroup,
  Validators,
} from "@angular/forms";
import {
  MAT_DIALOG_DATA,
  MatDialogRef,
} from "@angular/material/dialog";
import {DepositDataFile} from "@deposit/models/deposit-data-file.model";
import {DepositDataFileAction} from "@deposit/stores/data-file/deposit-data-file.action";
import {DepositDataFileState} from "@deposit/stores/data-file/deposit-data-file.state";
import {depositDataFileMetadataTypeActionNameSpace} from "@deposit/stores/data-file/metadata-type/deposit-data-file-metadata-type.action";
import {DepositDataFileMetadataTypeState} from "@deposit/stores/data-file/metadata-type/deposit-data-file-metadata-type.state";
import {Enums} from "@enums";
import {MetadataType} from "@models";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {SharedAbstractDialog} from "@shared/components/dialogs/shared-abstract/shared-abstract.dialog";
import {BaseFormDefinition} from "@shared/models/base-form-definition.model";
import {Observable} from "rxjs";
import {tap} from "rxjs/operators";
import {
  EnumUtil,
  FormValidationHelper,
  isNonEmptyString,
  isNullOrUndefined,
  MemoizedUtil,
  PropertyName,
  ResourceNameSpace,
  SolidifyValidator,
  StoreUtil,
} from "solidify-frontend";

@Component({
  selector: "dlcm-deposit-file-change-data-category-dialog",
  templateUrl: "./deposit-file-change-data-category.dialog.html",
  styleUrls: ["./deposit-file-change-data-category.dialog.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DepositFileChangeDataCategoryDialog extends SharedAbstractDialog<DepositFileChangeDataCategoryDialogData, boolean> implements OnInit {
  isLoadingObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, DepositDataFileState);

  form: FormGroup;
  formDefinition: FormComponentFormDefinition = new FormComponentFormDefinition();

  metadataTypeLabelCallback: (value: MetadataType) => string = value => value.fullName;

  depositDataFileMetadataTypeActionNameSpace: ResourceNameSpace = depositDataFileMetadataTypeActionNameSpace;
  depositDataFileMetadataTypeState: typeof DepositDataFileMetadataTypeState = DepositDataFileMetadataTypeState;

  get enumsDataFile(): typeof Enums.DataFile {
    return Enums.DataFile;
  }

  get enumUtil(): typeof EnumUtil {
    return EnumUtil;
  }

  isFormValid: boolean = true;

  get dataTypeEnum(): typeof Enums.DataFile.DataTypeEnum {
    return Enums.DataFile.DataTypeEnum;
  }

  get formValidationHelper(): typeof FormValidationHelper {
    return FormValidationHelper;
  }

  constructor(private readonly _store: Store,
              private readonly _actions$: Actions,
              protected readonly _dialogRef: MatDialogRef<DepositFileChangeDataCategoryDialog>,
              @Inject(MAT_DIALOG_DATA) public readonly data: DepositFileChangeDataCategoryDialogData,
              private readonly _fb: FormBuilder) {
    super(_dialogRef, data);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.form = this._fb.group({
      [this.formDefinition.dataCategory]: [this.data.depositDataFile.dataCategory, [Validators.required, SolidifyValidator]],
      [this.formDefinition.dataType]: [this.data.depositDataFile.dataType, [Validators.required, SolidifyValidator]],
      [this.formDefinition.metadataType]: [isNullOrUndefined(this.data.depositDataFile.metadataType) ? "" : this.data.depositDataFile.metadataType.resId],
    });

    this.subscribe(this.form.get(this.formDefinition.dataType).valueChanges.pipe(
      tap(value => {
        const metadataFormControl = this.form.get(this.formDefinition.metadataType);
        if (value === Enums.DataFile.DataTypeEnum.CustomMetadata) {
          metadataFormControl.setValidators([Validators.required]);
        } else {
          metadataFormControl.setValidators([]);
        }
        metadataFormControl.updateValueAndValidity();
      }),
    ));
  }

  onSubmit(): void {
    let metadata: MetadataType = null;
    if (!isNullOrUndefined(this.form.get(this.formDefinition.metadataType).value) && isNonEmptyString(this.form.get(this.formDefinition.metadataType).value)) {
      metadata = {
        resId: this.form.get(this.formDefinition.metadataType).value,
      } as MetadataType;
    }

    const action = new DepositDataFileAction.ChangeDataCategory(this.data.parentResId, {
      resId: this.data.depositDataFile.resId,
      dataCategory: this.form.get(this.formDefinition.dataCategory).value,
      dataType: this.form.get(this.formDefinition.dataType).value,
      metadataType: !isNullOrUndefined(metadata) ? metadata : undefined,
    });

    this.subscribe(StoreUtil.dispatchActionAndWaitForSubActionCompletion(this._store, this._actions$,
      action,
      DepositDataFileAction.ChangeDataCategorySuccess,
      resultAction => {
        this.submit(true);
      }));
  }

  getFormControl(key: string): AbstractControl {
    return FormValidationHelper.getFormControl(this.form, key);
  }

  getDataCategory(): Enums.DataFile.DataCategoryEnum {
    return this.form.get(this.formDefinition.dataCategory).value;
  }

  selectCategory(): void {
    this.isFormValid = false;
  }

  selectDataType(): void {
    this.isFormValid = true;
  }
}

class FormComponentFormDefinition extends BaseFormDefinition {
  @PropertyName() dataCategory: string;
  @PropertyName() dataType: string;
  @PropertyName() metadataType: string;
}

export interface DepositFileChangeDataCategoryDialogData {
  parentResId: string;
  depositDataFile: DepositDataFile;
}
