/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Portal - home-archive-detail.presentational.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  Input,
  OnChanges,
  OnInit,
  Output,
} from "@angular/core";
import {FormBuilder} from "@angular/forms";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {ArchiveMetadata} from "@app/shared/models/business/archive-metadata.model";
import {MetadataUtil} from "@app/shared/utils/metadata.util";
import {Enums} from "@enums";
import {ArchiveStatisticsDto} from "@home/models/archive-statistics-dto.model";
import {
  Archive,
  ArchiveContributor,
} from "@home/models/archive.model";
import {ArchiveAccessRightService} from "@home/services/archive-access-right.service";
import {
  ArchiveUserRating,
  Person,
} from "@models";
import {Navigate} from "@ngxs/router-plugin";
import {GetShortDoiWrapper} from "@shared/components/presentationals/shared-doi-menu/shared-doi-menu.presentational";
import {ImageDisplayMode} from "@shared/components/presentationals/shared-image-display/shared-image-display.presentational";
import {SharedPersonOverlayPresentational} from "@shared/components/presentationals/shared-person-overlay/shared-person-overlay.presentational";
import {ApiEnum} from "@shared/enums/api.enum";
import {LabelTranslateEnum} from "@shared/enums/label-translate.enum";
import {
  HomePageRoutesEnum,
  RoutesEnum,
} from "@shared/enums/routes.enum";
import {ViewModeTableEnum} from "@shared/enums/view-mode-table.enum";
import {
  ArchiveMetadataPackages,
  MetadataPackagesEnum,
} from "@shared/models/business/archive-metadata-packages.model";
import {KeyValueInfo} from "@shared/models/key-value-info.model";
import {SecurityService} from "@shared/services/security.service";
import {
  BehaviorSubject,
  Observable,
} from "rxjs";
import {
  Citation,
  ClipboardUtil,
  DateUtil,
  EnumUtil,
  isNonEmptyString,
  isNullOrUndefined,
  NotificationService,
  ObservableUtil,
  RegexUtil,
  Type,
} from "solidify-frontend";
import {ApiActionNameEnum} from "../../../../../shared/enums/api-action-name.enum";

@Component({
  selector: "dlcm-home-archive-detail",
  templateUrl: "./home-archive-detail.presentational.html",
  styleUrls: ["./home-archive-detail.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class HomeArchiveDetailPresentational extends SharedAbstractPresentational implements OnInit, OnChanges {
  private readonly _downloadBS: BehaviorSubject<Archive | undefined> = new BehaviorSubject<Archive | undefined>(undefined);
  @Output("downloadChange")
  readonly downloadObs: Observable<Archive | undefined> = ObservableUtil.asObservable(this._downloadBS);

  private readonly _showArchiveBS: BehaviorSubject<Archive | undefined> = new BehaviorSubject<Archive | undefined>(undefined);
  @Output("showArchiveChange")
  readonly showArchiveObs: Observable<Archive | undefined> = ObservableUtil.asObservable(this._showArchiveBS);

  private readonly _addToCartBS: BehaviorSubject<Archive | undefined> = new BehaviorSubject<Archive | undefined>(undefined);
  @Output("addToCartChange")
  readonly addToCartObs: Observable<Archive | undefined> = ObservableUtil.asObservable(this._addToCartBS);

  private readonly _askAccessBS: BehaviorSubject<Archive | undefined> = new BehaviorSubject<Archive | undefined>(undefined);
  @Output("askAccessChange")
  readonly askAccessObs: Observable<Archive | undefined> = ObservableUtil.asObservable(this._askAccessBS);

  private readonly _navigateBS: BehaviorSubject<Navigate> = new BehaviorSubject<Navigate>(undefined);
  @Output("navigateChange")
  readonly navigateObs: Observable<Navigate> = ObservableUtil.asObservable(this._navigateBS);

  private readonly _rateBS: BehaviorSubject<ArchiveUserRating[]> = new BehaviorSubject<ArchiveUserRating[]>(undefined);
  @Output("rateChange")
  readonly rateObs: Observable<ArchiveUserRating[]> = ObservableUtil.asObservable(this._rateBS);

  @Input()
  archive: Archive;

  @Input()
  archiveStatistics: ArchiveStatisticsDto;

  @Input()
  archivePackages: ArchiveMetadataPackages;

  @Input()
  listArchiveUserRating: ArchiveUserRating[];

  @Input()
  relativeArchive: Archive[];

  @Input()
  listCitations: Citation[];

  @Input()
  listBibliographies: Citation[];

  @Input()
  isLoadingPrepareDownload: boolean;

  @Input()
  isLoggedIn: boolean;

  @Input()
  isLoadingArchiveRating: boolean;

  get metadata(): ArchiveMetadata.PublicMetadata | undefined {
    if (isNullOrUndefined(this.archive) || isNullOrUndefined(this.archive.archiveMetadata)) {
      return undefined;
    }
    return this.archive.archiveMetadata.metadata;
  }

  listInfo: Info[];

  imageArchive: string | undefined;

  viewModeList: ViewModeTableEnum = ViewModeTableEnum.tiles;

  personOverlayComponent: Type<SharedPersonOverlayPresentational> = SharedPersonOverlayPresentational;

  get viewModeListEnum(): typeof ViewModeTableEnum {
    return ViewModeTableEnum;
  }

  private readonly _shortDoiBS: BehaviorSubject<GetShortDoiWrapper | undefined> = new BehaviorSubject<GetShortDoiWrapper | undefined>(undefined);
  @Output("getShortDoi")
  readonly shortDoiObs: Observable<GetShortDoiWrapper | undefined> = ObservableUtil.asObservable(this._shortDoiBS);

  get typeInfoEnum(): typeof TypeInfoEnum {
    return TypeInfoEnum;
  }

  get enumUtil(): typeof EnumUtil {
    return EnumUtil;
  }

  get imageDisplayMode(): typeof ImageDisplayMode {
    return ImageDisplayMode;
  }

  constructor(protected readonly _securityService: SecurityService,
              protected readonly _notificationService: NotificationService,
              readonly archiveAccessRightService: ArchiveAccessRightService,
              private readonly _fb: FormBuilder,
              private readonly _cd: ChangeDetectorRef) {
    super();
  }

  ngOnInit(): void {
    this.imageArchive = `${ApiEnum.accessPublicMetadata}/${this.archive.resId}/${ApiActionNameEnum.THUMBNAIL}`;
    this._defineListInfo();
  }

  private _defineListInfo(): void {
    this.listInfo = [
      {
        labelToTranslate: LabelTranslateEnum.organizationalUnit,
        value: this.archive.organizationalUnitId,
        type: TypeInfoEnum.orgUnit,
      },
      {
        labelToTranslate: LabelTranslateEnum.type,
        value: MetadataUtil.getType(this.metadata) === "DATASET" ? LabelTranslateEnum.dataset : MetadataUtil.getType(this.metadata),
        type: TypeInfoEnum.translate,
      },
      {
        labelToTranslate: LabelTranslateEnum.doi,
        value: MetadataUtil.getDOI(this.metadata),
        type: TypeInfoEnum.doi,
      },
      {
        labelToTranslate: LabelTranslateEnum.embargoAccessLevel,
        value: EnumUtil.getLabel(Enums.Deposit.AccessEnumTranslate, MetadataUtil.getEmbargoAccessLevel(this.metadata)),
        type: TypeInfoEnum.translate,
      },
      {
        labelToTranslate: LabelTranslateEnum.embargoEndDate,
        value: DateUtil.convertDateToDateString(MetadataUtil.getEmbargoEndDate(this.metadata)),
      },
      {
        labelToTranslate: LabelTranslateEnum.submittedDate,
        value: DateUtil.convertDateToDateTimeString(MetadataUtil.getSubmittedDate(this.metadata)),
      },
      {
        labelToTranslate: LabelTranslateEnum.descriptionOther,
        value: MetadataUtil.getDescriptionOther(this.metadata),
      },
      {
        labelToTranslate: LabelTranslateEnum.descriptionTechnicalInfo,
        value: MetadataUtil.getDescriptionTechnicalInfo(this.metadata),
      },
      {
        labelToTranslate: LabelTranslateEnum.descriptionSeriesInformation,
        value: MetadataUtil.getDescriptionSeriesInformation(this.metadata),
      },
      {
        labelToTranslate: LabelTranslateEnum.descriptionTableOfContents,
        value: MetadataUtil.getDescriptionTableOfContents(this.metadata),
      },
      {
        labelToTranslate: LabelTranslateEnum.descriptionMethods,
        value: MetadataUtil.getDescriptionMethods(this.metadata),
      },
    ];
    this.listInfo = this.listInfo.filter(i => isNonEmptyString(i.value));
  }

  download(archive: Archive): void {
    this._downloadBS.next(archive);
  }

  getMetadataDatacite(): string {
    return MetadataUtil.getDatacite(this.metadata);
  }

  getShortDoi(longDoi: GetShortDoiWrapper): void {
    this._shortDoiBS.next(longDoi);
  }

  addToCart(archive: Archive): void {
    this._addToCartBS.next(archive);
  }

  askAccess(archive: Archive): void {
    this._askAccessBS.next(archive);
  }

  showDetail(archive: Archive): void {
    this._showArchiveBS.next(archive);
  }

  navigateToFilesList(): void {
    if (this.archive.isCollection) {
      this._navigateBS.next(new Navigate([RoutesEnum.homeDetail, this.archive.resId, HomePageRoutesEnum.collections]));
    } else {
      this._navigateBS.next(new Navigate([RoutesEnum.homeDetail, this.archive.resId, HomePageRoutesEnum.files]));
    }
  }

  getMetadataVersion(archivePackages: ArchiveMetadataPackages): string {
    return archivePackages.metadata[MetadataPackagesEnum.metadataVersion];
  }

  getRetentionExpiration(archivePackages: ArchiveMetadataPackages): Date {
    const retentionEnd = archivePackages.metadata[MetadataPackagesEnum.aipRetentionEnd];
    return DateUtil.convertOffsetDateTimeIso8601ToDate(retentionEnd);
  }

  getDisposalApproval(archivePackages: ArchiveMetadataPackages): string {
    return archivePackages.metadata[MetadataPackagesEnum.aipDispositionApproval] ? LabelTranslateEnum.yes : LabelTranslateEnum.no;
  }

  getCreationDate(archivePackages: ArchiveMetadataPackages): Date {
    const creation = archivePackages.metadata[MetadataPackagesEnum.creation];
    return DateUtil.convertOffsetDateTimeIso8601ToDate(creation);
  }

  copyToClipboard(id: string): void {
    ClipboardUtil.copyStringToClipboard(id);
    this._notificationService.showInformation(LabelTranslateEnum.idCopyToClipboard);
  }

  getDepositCreationDate(archivePackages: ArchiveMetadataPackages): Date {
    const creation = archivePackages.metadata.packages.deposits.events.creation?.date;
    return DateUtil.convertOffsetDateTimeIso8601ToDate(creation);
  }

  getDepositApprobationDate(archivePackages: ArchiveMetadataPackages): Date {
    const approval = archivePackages.metadata.packages.deposits.events?.appraisal?.date;
    return DateUtil.convertOffsetDateTimeIso8601ToDate(approval);
  }

  getSensitivityTooltip(dataSensitivity: Enums.Deposit.DataSensitivityEnum): string {
    return (EnumUtil.getKeyValue(Enums.Deposit.DataSensitivityEnumTranslate, dataSensitivity) as KeyValueInfo)?.infoToTranslate;
  }

  computeLink(text: string): string {
    if (isNullOrUndefined(text)) {
      return text;
    }
    return text.replace(RegexUtil.urlInComplexText, match => `<a href="${match}" target="_blank" class="no-hover-animation">${match}</a>`);
  }

  updateRate($event: ArchiveUserRating[]): void {
    this._rateBS.next($event);
  }

  readonly CONTRIBUTOR_SEPARATOR: string = ", ";

  getContributorOverlayData(contributor: ArchiveContributor): Person {
    const fullName = contributor.creatorName;
    const indexOfSpace = fullName.indexOf(this.CONTRIBUTOR_SEPARATOR);
    const lastName = indexOfSpace === -1 ? fullName : fullName.substring(0, indexOfSpace);
    const firstName = indexOfSpace === -1 ? "" : fullName.substring(indexOfSpace + this.CONTRIBUTOR_SEPARATOR.length);
    return {
      firstName: firstName,
      lastName: lastName,
      orcid: contributor.orcid,
      institutions: contributor.affiliation.map(a => ({name: a})),
    };
  }
}

class Info {
  labelToTranslate: string;
  value?: string;
  code?: boolean = false;
  type?: TypeInfoEnum;
}

enum TypeInfoEnum {
  doi = 1,
  orgUnit = 2,
  translate = 3,
}
