/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Portal - home-twitter-widget.presentational.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {
  AfterViewInit,
  ChangeDetectionStrategy,
  Component,
  ElementRef,
  Inject,
  Input,
  Renderer2,
} from "@angular/core";
import {WINDOW} from "@app/app.module";
import {TwitterChromeEnum} from "@app/features/home/enums/twitter-chrome.enum";
import {TwitterSourceTypeEnum} from "@app/features/home/enums/twitter-source-type.enum";
import {TwitterMetadata} from "@app/features/home/models/twitter-metadata.model";
import {
  DataSource,
  Options,
} from "@app/features/home/models/twitter-widget.model";
import {TwitterService} from "@app/features/home/services/twitter.service";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {AppState} from "@app/stores/app.state";
import {Store} from "@ngxs/store";
import {
  catchError,
  distinctUntilChanged,
  take,
  tap,
} from "rxjs/operators";
import {
  LoggingService,
  MemoizedUtil,
} from "solidify-frontend";

@Component({
  selector: "dlcm-home-twitter-widget-presentational",
  templateUrl: "./home-twitter-widget.presentational.html",
  styleUrls: ["./home-twitter-widget.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class HomeTwitterWidgetPresentational extends SharedAbstractPresentational implements AfterViewInit {
  @Input()
  sourceType: TwitterSourceTypeEnum | undefined;

  @Input()
  screenName: string | undefined;

  @Input()
  userId: string | undefined;

  @Input()
  tweetLimit: number | undefined;

  @Input()
  borderColor: string | undefined;

  @Input()
  ariaPolite: string | undefined;

  @Input()
  height: number | undefined;

  @Input()
  chrome: TwitterChromeEnum[] = [TwitterChromeEnum.noborders, TwitterChromeEnum.transparent];

  constructor(private readonly _renderer: Renderer2,
              private readonly _element: ElementRef,
              private readonly _twitterService: TwitterService,
              private readonly _loggingService: LoggingService,
              private readonly _store: Store,
              @Inject(WINDOW) private readonly _window: Window) {
    super();
  }

  ngAfterViewInit(): void {
    // https://developer.twitter.com/en/docs/twitter-for-websites/javascript-api/guides/scripting-factory-functions
    this.subscribe(this._twitterService.loadScript(this._renderer)
      .pipe(
        take(1),
        tap((twitterMetadata: TwitterMetadata) => {
          const nativeElement: Element = this._element.nativeElement;
          const dataSource: DataSource = {
            sourceType: this.sourceType,
            screenName: this.screenName,
            userId: this.userId,
          };
          const options: Options = {
            theme: MemoizedUtil.selectSnapshot(this._store, AppState, state => state.darkMode) ? "dark" : undefined,
            tweetLimit: this.tweetLimit,
            height: this.height,
            chrome: this.chrome,
            ariaPolite: this.ariaPolite,
            borderColor: this.borderColor,
          };

          this.subscribe(MemoizedUtil.select(this._store, AppState, state => state.darkMode).pipe(
            distinctUntilChanged(),
            tap(enabled => {
              const list: NodeList = this._element.nativeElement.querySelectorAll("iframe");
              list.forEach(node => {
                this._element.nativeElement.removeChild(node);
              });
              options.theme = enabled ? "dark" : undefined;
              twitterMetadata.widgets.createTimeline(dataSource, nativeElement, options)
                .then((iframe: any) => {})
                .catch((message: string) => {
                  this._loggingService.logWarning("Twitter : Could not create widget", message);
                });
            })));
        }),
        catchError((err: any) => {
          this._loggingService.logWarning("Twitter : Error loading twitter widget", err);
          throw err;
        }),
      ));
  }
}
