/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Portal - home.helper.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {
  NavigationExtras,
  Params,
} from "@angular/router";
import {Enums} from "@enums";
import {environment} from "@environments/environment";
import {SearchFacet} from "@models";
import {Navigate} from "@ngxs/router-plugin";
import {Store} from "@ngxs/store";
import {RoutesEnum} from "@shared/enums/routes.enum";
import {ViewModeTableEnum} from "@shared/enums/view-mode-table.enum";
import {Observable} from "rxjs";
import {
  isEmptyArray,
  isNonEmptyString,
  isNotNullNorUndefined,
  isNullOrUndefined,
  MappingObject,
  MappingObjectUtil,
  StringUtil,
} from "solidify-frontend";

export class HomeHelper {
  static SEARCH_QUERY_PARAM: string = "search";
  static VIEW_QUERY_PARAM: string = "view";
  static VALUES_SEPARATOR: string = ";";
  static QUERY_PARAM_SEPARATOR: string = "&";
  static QUERY_PARAM_VALUE_AFFECTION: string = "=";

  static LIST_FACET_USE_OPERATOR_AND: Enums.Facet.Name[] = environment.listFacetUseOperatorAnd;

  static getDefaultSearchQueryParams(): any {
    return {[this.SEARCH_QUERY_PARAM]: "*"};
  }

  static navigateToSearch(store: Store, searchTerm?: string | undefined,
                          facetsSelected?: MappingObject<Enums.Facet.Name, string[]> | undefined,
                          viewModeTable?: ViewModeTableEnum | undefined): Observable<any> {
    let queryParams = {} as MappingObject<string, string>;

    if (isNonEmptyString(searchTerm)) {
      queryParams = {[this.SEARCH_QUERY_PARAM]: encodeURIComponent(searchTerm)} as MappingObject<string, string>;
    }
    if (isNotNullNorUndefined(facetsSelected)) {
      MappingObjectUtil.forEach(facetsSelected, ((value, key) => {
        const valueString = value.map(v => encodeURIComponent(v)).join(this.VALUES_SEPARATOR);
        MappingObjectUtil.set(queryParams, key as string, valueString);
      }));
    }

    let completeSearchString = undefined;
    MappingObjectUtil.forEach(queryParams, ((value: string, key: string) => {
      if (isNotNullNorUndefined(completeSearchString)) {
        completeSearchString = completeSearchString + this.QUERY_PARAM_SEPARATOR;
      } else {
        completeSearchString = StringUtil.stringEmpty;
      }
      completeSearchString = completeSearchString + key + this.QUERY_PARAM_VALUE_AFFECTION + value;
    }));

    const navigationExtras: NavigationExtras = {
      [this.VIEW_QUERY_PARAM]: viewModeTable,
    };
    if (isNotNullNorUndefined(completeSearchString)) {
      navigationExtras[this.SEARCH_QUERY_PARAM] = completeSearchString;
    }
    return store.dispatch(new Navigate([RoutesEnum.homeSearch], navigationExtras));
  }

  static extractSearchInfosFromUrl(params: Params): { search: string, facetsSelected: MappingObject<Enums.Facet.Name, string[]>, viewTabMode: ViewModeTableEnum } {
    let searchEncoded = undefined;

    const viewTabMode = params[this.VIEW_QUERY_PARAM];
    const queryParamSearch = params[this.SEARCH_QUERY_PARAM];
    if (!isNullOrUndefined(queryParamSearch)) {
      searchEncoded = queryParamSearch;
    }

    let searchString = StringUtil.stringEmpty;
    const facetsSelected = {} as MappingObject<Enums.Facet.Name, string[]>;
    if (isNotNullNorUndefined(searchEncoded)) {
      const listParams = searchEncoded.split(this.QUERY_PARAM_SEPARATOR);
      listParams.forEach(p => {
        const splitParam = p.split(this.QUERY_PARAM_VALUE_AFFECTION);
        const decodedValue = decodeURIComponent(splitParam[1]);
        if (splitParam[0] === this.SEARCH_QUERY_PARAM) {
          searchString = decodedValue;
        } else {
          if (splitParam.length > 1) {
            MappingObjectUtil.set(facetsSelected, splitParam[0], decodedValue.split(this.VALUES_SEPARATOR));
          }
        }
      });
    }

    return {
      search: searchString,
      facetsSelected: facetsSelected,
      viewTabMode: viewTabMode,
    };
  }

  static getFacetsSelectedForBackend(facetsSelected: MappingObject<Enums.Facet.Name, string[]>): SearchFacet[] {
    const facetsSelectedForBackend: SearchFacet[] = [];
    MappingObjectUtil.forEach(facetsSelected, (value, key) => {
      if (isNullOrUndefined(value) || isEmptyArray(value)) {
        return;
      }
      if (this.LIST_FACET_USE_OPERATOR_AND.indexOf(key as Enums.Facet.Name) === -1) {
        // FOR OR, equal value =  [ "orgUnitName1", "orgUnitName2" ]
        const facet = {
          field: key,
          type: undefined,
        } as SearchFacet;
        if (value.length === 1) {
          facet.equalValue = value[0];
        } else {
          facet.equalValues = value;
        }
        facetsSelectedForBackend.push(facet);
      } else {
        // FOR AND, equal value = "orgUnitName1", need to create a second search facet with "orgUnitName2"
        value.forEach(v => {
          facetsSelectedForBackend.push({
            field: key,
            type: undefined,
            equalValue: v,
          });
        });
      }
    });
    return facetsSelectedForBackend;
  }
}
