/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Portal - preservation-space-notification.state.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {Injectable} from "@angular/core";
import {NotificationModeEnum} from "@app/features/preservation-space/notification/enums/notification-mode.enum";
import {
  PreservationSpaceNotificationAction,
  preservationSpaceNotificationActionNameSpace,
} from "@app/features/preservation-space/notification/stores/preservation-space-notification.action";
import {
  PreservationSpaceNotificationStatusHistoryState,
  PreservationSpaceNotificationStatusHistoryStateModel,
} from "@app/features/preservation-space/notification/stores/status-history/preservation-space-notification-status-history.state";
import {appActionNameSpace} from "@app/stores/app.action";
import {Enums} from "@enums";
import {NotificationDlcm} from "@models";
import {Navigate} from "@ngxs/router-plugin";
import {
  Action,
  Actions,
  Selector,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {ApiActionNameEnum} from "@shared/enums/api-action-name.enum";
import {ApiEnum} from "@shared/enums/api.enum";
import {LocalStorageEnum} from "@shared/enums/local-storage.enum";
import {
  RoutesEnum,
  urlSeparator,
} from "@shared/enums/routes.enum";
import {StateEnum} from "@shared/enums/state.enum";
import {ViewModeEnum} from "@shared/enums/view-mode.enum";
import {SharedRoleAction} from "@shared/stores/role/shared-role.action";
import {defaultStatusHistoryInitValue} from "@shared/stores/status-history/status-history.state";
import {
  Observable,
  pipe,
} from "rxjs";
import {
  catchError,
  map,
  tap,
} from "rxjs/operators";
import {
  ApiService,
  CollectionTyped,
  CookieConsentUtil,
  CookieType,
  defaultResourceStateInitValue,
  isNullOrUndefined,
  LocalStorageHelper,
  MappingObjectUtil,
  MARK_AS_TRANSLATABLE,
  NotificationService,
  ofSolidifyActionCompleted,
  OverrideDefaultAction,
  QueryParametersUtil,
  ResourceState,
  ResourceStateModel,
  Result,
  SolidifyHttpErrorResponseModel,
  SolidifyStateError,
  StoreUtil,
} from "solidify-frontend";

export interface PreservationSpaceNotificationStateModel extends ResourceStateModel<NotificationDlcm> {
  mode: NotificationModeEnum;
  orgUnitId: string | undefined;
  preservationSpace_notification_statusHistory: PreservationSpaceNotificationStatusHistoryStateModel;
}

@Injectable()
@State<PreservationSpaceNotificationStateModel>({
  name: StateEnum.preservationSpace_notification,
  defaults: {
    ...defaultResourceStateInitValue(),
    mode: NotificationModeEnum.inbox,
    orgUnitId: undefined,
    preservationSpace_notification_statusHistory: {...defaultStatusHistoryInitValue()},
  },
  children: [
    PreservationSpaceNotificationStatusHistoryState,
  ],
})
export class PreservationSpaceNotificationState extends ResourceState<PreservationSpaceNotificationStateModel, NotificationDlcm> {
  constructor(protected readonly _apiService: ApiService,
              protected readonly _store: Store,
              protected readonly _notificationService: NotificationService,
              protected readonly _actions$: Actions) {
    super(_apiService, _store, _notificationService, _actions$, {
      nameSpace: preservationSpaceNotificationActionNameSpace,
      routeRedirectUrlAfterSuccessCreateAction: (resId: string) => RoutesEnum.preservationSpaceNotificationInboxDetail + urlSeparator + resId,
      routeRedirectUrlAfterSuccessUpdateAction: (resId: string) => RoutesEnum.preservationSpaceNotificationInboxDetail + urlSeparator + resId,
      notificationResourceCreateSuccessTextToTranslate: MARK_AS_TRANSLATABLE("preservationSpace.notifications.notification.resource.create"),
      notificationResourceDeleteSuccessTextToTranslate: MARK_AS_TRANSLATABLE("preservationSpace.notifications.notification.resource.delete"),
      notificationResourceUpdateSuccessTextToTranslate: MARK_AS_TRANSLATABLE("preservationSpace.notifications.notification.resource.update"),
    });
  }

  protected get _urlResource(): string {
    return ApiEnum.adminNotifications;
  }

  @Selector()
  static isLoading(state: PreservationSpaceNotificationStateModel): boolean {
    return StoreUtil.isLoadingState(state);
  }

  @Selector()
  static isLoadingWithDependency(state: PreservationSpaceNotificationStateModel): boolean {
    return this.isLoading(state);
  }

  @Selector()
  static currentTitle(state: PreservationSpaceNotificationStateModel): string | undefined {
    if (isNullOrUndefined(state.current)) {
      return undefined;
    }
    return state.current.emitter["person"].fullName;
  }

  @Selector()
  static isReadyToBeDisplayed(state: PreservationSpaceNotificationStateModel): boolean {
    return this.isReadyToBeDisplayedInCreateMode
      && !isNullOrUndefined(state.current);
  }

  @Selector()
  static isReadyToBeDisplayedInCreateMode(state: PreservationSpaceNotificationStateModel): boolean {
    return true;
  }

  @OverrideDefaultAction()
  @Action(PreservationSpaceNotificationAction.LoadResource)
  loadResource(ctx: StateContext<PreservationSpaceNotificationStateModel>, action: PreservationSpaceNotificationAction.LoadResource): Observable<boolean> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });

    return StoreUtil.dispatchParallelActionAndWaitForSubActionsCompletion(ctx, [
      {
        action: new SharedRoleAction.GetAll(undefined, false, false),
        subActionCompletions: [
          this._actions$.pipe(ofSolidifyActionCompleted(SharedRoleAction.GetAllSuccess)),
          this._actions$.pipe(ofSolidifyActionCompleted(SharedRoleAction.GetAllFail)),
        ],
      },
    ]).pipe(
      map(result => {
        if (result.success) {
          ctx.dispatch(new PreservationSpaceNotificationAction.LoadResourceSuccess(action));
        } else {
          ctx.dispatch(new PreservationSpaceNotificationAction.LoadResourceFail(action));
        }
        return result.success;
      }),
    );
  }

  @Action(PreservationSpaceNotificationAction.SetMode)
  setMode(ctx: StateContext<PreservationSpaceNotificationStateModel>, action: PreservationSpaceNotificationAction.SetMode): void {
    ctx.patchState({
      mode: action.mode,
      orgUnitId: action.orgUnitId,
    });
  }

  @OverrideDefaultAction()
  @Action(PreservationSpaceNotificationAction.GetAll)
  getAll(ctx: StateContext<PreservationSpaceNotificationStateModel>, action: PreservationSpaceNotificationAction.GetAll): Observable<CollectionTyped<NotificationDlcm>> {
    let reset = {};
    if (!action.keepCurrentContext) {
      reset = {
        list: undefined,
        total: 0,
      };
    }
    let queryParameters = StoreUtil.getQueryParametersToApply(action.queryParameters, ctx);
    const orgUnitId = ctx.getState().orgUnitId;
    if (!isNullOrUndefined(orgUnitId)) {
      queryParameters = QueryParametersUtil.clone(queryParameters);
      const search = QueryParametersUtil.getSearchItems(queryParameters);
      MappingObjectUtil.set(search, "notifiedOrgUnit.resId", orgUnitId);
    }
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
      queryParameters: queryParameters,
      ...reset,
    });
    let mode = ApiActionNameEnum.INBOX;
    if (ctx.getState().mode === NotificationModeEnum.sent) {
      mode = ApiActionNameEnum.SENT;
    }
    return this._apiService.get<NotificationDlcm>(this._urlResource + urlSeparator + mode, ctx.getState().queryParameters)
      .pipe(
        action.cancelIncomplete ? StoreUtil.cancelUncompleted(ctx, this._actions$, [this._nameSpace.GetAll, Navigate]) : pipe(),
        tap((collection: CollectionTyped<NotificationDlcm>) => {
          ctx.dispatch(new PreservationSpaceNotificationAction.GetAllSuccess(action, collection));
        }),
        catchError((error: SolidifyHttpErrorResponseModel) => {
          ctx.dispatch(new PreservationSpaceNotificationAction.GetAllFail(action));
          throw new SolidifyStateError(this, error);
        }),
      );
  }

  @OverrideDefaultAction()
  @Action(PreservationSpaceNotificationAction.GetById)
  getById(ctx: StateContext<PreservationSpaceNotificationStateModel>, action: PreservationSpaceNotificationAction.GetById): Observable<NotificationDlcm> {
    let reset = {};
    if (!action.keepCurrentContext) {
      reset = {
        current: undefined,
      };
    }
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
      ...reset,
    });

    let mode = ApiActionNameEnum.INBOX;
    if (ctx.getState().mode === NotificationModeEnum.sent) {
      mode = ApiActionNameEnum.SENT;
    }
    return this._apiService.getById<NotificationDlcm>(this._urlResource + urlSeparator + mode, action.id)
      .pipe(
        tap((model: NotificationDlcm) => {
          ctx.dispatch(new PreservationSpaceNotificationAction.GetByIdSuccess(action, model));
        }),
        catchError((error: SolidifyHttpErrorResponseModel) => {
          ctx.dispatch(new PreservationSpaceNotificationAction.GetByIdFail(action));
          throw new SolidifyStateError(this, error);
        }),
      );
  }

  @Action(PreservationSpaceNotificationAction.SetProcessed)
  setProcessed(ctx: StateContext<PreservationSpaceNotificationStateModel>, action: PreservationSpaceNotificationAction.SetProcessed): Observable<Result> {
    return this._apiService.post<Result>(this._urlResource + urlSeparator + action.notificationId + urlSeparator + ApiActionNameEnum.SET_PROCESSED)
      .pipe(
        tap(result => ctx.dispatch(new PreservationSpaceNotificationAction.SetProcessedSuccess(action))),
        catchError((error: SolidifyHttpErrorResponseModel) => {
          ctx.dispatch(new PreservationSpaceNotificationAction.SetProcessedFail(action));
          throw new SolidifyStateError(this, error);
        }),
      );
  }

  @Action(PreservationSpaceNotificationAction.SetProcessedSuccess)
  setProcessedSuccess(ctx: StateContext<PreservationSpaceNotificationStateModel>, action: PreservationSpaceNotificationAction.SetProcessedSuccess): void {
    this._removeNotificationPendingListAndRefresh(ctx, (action.parentAction as PreservationSpaceNotificationAction.SetRefuse).notificationId, (action.parentAction as PreservationSpaceNotificationAction.SetRefuse).mode);
    const notificationCategory = action.parentAction.notificationCategory;
    if (notificationCategory === Enums.Notification.CategoryEnum.REQUEST) {
      this._notificationService.showSuccess(MARK_AS_TRANSLATABLE("preservationSpace.notifications.setProcessed.success"));
    } else if (notificationCategory === Enums.Notification.CategoryEnum.INFO) {
      this._notificationService.showSuccess(MARK_AS_TRANSLATABLE("preservationSpace.notifications.setRead.success"));
    }
    LocalStorageHelper.removeItemInList(LocalStorageEnum.notificationInboxPending, action.parentAction.notificationId);
    this._store.dispatch(new appActionNameSpace.UpdateNotificationInbox());
  }

  @Action(PreservationSpaceNotificationAction.SetProcessedFail)
  setProcessedFail(ctx: StateContext<PreservationSpaceNotificationStateModel>, action: PreservationSpaceNotificationAction.SetProcessedFail): void {
    const notificationCategory = action.parentAction.notificationCategory;
    if (notificationCategory === Enums.Notification.CategoryEnum.REQUEST) {
      this._notificationService.showError(MARK_AS_TRANSLATABLE("preservationSpace.notifications.setProcessed.fail"));
    } else if (notificationCategory === Enums.Notification.CategoryEnum.INFO) {
      this._notificationService.showError(MARK_AS_TRANSLATABLE("preservationSpace.notifications.setRead.fail"));
    }
  }

  @Action(PreservationSpaceNotificationAction.SetRefuse)
  setRefuse(ctx: StateContext<PreservationSpaceNotificationStateModel>, action: PreservationSpaceNotificationAction.SetRefuse): Observable<Result> {
    return this._apiService.post<Result>(this._urlResource + urlSeparator + action.notificationId + urlSeparator + ApiActionNameEnum.SET_REFUSED)
      .pipe(
        tap(result => ctx.dispatch(new PreservationSpaceNotificationAction.SetRefuseSuccess(action))),
        catchError((error: SolidifyHttpErrorResponseModel) => {
          ctx.dispatch(new PreservationSpaceNotificationAction.SetRefuseFail(action));
          throw new SolidifyStateError(this, error);
        }),
      );
  }

  @Action(PreservationSpaceNotificationAction.SetRefuseSuccess)
  setRefuseSuccess(ctx: StateContext<PreservationSpaceNotificationStateModel>, action: PreservationSpaceNotificationAction.SetRefuseSuccess): void {
    this._removeNotificationPendingListAndRefresh(ctx, action.parentAction.notificationId, action.parentAction.mode);
    this._notificationService.showSuccess(MARK_AS_TRANSLATABLE("preservationSpace.notifications.setRefuse.success"));
    LocalStorageHelper.removeItemInList(LocalStorageEnum.notificationInboxPending, action.parentAction.notificationId);
    this._store.dispatch(new appActionNameSpace.UpdateNotificationInbox());
  }

  @Action(PreservationSpaceNotificationAction.SetRefuseFail)
  setRefuseFail(ctx: StateContext<PreservationSpaceNotificationStateModel>, action: PreservationSpaceNotificationAction.SetRefuseFail): void {
    this._notificationService.showError(MARK_AS_TRANSLATABLE("preservationSpace.notifications.setRefuse.fail"));
  }

  @Action(PreservationSpaceNotificationAction.SetPending)
  setPending(ctx: StateContext<PreservationSpaceNotificationStateModel>, action: PreservationSpaceNotificationAction.SetPending): Observable<Result> {
    return this._apiService.post<Result>(this._urlResource + urlSeparator + action.notificationId + urlSeparator + ApiActionNameEnum.SET_PENDING)
      .pipe(
        tap(result => ctx.dispatch(new PreservationSpaceNotificationAction.SetPendingSuccess(action))),
        catchError((error: SolidifyHttpErrorResponseModel) => {
          ctx.dispatch(new PreservationSpaceNotificationAction.SetPendingFail(action));
          throw new SolidifyStateError(this, error);
        }),
      );
  }

  @Action(PreservationSpaceNotificationAction.SetPendingSuccess)
  setPendingSuccess(ctx: StateContext<PreservationSpaceNotificationStateModel>, action: PreservationSpaceNotificationAction.SetPendingSuccess): void {
    this._removeNotificationPendingListAndRefresh(ctx, action.parentAction.notificationId, action.parentAction.mode);
    const notificationCategory = action.parentAction.notificationCategory;
    if (notificationCategory === Enums.Notification.CategoryEnum.REQUEST) {
      this._notificationService.showSuccess(MARK_AS_TRANSLATABLE("preservationSpace.notifications.setPending.success"));
    } else if (notificationCategory === Enums.Notification.CategoryEnum.INFO) {
      this._notificationService.showSuccess(MARK_AS_TRANSLATABLE("preservationSpace.notifications.setUnread.success"));
    }
    if (CookieConsentUtil.isFeatureAuthorized(CookieType.localStorage, LocalStorageEnum.notificationInboxPending)) {
      LocalStorageHelper.addItemInList(LocalStorageEnum.notificationInboxPending, action.parentAction.notificationId);
    }
    this._store.dispatch(new appActionNameSpace.UpdateNotificationInbox());
  }

  private _removeNotificationPendingListAndRefresh(ctx: StateContext<PreservationSpaceNotificationStateModel>,
                                                  notificationId: string,
                                                  mode: ViewModeEnum): void {
    LocalStorageHelper.removeItemInList(LocalStorageEnum.notificationInboxPending, notificationId);
    if (mode === ViewModeEnum.detail) {
      ctx.dispatch(new PreservationSpaceNotificationAction.GetById(notificationId));
    } else if (mode === ViewModeEnum.list) {
      ctx.dispatch(new PreservationSpaceNotificationAction.GetAll());
    }
  }

  @Action(PreservationSpaceNotificationAction.SetPendingFail)
  setPendingFail(ctx: StateContext<PreservationSpaceNotificationStateModel>, action: PreservationSpaceNotificationAction.SetPendingFail): void {
    const notificationCategory = action.parentAction.notificationCategory;
    if (notificationCategory === Enums.Notification.CategoryEnum.REQUEST) {
      this._notificationService.showError(MARK_AS_TRANSLATABLE("preservationSpace.notifications.setPending.fail"));
    } else if (notificationCategory === Enums.Notification.CategoryEnum.INFO) {
      this._notificationService.showError(MARK_AS_TRANSLATABLE("preservationSpace.notifications.setUnread.fail"));
    }
  }

  private _redirectToNotificationDetail(ctx: StateContext<PreservationSpaceNotificationStateModel>, notificationId: string): void {
    ctx.dispatch(new PreservationSpaceNotificationAction.GetById(notificationId));
  }

  @OverrideDefaultAction()
  @Action(PreservationSpaceNotificationAction.Clean)
  clean(ctx: StateContext<PreservationSpaceNotificationStateModel>, action: PreservationSpaceNotificationAction.Clean): void {
    const orgUnitId = ctx.getState().orgUnitId;
    const mode = ctx.getState().mode;
    super.clean(ctx, action);
    ctx.patchState({
      orgUnitId,
      mode,
    });
  }
}
