/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Portal - shared-avatar-upload-wrapper.container.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {AdminPersonState} from "@admin/person/stores/admin-person.state";
import {adminUserActionNameSpace} from "@admin/user/stores/admin-user.action";
import {AdminUserState} from "@admin/user/stores/admin-user.state";
import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  HostBinding,
  Input,
  OnInit,
} from "@angular/core";
import {FormBuilder} from "@angular/forms";
import {MatDialog} from "@angular/material/dialog";
import {DomSanitizer} from "@angular/platform-browser";
import {ActivatedRoute} from "@angular/router";
import {AppUserAction} from "@app/stores/user/app-user.action";
import {AppUserState} from "@app/stores/user/app-user.state";
import {UserInfo} from "@models";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {SharedAbstractImageUploadWrapperContainer} from "@shared/components/containers/shared-abstract-image-upload-wrapper/shared-abstract-image-upload-wrapper.container";
import {
  SharedUploadImageDialog,
  SharedUploadImageDialogData,
} from "@shared/components/dialogs/shared-upload-image/shared-upload-image.dialog";
import {ImageDisplayMode} from "@shared/components/presentationals/shared-image-display/shared-image-display.presentational";
import {LabelTranslateEnum} from "@shared/enums/label-translate.enum";
import {ResourceLogoStateModel} from "@shared/stores/resource-logo/resource-logo-state.model";
import {
  take,
  tap,
} from "rxjs/operators";
import {
  DialogUtil,
  isEmptyString,
  isNullOrUndefined,
  isTrue,
  MemoizedUtil,
  ofSolidifyActionCompleted,
  StoreUtil,
} from "solidify-frontend";

@Component({
  selector: "dlcm-shared-avatar-upload-wrapper-container",
  templateUrl: "./shared-avatar-upload-wrapper.container.html",
  styleUrls: ["./shared-avatar-upload-wrapper.container.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class SharedAvatarUploadWrapperContainer extends SharedAbstractImageUploadWrapperContainer implements OnInit {
  @Input()
  userInfo: UserInfo;

  @Input()
  resourceLogoState: typeof AppUserState | typeof AdminUserState | typeof AdminPersonState;

  @Input()
  isAvatarMissing: boolean = false;

  @Input()
  @HostBinding("class.is-editable")
  isEditable: boolean = true;

  get imageDisplayMode(): typeof ImageDisplayMode {
    return ImageDisplayMode;
  }

  constructor(protected readonly _store: Store,
              protected readonly _route: ActivatedRoute,
              protected readonly _fb: FormBuilder,
              protected readonly _sanitizer: DomSanitizer,
              protected readonly _cd: ChangeDetectorRef,
              protected readonly _actions$: Actions,
              protected readonly _dialog: MatDialog) {
    super(_store,
      _route,
      _fb,
      _sanitizer,
      _cd,
      _actions$,
      _dialog);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this._getUserPhoto();
  }

  override openModal(): void {
    if (isTrue(this.isEditable)) {
      super.openModal();
    }
  }

  protected _defineSharedImageDialogData(): SharedUploadImageDialogData {
    return {
      titleToTranslate: LabelTranslateEnum.uploadAvatar,
      imageChangedEvent: undefined,
      isLoadingObs: this.isLoadingObs,
      aspectRatio: 1,
      roundCropped: true,
      resizeToHeight: 145 * 2,
      resizeToWidth: 145 * 2,
    };
  }

  private _getUserPhoto(): void {
    if (this.isAvatarMissing) {
      return;
    }
    this.subscribe(
      StoreUtil.dispatchSequentialActionAndWaitForSubActionsCompletion(this._store, [
        {
          action: new this.resourceLogoNameSpace.GetPhoto(this.resId),
          subActionCompletions: [
            this._actions$.pipe(ofSolidifyActionCompleted(this.resourceLogoNameSpace.GetPhotoSuccess)),
            this._actions$.pipe(ofSolidifyActionCompleted(this.resourceLogoNameSpace.GetPhotoFail)),
          ],
        },
      ]).pipe(
        take(1),
        tap((result) => {
          if (result.success) {
            this.photoObs = MemoizedUtil.select(this._store, this.resourceLogoState, (state: ResourceLogoStateModel<any>) => state.logo);
          }
        }),
      ),
    );
  }

  override fileChangeEvent(event: any): void {
    if (this.resourceLogoNameSpace === adminUserActionNameSpace && this.userInfo.externalUid === MemoizedUtil.selectSnapshot(this._store, AppUserState, state => state.current.externalUid)) {
      // need to refresh user profile
      this._dispatchFileEventAndRefreshProfile(event);
    } else {
      super.fileChangeEvent(event);
    }
  }

  private _dispatchFileEventAndRefreshProfile(event: any): void {
    if (isNullOrUndefined(event.target.value) || isEmptyString(event.target.value)) {
      return;
    }
    this._sharedUploadImageDialogData.imageChangedEvent = event;
    this.subscribe(DialogUtil.open(this._dialog, SharedUploadImageDialog, this._sharedUploadImageDialogData, {
      width: "90%",
    }, blob => {
      this.subscribe(StoreUtil.dispatchActionAndWaitForSubActionCompletion(this._store, this._actions$,
        new this.resourceLogoNameSpace.UploadPhoto(this.resId, blob as any),
        this.resourceLogoNameSpace.UploadPhotoSuccess,
        resultAction => {
          this._store.dispatch(new this.resourceLogoNameSpace.GetPhoto(this.resId));
          this._store.dispatch(new AppUserAction.GetCurrentUser());
          this._avatarChangeBS.next(true);
        }));
      this.fileInput.nativeElement.value = null;
    }, () => {
      this.fileInput.nativeElement.value = null;
    }));
  }
}
