/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Portal - shared-logo-wrapper.container.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  Input,
  OnChanges,
  OnInit,
  SimpleChanges,
} from "@angular/core";
import {UserInfo} from "@models";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {SharedAbstractContainer} from "@shared/components/containers/shared-abstract/shared-abstract.container";
import {ImageDisplayMode} from "@shared/components/presentationals/shared-image-display/shared-image-display.presentational";
import {ResourceLogoNameSpace} from "@shared/stores/resource-logo/resource-logo-namespace.model";
import {ResourceLogoStateModel} from "@shared/stores/resource-logo/resource-logo-state.model";
import {Observable} from "rxjs/index";
import {
  distinctUntilChanged,
  filter,
  takeWhile,
  tap,
} from "rxjs/operators";
import {
  BaseResource,
  isFalse,
  isNotNullNorUndefined,
  isNullOrUndefined,
  MappingObject,
  MappingObjectUtil,
  MemoizedUtil,
} from "solidify-frontend";

@Component({
  selector: "dlcm-shared-logo-wrapper-container",
  templateUrl: "./shared-logo-wrapper.container.html",
  styleUrls: ["./shared-logo-wrapper.container.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class SharedLogoWrapperContainer<TResource extends BaseResource> extends SharedAbstractContainer implements OnChanges, OnInit {
  // COMPONENT FOR DATATABLE

  isLoading: boolean = true;

  @Input()
  logoNamespace: ResourceLogoNameSpace;

  @Input()
  logoState: any;

  @Input()
  isUser: boolean;

  @Input()
  userInfo: UserInfo;

  private _idResource: string;

  @Input()
  set idResource(idResource: string) {
    this._idResource = idResource;
  }

  get idResource(): string {
    return this._idResource;
  }

  @Input()
  isLogoPresent: boolean = true;

  @Input()
  imageMode: ImageDisplayMode = ImageDisplayMode.IN_LIST;

  listPhotosObs: Observable<MappingObject<string, string>>;

  photo: string | undefined;

  get imageDisplayMode(): typeof ImageDisplayMode {
    return ImageDisplayMode;
  }

  constructor(private readonly _store: Store,
              private readonly _actions$: Actions,
              private readonly _changeDetector: ChangeDetectorRef) {
    super();
  }

  ngOnInit(): void {
    super.ngOnInit();
    this._loadPhoto(this.logoNamespace, this.idResource, this.userInfo);
  }

  ngOnChanges(changes: SimpleChanges): void {
    this._loadPhoto(this.logoNamespace, this.idResource, this.userInfo);
  }

  private _loadPhoto(logoNamespace: ResourceLogoNameSpace, idResource: string, userInfo: UserInfo): void {
    if (isFalse(this.isLogoPresent)) {
      this.isLoading = false;
      return;
    }
    if (isNotNullNorUndefined(logoNamespace) && (isNotNullNorUndefined(idResource) || isNotNullNorUndefined(userInfo))) {
      this.photo = undefined;
      this.isLoading = true;
      this.listPhotosObs = MemoizedUtil.select(this._store, this.logoState, (state: ResourceLogoStateModel<TResource>) => state.listLogo);
      this._getLogo();
    }
  }

  protected _getLogo(): void {
    this._store.dispatch(new this.logoNamespace.GetPhotoByResId(this.idResource));

    const listPhotos = this._store.selectSnapshot((state: ResourceLogoStateModel<TResource>) => state.listLogo);
    const photo = this._getPhoto(listPhotos);
    if (isNullOrUndefined(photo)) {
      this.subscribe(this.listPhotosObs.pipe(
        distinctUntilChanged(),
        filter(list => isNotNullNorUndefined(this._getPhoto(list))),
        takeWhile((list) => this.isLoading === true),
        tap((list) => {
          this.photo = MappingObjectUtil.get(MemoizedUtil.selectSnapshot(this._store, this.logoState, (state: ResourceLogoStateModel<TResource>) => state.listLogo), this._idResource);
          if (isNotNullNorUndefined(this.photo)) {
            this.isLoading = false;
            this._changeDetector.detectChanges();
          }
        })));
    }
  }

  protected _getPhoto(list: MappingObject<string, string>): string | undefined {
    if (isNullOrUndefined(list)) {
      return undefined;
    }

    const photo: string = MappingObjectUtil.get(list, this._idResource);
    if (isNullOrUndefined(photo)) {
      return undefined;
    }
    return photo;
  }

}
