/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Portal - shared-avatar.presentational.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {
  ChangeDetectionStrategy,
  Component,
  HostBinding,
  Input,
} from "@angular/core";
import {
  DomSanitizer,
  SafeResourceUrl,
} from "@angular/platform-browser";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {UserInfo} from "@models";
import {ImageDisplayMode} from "@shared/components/presentationals/shared-image-display/shared-image-display.presentational";
import {
  isNotNullNorUndefinedNorWhiteString,
  isNullOrUndefined,
} from "solidify-frontend";

@Component({
  selector: "dlcm-shared-avatar",
  templateUrl: "./shared-avatar.presentational.html",
  styleUrls: ["./shared-avatar.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class SharedAvatarPresentational extends SharedAbstractPresentational {
  private _userInfo: UserInfo;

  @Input()
  set userInfo(value: UserInfo) {
    this._userInfo = value;
    this.computeInitial();
  }

  get userInfo(): UserInfo {
    return this._userInfo;
  }

  @HostBinding("class.is-photo")
  private _photoUserSanitize: SafeResourceUrl | undefined;

  @Input()
  set photoUser(value: string) {
    if (isNullOrUndefined(value)) {
      this._photoUserSanitize = undefined;
      return;
    }
    this._photoUserSanitize = this.sanitizeUrl(value);
  }

  get photoUserSanitize(): SafeResourceUrl {
    return this._photoUserSanitize;
  }

  @HostBinding("class")
  @Input()
  mode: ImageDisplayMode = ImageDisplayMode.IN_LIST;

  private readonly _DEFAULT_INITIAL: string = "";

  initial: string = this._DEFAULT_INITIAL;

  @HostBinding("style.width")
  @HostBinding("style.height")
  get size(): string {
    let size;
    switch (this.mode) {
      case ImageDisplayMode.MAIN:
        size = 300;
        break;
      case ImageDisplayMode.UPLOAD:
        size = 145;
        break;
      case ImageDisplayMode.IN_OVERLAY:
        size = 125;
        break;
      case ImageDisplayMode.IN_LIST:
      case ImageDisplayMode.NEXT_FORM_FIELD:
        size = 45;
        break;
      default:
        size = 300;
    }
    return size + "px";
  }

  constructor(private readonly _sanitizer: DomSanitizer) {
    super();
  }

  computeInitial(): void {
    if (isNullOrUndefined(this._userInfo)) {
      this.initial = this._DEFAULT_INITIAL;
    } else {
      const firstNameInitial = isNotNullNorUndefinedNorWhiteString(this._userInfo.firstName) ? this._userInfo.firstName.substring(0, 1) : "";
      const lastNameInitial = isNotNullNorUndefinedNorWhiteString(this._userInfo.lastName) ? this._userInfo.lastName.substring(0, 1) : "";
      this.initial = firstNameInitial + lastNameInitial;
    }
  }

  sanitizeUrl(url: string | undefined): undefined | SafeResourceUrl {
    if (isNullOrUndefined(url)) {
      return undefined;
    } else {
      return this._sanitizer.bypassSecurityTrustResourceUrl(url);
    }
  }
}
