/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Portal - shared-aip-list.routable.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  Injector,
  OnDestroy,
  OnInit,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {
  ActivatedRoute,
  NavigationCancel,
  NavigationEnd,
  Router,
} from "@angular/router";
import {appAuthorizedOrganizationalUnitNameSpace} from "@app/stores/authorized-organizational-unit/app-authorized-organizational-unit.action";
import {AppAuthorizedOrganizationalUnitState} from "@app/stores/authorized-organizational-unit/app-authorized-organizational-unit.state";
import {Enums} from "@enums";
import {environment} from "@environments/environment";
import {
  Aip,
  OrganizationalUnit,
} from "@models";
import {Navigate} from "@ngxs/router-plugin";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {DataTableComponentEnum} from "@shared/enums/data-table-component.enum";
import {LabelTranslateEnum} from "@shared/enums/label-translate.enum";
import {
  PreservationPlanningRoutesEnum,
  RoutesEnum,
  SharedAipRoutesEnum,
  urlSeparator,
} from "@shared/enums/routes.enum";
import {StateEnum} from "@shared/enums/state.enum";
import {
  AipHelper,
  AipMode,
} from "@shared/features/aip/helpers/aip.helper";
import {sharedAipCollectionStatusHistoryNamespace} from "@shared/features/aip/stores/aip-collection/status-history/shared-aip-collection-status-history.action";
import {SharedAipCollectionStatusHistoryState} from "@shared/features/aip/stores/aip-collection/status-history/shared-aip-collection-status-history.state";
import {
  SharedAipAction,
  sharedAipActionNameSpace,
} from "@shared/features/aip/stores/shared-aip.action";
import {
  SharedAipState,
  SharedAipStateModel,
} from "@shared/features/aip/stores/shared-aip.state";
import {DataTableComponentHelper} from "@shared/helpers/data-table-component.helper";
import {SecurityService} from "@shared/services/security.service";
import {Observable} from "rxjs";
import {
  distinctUntilChanged,
  filter,
  tap,
} from "rxjs/operators";
import {
  AbstractListRoutable,
  DataTableFieldTypeEnum,
  isNotNullNorUndefined,
  isNullOrUndefined,
  MappingObjectUtil,
  MemoizedUtil,
  OrderEnum,
  QueryParametersUtil,
  ResourceNameSpace,
  RouterExtensionService,
  Sort,
} from "solidify-frontend";

@Component({
  selector: "dlcm-shared-aip-list-routable",
  templateUrl: "../../../../../../../../node_modules/solidify-frontend/lib/components/routables/abstract-list/abstract-list.routable.html",
  styleUrls: ["./shared-aip-list.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class SharedAipListRoutable extends AbstractListRoutable<Aip, SharedAipStateModel> implements OnInit, OnDestroy {
  KEY_BACK_BUTTON: string | undefined = LabelTranslateEnum.back;
  readonly KEY_CREATE_BUTTON: string = undefined;
  readonly KEY_PARAM_NAME: keyof Aip & string = "info.name" as any;

  appAuthorizedOrganizationalUnitSort: Sort<OrganizationalUnit> = {
    field: "name",
    order: OrderEnum.ascending,
  };
  appAuthorizedOrganizationalUnitNameSpace: ResourceNameSpace = appAuthorizedOrganizationalUnitNameSpace;
  appAuthorizedOrganizationalUnitState: typeof AppAuthorizedOrganizationalUnitState = AppAuthorizedOrganizationalUnitState;

  mode: AipMode;

  stickyTopPosition: number = environment.defaultStickyDatatableHeight - 1;

  override skipInitialQuery: boolean = true;

  private readonly _KEY_ARCHIVAL_UNIT: keyof Aip = "archivalUnit";

  columnsSkippedToClear: string[] = [
    this._KEY_ARCHIVAL_UNIT,
  ];

  storagion_number: number | undefined = undefined;

  archivalUnit(value: SharedAipRoutesEnum): void {
    if (value === SharedAipRoutesEnum.aipTabUnit) {
      this.getAipListWithParameter(true);
    } else if (value === SharedAipRoutesEnum.aipTabCollections) {
      this.getAipListWithParameter(false);
    } else {
      this.getAipListWithParameter(undefined);
    }
  }

  constructor(protected readonly _store: Store,
              protected readonly _route: ActivatedRoute,
              protected readonly _routerExt: RouterExtensionService,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _actions$: Actions,
              protected readonly _dialog: MatDialog,
              protected readonly _injector: Injector,
              protected readonly _router: Router,
              protected readonly _securityService: SecurityService) {
    super(_store, _changeDetector, _route, _routerExt, _actions$, _dialog, StateEnum.shared_aip, sharedAipActionNameSpace, _injector, {
      canCreate: false,
      historyState: SharedAipCollectionStatusHistoryState,
      historyStateAction: sharedAipCollectionStatusHistoryNamespace,
      historyStatusEnumTranslate: Enums.Package.StatusEnumTranslate,
    }, StateEnum.shared);
    this.cleanIsNeeded = true;
  }

  ngOnInit(): void {
    //only clean the query-parameters if coming from the main menu but not from aip-detail since it already calls the clean in ngOnDestroy()
    if (isNotNullNorUndefined(this._routerExt.getPreviousUrl())) {
      if (this._routerExt.getPreviousUrl().includes(SharedAipRoutesEnum.aipDetail)) {
        this.cleanIsNeeded = false;
      } else {
        this.clean();
      }
    }
    super.ngOnInit();
    this.mode = AipHelper.determineMode(this._store);
    this._computeBackButtonLabel();
    this._updateListWithCurrentTab();
    this.subscribe(this._observeRoutingUpdate());
    if (this.mode === AipMode.AIP) {
      this.storagion_number = +this._route.snapshot.parent.parent.paramMap.get(PreservationPlanningRoutesEnum.storagionNumberWithoutPrefixParam);
    }
  }

  private _computeBackButtonLabel(): void {
    switch (this.mode) {
      case AipMode.AIP:
        this.KEY_BACK_BUTTON = LabelTranslateEnum.backToStoragionList;
        break;
      case AipMode.DOWNLOADED_AIP:
        this.KEY_BACK_BUTTON = LabelTranslateEnum.backToPreservationPlanning;
        break;
      case AipMode.AIP_STEWARD:
      default:
        this.KEY_BACK_BUTTON = LabelTranslateEnum.back;
        break;
    }
  }

  private _observeRoutingUpdate(): Observable<any> {
    return this._router.events
      .pipe(
        filter(event => event instanceof NavigationCancel || event instanceof NavigationEnd),
        distinctUntilChanged(),
        tap(event => {
          this._updateListWithCurrentTab();
        }),
      );
  }

  private _updateListWithCurrentTab(): void {
    const tab = this._route.snapshot.paramMap.get(SharedAipRoutesEnum.tabWithoutPrefixParam);
    this.archivalUnit(tab as SharedAipRoutesEnum);
  }

  override showDetail(model: Aip): void {
    if (this.mode === AipMode.AIP) {
      this._store.dispatch(new Navigate([RoutesEnum.preservationPlanningAip + urlSeparator + this.storagion_number + urlSeparator + SharedAipRoutesEnum.aipDetail, model.resId]));
    } else {
      this._store.dispatch(new Navigate([RoutesEnum.preservationPlanningAipDownloadedDetail, model.resId]));
    }
  }

  defineColumns(): void {
    this.columns = [
      {
        field: "info.name" as any,
        header: LabelTranslateEnum.nameLabel,
        type: DataTableFieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "smartSize",
        header: LabelTranslateEnum.size,
        type: DataTableFieldTypeEnum.string,
        order: OrderEnum.none,
        filterableField: "archiveSize",
        sortableField: "archiveSize",
        isFilterable: true,
        isSortable: true,
        alignment: "right",
        width: "100px",
      },
      {
        field: "archiveFileNumber",
        header: LabelTranslateEnum.files,
        type: DataTableFieldTypeEnum.number,
        order: OrderEnum.none,
        filterableField: "archiveFileNumber",
        isFilterable: true,
        isSortable: true,
        width: "80px",
      },
      {
        field: "info.status" as any,
        header: LabelTranslateEnum.status,
        type: DataTableFieldTypeEnum.singleSelect,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
        translate: true,
        filterEnum: Enums.Package.StatusEnumTranslate,
        component: DataTableComponentHelper.get(DataTableComponentEnum.status),
        alignment: "center",
      },
      {
        field: "info.organizationalUnitId" as any,
        header: LabelTranslateEnum.organizationalUnit,
        type: DataTableFieldTypeEnum.searchableSingleSelect,
        order: OrderEnum.none,
        component: DataTableComponentHelper.get(DataTableComponentEnum.organizationalUnitName),
        isFilterable: true,
        isSortable: true,
        resourceNameSpace: this.appAuthorizedOrganizationalUnitNameSpace,
        resourceState: this.appAuthorizedOrganizationalUnitState as any,
        searchableSingleSelectSort: this.appAuthorizedOrganizationalUnitSort,
      },
      {
        field: "creation.when" as any,
        header: LabelTranslateEnum.created,
        type: DataTableFieldTypeEnum.datetime,
        order: OrderEnum.none,
        isFilterable: false,
        isSortable: true,
      },
      {
        field: "lastUpdate.when" as any,
        header: LabelTranslateEnum.updated,
        type: DataTableFieldTypeEnum.datetime,
        order: OrderEnum.descending,
        isFilterable: false,
        isSortable: true,
      },
    ];
  }

  getAipListWithParameter(isArchivalUnit: boolean | undefined): void {
    let queryParameter = MemoizedUtil.queryParametersSnapshot(this._store, SharedAipState);
    queryParameter = QueryParametersUtil.clone(queryParameter);
    if (isNullOrUndefined(queryParameter.sort?.field)) {
      const columnToSort = this.columns.find(c => c.order !== OrderEnum.none);
      if (isNotNullNorUndefined(columnToSort)) {
        queryParameter.sort = {
          field: columnToSort.field,
          order: columnToSort.order,
        };
      }
    }
    if (isNullOrUndefined(isArchivalUnit)) {
      MappingObjectUtil.delete(queryParameter.search.searchItems, this._KEY_ARCHIVAL_UNIT);
    } else {
      MappingObjectUtil.set(queryParameter.search.searchItems, this._KEY_ARCHIVAL_UNIT, isArchivalUnit.toString());
    }
    this._store.dispatch(new SharedAipAction.GetAll(queryParameter, false));
  }

  conditionDisplayEditButton(model: Aip | undefined): boolean {
    return false;
  }

  conditionDisplayDeleteButton(model: Aip | undefined): boolean {
    return this._securityService.isRoot()
      && this.mode === AipMode.DOWNLOADED_AIP;
  }

  back(): void {
    if (this.mode === AipMode.AIP) {
      this._store.dispatch(new Navigate([RoutesEnum.preservationPlanningAip]));
    } else {
      this._store.dispatch(new Navigate([RoutesEnum.preservationPlanning]));
    }
  }

  ngOnDestroy(): void {
    super.ngOnDestroy();
  }
}
