/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Portal - shared-aip.state.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {HttpParams} from "@angular/common/http";
import {Injectable} from "@angular/core";
import {
  ActivatedRoute,
  Router,
} from "@angular/router";
import {Aip} from "@models";
import {Navigate} from "@ngxs/router-plugin";
import {
  Action,
  Actions,
  Selector,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {
  AppRoutesEnum,
  RoutesEnum,
  SharedAipRoutesEnum,
  urlSeparator,
} from "@shared/enums/routes.enum";
import {StateEnum} from "@shared/enums/state.enum";
import {AipHelper} from "@shared/features/aip/helpers/aip.helper";
import {
  defaultSharedAipCollectionValue,
  SharedAipCollectionState,
  SharedAipCollectionStateModel,
} from "@shared/features/aip/stores/aip-collection/shared-aip-collection.state";
import {
  defaultSharedAipDataFileValue,
  SharedAipDataFileState,
  SharedAipDataFileStateModel,
} from "@shared/features/aip/stores/aip-data-file/shared-aip-data-file.state";
import {
  defaultSharedAipOrgUnitValue,
  SharedAipOrganizationalUnitState,
  SharedAipOrganizationalUnitStateModel,
} from "@shared/features/aip/stores/organizational-unit/shared-aip-organizational-unit.state";
import {
  SharedAipAction,
  sharedAipActionNameSpace,
} from "@shared/features/aip/stores/shared-aip.action";
import {
  SharedAipStatusHistoryState,
  SharedAipStatusHistoryStateModel,
} from "@shared/features/aip/stores/status-history/shared-aip-status-history.state";
import {defaultStatusHistoryInitValue} from "@shared/stores/status-history/status-history.state";
import {Observable} from "rxjs";
import {
  catchError,
  tap,
} from "rxjs/operators";
import {
  ApiService,
  defaultResourceStateInitValue,
  DownloadService,
  isNullOrUndefined,
  MARK_AS_TRANSLATABLE,
  NotificationService,
  OverrideDefaultAction,
  ResourceState,
  ResourceStateModel,
  Result,
  ResultActionStatusEnum,
  SolidifyHttpErrorResponseModel,
  SolidifyStateError,
  StoreUtil,
} from "solidify-frontend";
import {ApiActionNameEnum} from "../../../enums/api-action-name.enum";

export interface SharedAipStateModel extends ResourceStateModel<Aip> {
  [StateEnum.shared_aip_dataFile]: SharedAipDataFileStateModel;
  [StateEnum.shared_aip_organizationalUnit]: SharedAipOrganizationalUnitStateModel;
  [StateEnum.shared_aip_collection]: SharedAipCollectionStateModel;
  preservationPlanning_aip_statusHistory: SharedAipStatusHistoryStateModel;
}

@Injectable()
@State<SharedAipStateModel>({
  name: StateEnum.shared_aip,
  defaults: {
    ...defaultResourceStateInitValue(),
    [StateEnum.shared_aip_dataFile]: defaultSharedAipDataFileValue(),
    [StateEnum.shared_aip_organizationalUnit]: defaultSharedAipOrgUnitValue(),
    [StateEnum.shared_aip_collection]: defaultSharedAipCollectionValue(),
    preservationPlanning_aip_statusHistory: {...defaultStatusHistoryInitValue()},
  },
  children: [
    SharedAipOrganizationalUnitState,
    SharedAipCollectionState,
    SharedAipStatusHistoryState,
    SharedAipDataFileState,
  ],
})
export class SharedAipState extends ResourceState<SharedAipStateModel, Aip> {
  private readonly _KEY_QUERY_PARAM_REASON: string = "reason";
  private readonly _KEY_QUERY_PARAM_DURATION: string = "duration";

  constructor(protected readonly _apiService: ApiService,
              protected readonly _store: Store,
              protected readonly _notificationService: NotificationService,
              protected readonly _actions$: Actions,
              private readonly _router: Router,
              private readonly _route: ActivatedRoute,
              private readonly _downloadService: DownloadService) {
    super(_apiService, _store, _notificationService, _actions$, {
      nameSpace: sharedAipActionNameSpace,
      routeRedirectUrlAfterSuccessDeleteAction: RoutesEnum.preservationPlanningAipDownloaded,
      notificationResourceDeleteSuccessTextToTranslate: MARK_AS_TRANSLATABLE("aipDownloaded.notification.resource.delete.success"),
      notificationResourceDeleteFailTextToTranslate: MARK_AS_TRANSLATABLE("aipDownloaded.notification.resource.delete.fail"),
    });
  }

  protected get _urlResource(): string {
    return AipHelper.generateUrlResource(this._store);
  }

  @Selector()
  static currentAipName(state: SharedAipStateModel): string | undefined {
    return state.current.info.name;
  }

  @Selector()
  static isLoadingWithDependency(state: SharedAipStateModel): boolean {
    return this.isLoading(state);
  }

  @Selector()
  static isReadyToBeDisplayed(state: SharedAipStateModel): boolean {
    return !isNullOrUndefined(state.current);
  }

  @Selector()
  static isLoading(state: SharedAipStateModel): boolean {
    return StoreUtil.isLoadingState(state);
  }

  @Action(SharedAipAction.Download)
  download(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.Download): void {
    const fileName = "aip_" + action.id + ".zip";
    this._downloadService.download(`${this._urlResource}/${action.id}/${ApiActionNameEnum.DL}`, fileName);
  }

  @OverrideDefaultAction()
  @Action(SharedAipAction.GetById)
  getById(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.GetById): Observable<Aip> {
    let reset = {};
    if (!action.keepCurrentContext) {
      reset = {
        current: undefined,
      };
    }
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
      ...reset,
    });

    const url = AipHelper.generateUrlResource(this._store, action.storagionNumber, action.url);
    return this._apiService.getById<Aip>(url, action.id)
      .pipe(
        tap((model: Aip) => {
          ctx.dispatch(new SharedAipAction.GetByIdSuccess(action, model));
        }),
        catchError((error: SolidifyHttpErrorResponseModel) => {
          ctx.dispatch(new SharedAipAction.GetByIdFail(action));
          throw new SolidifyStateError(this, error);
        }),
      );
  }

  @Action(SharedAipAction.Reindex)
  reindex(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.Reindex): Observable<Result> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });
    return this._apiService.post<Result>(`${this._urlResource}/${action.id}/${ApiActionNameEnum.REINDEX}`)
      .pipe(
        tap(result => {
          if (result?.status === ResultActionStatusEnum.EXECUTED) {
            ctx.dispatch(new SharedAipAction.ReindexSuccess(action));
          } else {
            ctx.dispatch(new SharedAipAction.ReindexFail(action));
          }
        }),
        catchError((error: SolidifyHttpErrorResponseModel) => {
          ctx.dispatch(new SharedAipAction.ReindexFail(action));
          throw error;
        }),
      );
  }

  @Action(SharedAipAction.ReindexSuccess)
  reindexSuccess(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.ReindexSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this._notificationService.showInformation(MARK_AS_TRANSLATABLE("notification.aip.action.reindex.success"));
    ctx.dispatch(new SharedAipAction.GetById(action.parentAction.id, true));
  }

  @Action(SharedAipAction.ReindexFail)
  reindexFail(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.ReindexFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this._notificationService.showError(MARK_AS_TRANSLATABLE("notification.aip.action.reindex.fail"));
  }

  @Action(SharedAipAction.DeepChecksum)
  deepChecksum(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.DeepChecksum): Observable<Result> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });
    const params = new HttpParams().set("level", "deep");
    return this._apiService.post<any, Result>(`${this._urlResource}/${action.id}/${ApiActionNameEnum.CHECK_FIXITY}`, {params: params})
      .pipe(
        tap(result => {
          if (result?.status === ResultActionStatusEnum.EXECUTED) {
            ctx.dispatch(new SharedAipAction.DeepChecksumSuccess(action));
          } else {
            ctx.dispatch(new SharedAipAction.DeepChecksumFail(action));
          }
        }),
        catchError((error: SolidifyHttpErrorResponseModel) => {
          ctx.dispatch(new SharedAipAction.DeepChecksumFail(action));
          throw error;
        }),
      );
  }

  @Action(SharedAipAction.DeepChecksumSuccess)
  deepChecksumSuccess(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.DeepChecksumSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this._notificationService.showInformation(MARK_AS_TRANSLATABLE("notification.aip.action.deepChecksum.success"));
    ctx.dispatch(new SharedAipAction.GetById(action.parentAction.id, true));
  }

  @Action(SharedAipAction.DeepChecksumFail)
  deepChecksumFail(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.DeepChecksumFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this._notificationService.showError(MARK_AS_TRANSLATABLE("notification.aip.action.deepChecksum.fail"));
  }

  @Action(SharedAipAction.SimpleChecksum)
  simpleChecksum(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.SimpleChecksum): Observable<Result> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });
    const params = new HttpParams().set("level", "simple");
    return this._apiService.post<any, Result>(`${this._urlResource}/${action.id}/${ApiActionNameEnum.CHECK_FIXITY}`, {params: params})
      .pipe(
        tap(result => {
          if (result?.status === ResultActionStatusEnum.EXECUTED) {
            ctx.dispatch(new SharedAipAction.SimpleChecksumSuccess(action));
          } else {
            ctx.dispatch(new SharedAipAction.SimpleChecksumFail(action));
          }
        }),
        catchError((error: SolidifyHttpErrorResponseModel) => {
          ctx.dispatch(new SharedAipAction.SimpleChecksumFail(action));
          throw error;
        }),
      );
  }

  @Action(SharedAipAction.SimpleChecksumSuccess)
  simpleChecksumSuccess(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.SimpleChecksumSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this._notificationService.showInformation(MARK_AS_TRANSLATABLE("notification.aip.action.simpleChecksum.success"));
    ctx.dispatch(new SharedAipAction.GetById(action.parentAction.id, true));
  }

  @Action(SharedAipAction.SimpleChecksumFail)
  simpleChecksumFail(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.SimpleChecksumFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this._notificationService.showError(MARK_AS_TRANSLATABLE("notification.aip.action.simpleChecksum.fail"));
  }

  @Action(SharedAipAction.Resume)
  resume(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.Resume): Observable<Result> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });
    return this._apiService.post<any, Result>(`${this._urlResource}/${action.id}/${ApiActionNameEnum.RESUME}`, null)
      .pipe(
        tap(result => {
          if (result?.status === ResultActionStatusEnum.EXECUTED) {
            ctx.dispatch(new SharedAipAction.ResumeSuccess(action));
          } else {
            ctx.dispatch(new SharedAipAction.ResumeFail(action));
          }
        }),
        catchError((error: SolidifyHttpErrorResponseModel) => {
          ctx.dispatch(new SharedAipAction.ResumeFail(action));
          throw error;
        }),
      );
  }

  @Action(SharedAipAction.ResumeSuccess)
  resumeSuccess(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.ResumeSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this._notificationService.showInformation(MARK_AS_TRANSLATABLE("notification.aip.action.resume.success"));
    ctx.dispatch(new SharedAipAction.GetById(action.parentAction.id, true));
  }

  @Action(SharedAipAction.ResumeFail)
  resumeFail(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.ResumeFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this._notificationService.showError(MARK_AS_TRANSLATABLE("notification.aip.action.resume.fail"));
  }

  @Action(SharedAipAction.GoToAip)
  goToAip(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.GoToAip): void {
    ctx.patchState({
      current: action.aip,
    });
    const pathAipDetail = RoutesEnum.preservationPlanningAip + urlSeparator + action.storagion_number + urlSeparator + SharedAipRoutesEnum.aipDetail + AppRoutesEnum.separator;
    const path = [pathAipDetail, action.aip.resId];
    ctx.dispatch(new Navigate(path));
  }

  @Action(SharedAipAction.Check)
  check(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.Check): Observable<Result> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });
    return this._apiService.post<any, Result>(`${this._urlResource}/${action.id}/${ApiActionNameEnum.CHECK}`, null)
      .pipe(
        tap(result => {
          if (result?.status === ResultActionStatusEnum.EXECUTED) {
            ctx.dispatch(new SharedAipAction.CheckSuccess(action));
          } else {
            ctx.dispatch(new SharedAipAction.CheckFail(action));
          }
        }),
        catchError((error: SolidifyHttpErrorResponseModel) => {
          ctx.dispatch(new SharedAipAction.CheckFail(action));
          throw error;
        }),
      );
  }

  @Action(SharedAipAction.CheckSuccess)
  checkSuccess(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.CheckSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this._notificationService.showInformation(MARK_AS_TRANSLATABLE("notification.aip.action.check.success"));
    ctx.dispatch(new SharedAipAction.GetById(action.parentAction.id, true));
  }

  @Action(SharedAipAction.CheckFail)
  checkFail(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.CheckFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this._notificationService.showError(MARK_AS_TRANSLATABLE("notification.aip.action.check.fail"));
  }

  @Action(SharedAipAction.Dispose)
  dispose(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.Dispose): Observable<string> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });
    return this._apiService.delete<string>(`${this._urlResource}/${action.id}`, null)
      .pipe(
        tap(result => {
          ctx.dispatch(new SharedAipAction.DisposeSuccess(action));
        }),
        catchError((error: SolidifyHttpErrorResponseModel) => {
          ctx.dispatch(new SharedAipAction.DisposeFail(action));
          throw error;
        }),
      );
  }

  @Action(SharedAipAction.DisposeSuccess)
  disposeSuccess(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.DisposeSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this._notificationService.showInformation(MARK_AS_TRANSLATABLE("notification.aip.action.dispose.success"));
    ctx.dispatch(new SharedAipAction.GetById(action.parentAction.id, true));
  }

  @Action(SharedAipAction.DisposeFail)
  disposeFail(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.DisposeFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this._notificationService.showError(MARK_AS_TRANSLATABLE("notification.aip.action.dispose.fail"));
  }

  @Action(SharedAipAction.ApproveDisposal)
  approveDisposal(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.ApproveDisposal): Observable<Result> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });
    const customParams = {
      [this._KEY_QUERY_PARAM_REASON]: action.reason,
    };
    return this._apiService.post<any, Result>(`${this._urlResource}/${action.id}/${ApiActionNameEnum.APPROVE_DISPOSAL}`, null, customParams)
      .pipe(
        tap(result => {
          if (result?.status === ResultActionStatusEnum.EXECUTED) {
            ctx.dispatch(new SharedAipAction.ApproveDisposalSuccess(action));
          } else {
            ctx.dispatch(new SharedAipAction.ApproveDisposalFail(action));
          }
        }),
        catchError((error: SolidifyHttpErrorResponseModel) => {
          ctx.dispatch(new SharedAipAction.ApproveDisposalFail(action));
          throw error;
        }),
      );
  }

  @Action(SharedAipAction.ApproveDisposalSuccess)
  approveDisposalSuccess(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.ApproveDisposalSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this._notificationService.showInformation(MARK_AS_TRANSLATABLE("notification.aip.action.approveDisposal.success"));
    ctx.dispatch(new SharedAipAction.GetById(action.parentAction.id, true));
  }

  @Action(SharedAipAction.ApproveDisposalFail)
  approveDisposalFail(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.ApproveDisposalFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this._notificationService.showError(MARK_AS_TRANSLATABLE("notification.aip.action.approveDisposal.fail"));
  }

  @Action(SharedAipAction.ApproveDisposalByOrgunit)
  approveDisposalByOrgunit(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.ApproveDisposalByOrgunit): Observable<Result> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });
    return this._apiService.post<any, Result>(`${this._urlResource}/${action.id}/${ApiActionNameEnum.APPROVE_DISPOSAL_BY_ORGUNIT}`, null, {
      [this._KEY_QUERY_PARAM_REASON]: action.reason,
    })
      .pipe(
        tap(result => {
          if (result?.status === ResultActionStatusEnum.EXECUTED) {
            ctx.dispatch(new SharedAipAction.ApproveDisposalByOrgunitSuccess(action));
          } else {
            ctx.dispatch(new SharedAipAction.ApproveDisposalByOrgunitFail(action));
          }
        }),
        catchError((error: SolidifyHttpErrorResponseModel) => {
          ctx.dispatch(new SharedAipAction.ApproveDisposalByOrgunitFail(action));
          throw error;
        }),
      );
  }

  @Action(SharedAipAction.ApproveDisposalByOrgunitSuccess)
  approveDisposalByOrgunitSuccess(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.ApproveDisposalByOrgunitSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this._notificationService.showInformation(MARK_AS_TRANSLATABLE("notification.aip.action.approveDisposalByOrgunit.success"));
    ctx.dispatch(new SharedAipAction.GetById(action.parentAction.id, true));
  }

  @Action(SharedAipAction.ApproveDisposalByOrgunitFail)
  approveDisposalByOrgunitFail(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.ApproveDisposalByOrgunitFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this._notificationService.showError(MARK_AS_TRANSLATABLE("notification.aip.action.approveDisposalByOrgunit.fail"));
  }

  @Action(SharedAipAction.ExtendRetention)
  extendRetention(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.ExtendRetention): Observable<Result> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });
    return this._apiService.post<any, Result>(`${this._urlResource}/${action.id}/${ApiActionNameEnum.EXTEND_RETENTION}`, null, {
      [this._KEY_QUERY_PARAM_DURATION]: action.duration + "",
    })
      .pipe(
        tap(result => {
          if (result?.status === ResultActionStatusEnum.EXECUTED) {
            ctx.dispatch(new SharedAipAction.ExtendRetentionSuccess(action));
          } else {
            ctx.dispatch(new SharedAipAction.ExtendRetentionFail(action));
          }
        }),
        catchError((error: SolidifyHttpErrorResponseModel) => {
          ctx.dispatch(new SharedAipAction.ExtendRetentionFail(action));
          throw error;
        }),
      );
  }

  @Action(SharedAipAction.ExtendRetentionSuccess)
  extendRetentionSuccess(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.ExtendRetentionSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this._notificationService.showInformation(MARK_AS_TRANSLATABLE("notification.aip.action.extendRetention.success"));
    ctx.dispatch(new SharedAipAction.GetById(action.parentAction.id, true));
  }

  @Action(SharedAipAction.ExtendRetentionFail)
  extendRetentionFail(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.ExtendRetentionFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this._notificationService.showError(MARK_AS_TRANSLATABLE("notification.aip.action.extendRetention.fail"));
  }

  @Action(SharedAipAction.PutInError)
  putInError(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.PutInError): Observable<Result> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });
    return this._apiService.post<Result>(`${this._urlResource}/${action.id}/${ApiActionNameEnum.PUT_IN_ERROR}`, null)
      .pipe(
        tap(result => {
          if (result?.status === ResultActionStatusEnum.EXECUTED) {
            ctx.dispatch(new SharedAipAction.PutInErrorSuccess(action));
          } else {
            ctx.dispatch(new SharedAipAction.PutInErrorFail(action));
          }
        }),
        catchError((error: SolidifyHttpErrorResponseModel) => {
          ctx.dispatch(new SharedAipAction.PutInErrorFail(action));
          throw error;
        }),
      );
  }

  @Action(SharedAipAction.PutInErrorSuccess)
  putInErrorSuccess(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.PutInErrorSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this._notificationService.showInformation(MARK_AS_TRANSLATABLE("notification.aip.action.put-in-error.success"));
    ctx.dispatch(new SharedAipAction.GetById(action.parentAction.id, true));
  }

  @Action(SharedAipAction.PutInErrorFail)
  putInErrorFail(ctx: StateContext<SharedAipStateModel>, action: SharedAipAction.PutInErrorFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
    this._notificationService.showError(MARK_AS_TRANSLATABLE("notification.aip.action.put-in-error.fail"));
  }
}
