/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Portal - application-maintenance-guard.service.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {Injectable} from "@angular/core";
import {
  ActivatedRouteSnapshot,
  CanActivate,
  Router,
} from "@angular/router";
import {AppState} from "@app/stores/app.state";
import {Enums} from "@enums";
import {environment} from "@environments/environment";
import {Navigate} from "@ngxs/router-plugin";
import {Store} from "@ngxs/store";
import {RoutesEnum} from "@shared/enums/routes.enum";
import {LocalStateModel} from "@shared/models/local-state.model";
import {PermissionUtil} from "@shared/utils/permission.util";
import {
  Observable,
  of,
} from "rxjs";
import {
  filter,
  map,
  take,
} from "rxjs/operators";
import {
  isFalse,
  isTrue,
  MemoizedUtil,
} from "solidify-frontend";

@Injectable({
  providedIn: "root",
})
export class ApplicationMaintenanceGuardService implements CanActivate {
  constructor(public readonly _router: Router,
              public readonly _store: Store) {
  }

  canActivate(route: ActivatedRouteSnapshot): Observable<boolean> {
    if (isTrue(MemoizedUtil.selectSnapshot(this._store, AppState, state => state.isServerOffline))) {
      this._store.dispatch(new Navigate([RoutesEnum.serverOffline]));
    }

    if (isFalse(environment.maintenanceMode)) {
      return of(true);
    }

    return MemoizedUtil.select(this._store, AppState, st => st.isApplicationInitialized).pipe(
      filter(isApplicationInitialized => isApplicationInitialized),
      take(1),
      map(() => {
        const currentUserRole = this._store.selectSnapshot<Enums.UserApplicationRole.UserApplicationRoleEnum[]>((st: LocalStateModel) => st.application.userRoles);
        const isLoggedIn = this._store.selectSnapshot<boolean>((st: LocalStateModel) => st.application.isLoggedIn);

        if (isLoggedIn === false || !PermissionUtil.isRootPermission(currentUserRole)) {
          this._store.dispatch(new Navigate([RoutesEnum.maintenance]));
          return false;
        }
        return true;
      }),
    );
  }
}
