/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Portal - security.service.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {Injectable} from "@angular/core";
import {AppState} from "@app/stores/app.state";
import {AppArchiveAclState} from "@app/stores/archive-acl/app-archive-acl.state";
import {DepositEditModeEnum} from "@deposit/enums/deposit-edit-mode.enum";
import {DepositAction} from "@deposit/stores/deposit.action";
import {DepositState} from "@deposit/stores/deposit.state";
import {Enums} from "@enums";
import {environment} from "@environments/environment";
import {
  Deposit,
  OrganizationalUnit,
  Role,
} from "@models";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {
  Observable,
  of,
} from "rxjs";
import {map} from "rxjs/operators";
import {
  isFalse,
  isNotNullNorUndefined,
  isNullOrUndefined,
  MemoizedUtil,
  SolidifySecurityService,
} from "solidify-frontend";

@Injectable({
  providedIn: "root",
})
export class SecurityService extends SolidifySecurityService {

  private readonly _ORGUNIT_ROLE_NEED_ACCESS_TO_CLOSED_DATASET: Enums.Role.RoleEnum[] = [Enums.Role.RoleEnum.STEWARD, Enums.Role.RoleEnum.MANAGER];
  private readonly _ORGUNIT_ROLE_NEED_TO_EDIT: Enums.Role.RoleEnum[] = [Enums.Role.RoleEnum.MANAGER];
  private readonly _DEPOSIT_ROLE_NEED_TO_CREATE: Enums.Role.RoleEnum[] = [Enums.Role.RoleEnum.MANAGER, Enums.Role.RoleEnum.STEWARD, Enums.Role.RoleEnum.APPROVER, Enums.Role.RoleEnum.CREATOR];
  private readonly _DEPOSIT_ROLE_NEED_TO_EDIT: Enums.Role.RoleEnum[] = [Enums.Role.RoleEnum.MANAGER, Enums.Role.RoleEnum.STEWARD, Enums.Role.RoleEnum.APPROVER, Enums.Role.RoleEnum.CREATOR];
  private readonly _DEPOSIT_ROLE_NEED_TO_VALIDATE: Enums.Role.RoleEnum[] = [Enums.Role.RoleEnum.MANAGER, Enums.Role.RoleEnum.STEWARD, Enums.Role.RoleEnum.APPROVER];
  private readonly _DEPOSIT_STATUS_ALLOW_FULL_ALTERATION: Enums.Deposit.StatusEnum[] = [Enums.Deposit.StatusEnum.IN_PROGRESS];
  private readonly _DEPOSIT_STATUS_ALLOW_METADATA_ALTERATION: Enums.Deposit.StatusEnum[] = [Enums.Deposit.StatusEnum.EDITING_METADATA];
  private readonly _DEPOSIT_STATUS_ALLOW_VALIDATION: Enums.Deposit.StatusEnum[] = [Enums.Deposit.StatusEnum.IN_VALIDATION];

  constructor(protected readonly _store: Store,
              protected readonly _actions$: Actions) {
    super(_store, _actions$, environment);
  }

  canCreateDepositOnOrgUnit(orgUnitId: string): boolean {
    if (this.isRootOrAdmin()) {
      return true;
    }
    if (!this.isMemberOfOrgUnit(orgUnitId)) {
      return false;
    }
    return this._currentUserHaveRoleInListForOrgUnit(orgUnitId, this._DEPOSIT_ROLE_NEED_TO_CREATE);
  }

  isStewardOfAnyOrgUnit(orgUnitList: OrganizationalUnit[]): boolean {
    return orgUnitList.some(o => this.isStewardOfOrgUnit(o.resId));
  }

  isApproverOfAnyOrgUnit(orgUnitList: OrganizationalUnit[]): boolean {
    return orgUnitList.some(o => this.isApproverOfOrgUnit(o.resId));
  }

  isManagerOfOrgUnit(orgUnitId: string): boolean {
    if (this.isRootOrAdmin()) {
      return true;
    }
    if (!this.isMemberOfOrgUnit(orgUnitId)) {
      return false;
    }
    return this._currentUserHaveRoleInListForOrgUnit(orgUnitId, this._ORGUNIT_ROLE_NEED_TO_EDIT);
  }

  isStewardOfOrgUnit(orgUnitId: string): boolean {
    if (this.isRootOrAdmin()) {
      return true;
    }
    if (!this.isMemberOfOrgUnit(orgUnitId)) {
      return false;
    }
    return this._currentUserHaveRoleInListForOrgUnit(orgUnitId, this._ORGUNIT_ROLE_NEED_ACCESS_TO_CLOSED_DATASET);
  }

  isApproverOfOrgUnit(orgUnitId: string): boolean {
    if (this.isRootOrAdmin()) {
      return true;
    }
    if (!this.isMemberOfOrgUnit(orgUnitId)) {
      return false;
    }
    return this._currentUserHaveRoleInListForOrgUnit(orgUnitId, this._DEPOSIT_ROLE_NEED_TO_VALIDATE);
  }

  isActiveArchiveAcl(archiveId: string): boolean {
    const listAcl = MemoizedUtil.listSnapshot(this._store, AppArchiveAclState);
    const archiveAcl = listAcl.find(a => a.aipId === archiveId);
    return isNotNullNorUndefined(archiveAcl) && isFalse(archiveAcl.expired);
  }

  canSeeDetailDeposit(deposit: Deposit): boolean {
    if (this.isRootOrAdmin()) {
      return true;
    }
    return this.isMemberOfOrgUnit(deposit.organizationalUnitId);
  }

  canSeeDetailDepositById(depositId: string): Observable<boolean> {
    if (this.isRootOrAdmin()) {
      return of(true);
    }

    let currentDeposit = MemoizedUtil.currentSnapshot(this._store, DepositState);
    if (isNullOrUndefined(currentDeposit) || currentDeposit.resId !== depositId) {
      return this._store.dispatch(new DepositAction.GetById(depositId)).pipe(
        map(state => {
          currentDeposit = MemoizedUtil.currentSnapshot(this._store, DepositState);
          return this.isMemberOfOrgUnit(currentDeposit.organizationalUnitId);
        }),
      );
    } else {
      return of(this.isMemberOfOrgUnit(currentDeposit.organizationalUnitId));
    }
  }

  canEditDeposit(deposit: Deposit): DepositEditModeEnum {
    const depositEditMode = this._depositEditionModeStep(deposit);
    if (depositEditMode === DepositEditModeEnum.none) {
      return DepositEditModeEnum.none;
    }
    if (this.isRootOrAdmin()) {
      return depositEditMode;
    }
    const currentRoleOnOrgUnit = this.getRoleInOrgUnit(deposit.organizationalUnitId);
    if (isNullOrUndefined(currentRoleOnOrgUnit)) {
      return DepositEditModeEnum.none;
    }
    if (SecurityService._isCurrentRoleOnOrgUnitAuthorized(currentRoleOnOrgUnit, this._DEPOSIT_ROLE_NEED_TO_EDIT)) {
      return depositEditMode;
    }
    return DepositEditModeEnum.none;
  }

  canApproveOrRejectDeposit(deposit: Deposit): boolean {
    if (isFalse(this._depositInValidationStep(deposit))) {
      return false;
    }
    if (this.isRootOrAdmin()) {
      return true;
    }
    const currentRoleOnOrgUnit = this.getRoleInOrgUnit(deposit.organizationalUnitId);
    if (isNullOrUndefined(currentRoleOnOrgUnit)) {
      return false;
    }
    return SecurityService._isCurrentRoleOnOrgUnitAuthorized(currentRoleOnOrgUnit, this._DEPOSIT_ROLE_NEED_TO_VALIDATE);
  }

  private _depositInValidationStep(deposit: Deposit): boolean {
    return SecurityService._depositInState(deposit, this._DEPOSIT_STATUS_ALLOW_VALIDATION);
  }

  private _depositEditionModeStep(deposit: Deposit): DepositEditModeEnum {
    if (SecurityService._depositInState(deposit, this._DEPOSIT_STATUS_ALLOW_METADATA_ALTERATION)) {
      return DepositEditModeEnum.metadata;
    }
    if (SecurityService._depositInState(deposit, this._DEPOSIT_STATUS_ALLOW_FULL_ALTERATION)) {
      return DepositEditModeEnum.full;
    }
    return DepositEditModeEnum.none;
  }

  private static _depositInState(deposit: Deposit, inState: Enums.Deposit.StatusEnum[]): boolean {
    return inState.includes(deposit.status);
  }

  private static _isCurrentRoleOnOrgUnitAuthorized(currentRoleOnOrgUnit: Role, listRolesAuthorized: Enums.Role.RoleEnum[]): boolean {
    return listRolesAuthorized.includes(currentRoleOnOrgUnit?.resId as Enums.Role.RoleEnum);
  }

  canEditDepositById(depositId: string): Observable<boolean> {
    if (this.isRootOrAdmin()) {
      return of(true);
    }

    return this.canSeeDetailDepositById(depositId).pipe(
      map((isAuthorized: boolean) => {
        if (isFalse(isAuthorized)) {
          return false;
        }
        const currentDeposit = MemoizedUtil.currentSnapshot(this._store, DepositState);
        if (!this._depositEditionModeStep(currentDeposit)) {
          return false;
        }
        return this._currentUserHaveRoleInListForOrgUnit(currentDeposit.organizationalUnitId, this._DEPOSIT_ROLE_NEED_TO_EDIT);
      }),
    );
  }

  private _currentUserHaveRoleInListForOrgUnit(orgUnitId: string, listRolesAuthorized: Enums.Role.RoleEnum[]): boolean {
    const currentRoleInOrgUnit = this.getRoleInOrgUnit(orgUnitId);
    if (isNullOrUndefined(currentRoleInOrgUnit)) {
      return false;
    }
    return SecurityService._isCurrentRoleOnOrgUnitAuthorized(currentRoleInOrgUnit, listRolesAuthorized);
  }

  isRootOrAdmin(): boolean {
    const applicationRoles = this._store.selectSnapshot(AppState.currentUserApplicationRoleResId);
    if (isNullOrUndefined(applicationRoles)) {
      return false;
    }
    return applicationRoles.includes(Enums.UserApplicationRole.UserApplicationRoleEnum.admin) || applicationRoles.includes(Enums.UserApplicationRole.UserApplicationRoleEnum.root);
  }

  isRoot(): boolean {
    const applicationRoles = this._store.selectSnapshot(AppState.currentUserApplicationRoleResId);
    if (isNullOrUndefined(applicationRoles)) {
      return false;
    }
    return applicationRoles.includes(Enums.UserApplicationRole.UserApplicationRoleEnum.root);
  }

  isUser(): boolean {
    const applicationRoles = this._store.selectSnapshot(AppState.currentUserApplicationRoleResId);
    if (isNullOrUndefined(applicationRoles)) {
      return false;
    }
    return applicationRoles.includes(Enums.UserApplicationRole.UserApplicationRoleEnum.user);
  }

  getRoleInOrgUnit(orgUnitId: string): Role | undefined {
    const listAuthorizedOrgUnit = this._store.selectSnapshot(AppState.listAuthorizedOrganizationalUnit);
    return listAuthorizedOrgUnit.find(o => isNotNullNorUndefined(o.role) && o.resId === orgUnitId)?.role;
  }

  getRoleEnumInOrgUnit(orgUnitId: string): Enums.Role.RoleEnum | undefined {
    return this.getRoleInOrgUnit(orgUnitId)?.resId as Enums.Role.RoleEnum;
  }

  isMemberOfOrgUnit(orgUnitId: string): boolean {
    return isNotNullNorUndefined(this.getRoleInOrgUnit(orgUnitId));
  }

  isLoggedIn(): boolean {
    return MemoizedUtil.selectSnapshot(this._store, AppState, state => state.isLoggedIn);
  }

  isLoggedInObs(): Observable<boolean> {
    return MemoizedUtil.select(this._store, AppState, state => state.isLoggedIn);
  }
}
