/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Portal - shared-license.state.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {HttpClient} from "@angular/common/http";
import {Injectable} from "@angular/core";
import {DLCM_CONSTANTS} from "@app/constants";
import {ApiEnum} from "@app/shared/enums/api.enum";
import {
  SharedLicenseAction,
  sharedLicenseActionNameSpace,
} from "@app/shared/stores/license/shared-license.action";
import {License} from "@models";
import {
  Action,
  Actions,
  Selector,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {StateEnum} from "@shared/enums/state.enum";
import {ResourceLogoStateModel} from "@shared/stores/resource-logo/resource-logo-state.model";
import {
  defaultResourceLogoStateInitValue,
  ResourceLogoState,
  ResourceLogoStateModeEnum,
} from "@shared/stores/resource-logo/resource-logo.state";
import {Observable} from "rxjs";
import {
  catchError,
  tap,
} from "rxjs/operators";
import {
  ApiService,
  NotificationService,
  QueryParameters,
  ResourceStateModel,
  SolidifyHttpErrorResponseModel,
  SolidifyStateError,
  StoreUtil,
} from "solidify-frontend";
import {environment} from "../../../../environments/environment";

export interface SharedLicenseStateModel extends ResourceLogoStateModel<License> {
}

@Injectable()
@State<SharedLicenseStateModel>({
  name: StateEnum.shared_license,
  defaults: {
    ...defaultResourceLogoStateInitValue(),
    queryParameters: new QueryParameters(environment.defaultEnumValuePageSizeOption),
  },
})
export class SharedLicenseState extends ResourceLogoState<SharedLicenseStateModel, License> {
  constructor(protected readonly _apiService: ApiService,
              protected readonly _store: Store,
              protected readonly _notificationService: NotificationService,
              protected readonly _actions$: Actions,
              protected readonly _httpClient: HttpClient) {
    super(_apiService, _store, _notificationService, _actions$, {
      nameSpace: sharedLicenseActionNameSpace,
    }, _httpClient, ResourceLogoStateModeEnum.logo);
  }

  protected get _urlResource(): string {
    return ApiEnum.adminLicenses;
  }

  protected get _urlLogoResource(): string {
    return this._urlResource;
  }

  @Selector()
  static isLoading<T>(state: ResourceStateModel<T>): boolean {
    return StoreUtil.isLoadingState(state);
  }

  @Action(SharedLicenseAction.GetByIdentifierId)
  getByIdentifierId(ctx: StateContext<SharedLicenseStateModel>, action: SharedLicenseAction.GetByIdentifierId): Observable<License> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });
    return this._apiService.getById<License>(this._urlResource, action.identifierId, {
      [DLCM_CONSTANTS.IDENTIFIER_TYPE_PARAM]: action.identifierType,
    })
      .pipe(
        tap(result => {
          ctx.dispatch(new SharedLicenseAction.GetByIdentifierIdSuccess(action, result));
        }),
        catchError((error: SolidifyHttpErrorResponseModel) => {
          ctx.dispatch(new SharedLicenseAction.GetByIdentifierIdFail(action));
          throw new SolidifyStateError(this, error);
        }),
      );
  }

  @Action(SharedLicenseAction.GetByIdentifierIdSuccess)
  getByIdentifierIdSuccess(ctx: StateContext<SharedLicenseStateModel>, action: SharedLicenseAction.GetByIdentifierIdSuccess): void {
    ctx.patchState({
      current: action.license,
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
  }

  @Action(SharedLicenseAction.GetByIdentifierIdFail)
  getByIdentifierIdFail(ctx: StateContext<SharedLicenseStateModel>, action: SharedLicenseAction.GetByIdentifierIdFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
  }
}
