/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Portal - shared-notification.state.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {Injectable} from "@angular/core";
import {ApiEnum} from "@app/shared/enums/api.enum";
import {environment} from "@environments/environment";
import {NotificationDlcm} from "@models";
import {
  Action,
  Actions,
  Selector,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {StateEnum} from "@shared/enums/state.enum";
import {
  SharedNotificationAction,
  sharedNotificationActionNameSpace,
} from "@shared/stores/notification/shared-notification.action";
import {Observable} from "rxjs";
import {
  catchError,
  tap,
} from "rxjs/operators";
import {
  ApiService,
  defaultResourceStateInitValue,
  MARK_AS_TRANSLATABLE,
  NotificationService,
  OverrideDefaultAction,
  QueryParameters,
  ResourceState,
  ResourceStateModel,
  SolidifyStateError,
  StoreUtil,
} from "solidify-frontend";

export interface SharedNotificationStateModel extends ResourceStateModel<NotificationDlcm> {
}

@Injectable()
@State<SharedNotificationStateModel>({
  name: StateEnum.shared_notification,
  defaults: {
    ...defaultResourceStateInitValue(),
    queryParameters: new QueryParameters(environment.defaultEnumValuePageSizeOption),
  },
})
export class SharedNotificationState extends ResourceState<SharedNotificationStateModel, NotificationDlcm> {
  constructor(protected readonly _apiService: ApiService,
              protected readonly _store: Store,
              protected readonly _notificationService: NotificationService,
              protected readonly _actions$: Actions) {
    super(_apiService, _store, _notificationService, _actions$, {
      nameSpace: sharedNotificationActionNameSpace,
      notificationResourceCreateSuccessTextToTranslate: MARK_AS_TRANSLATABLE("notification.notification.requestSendWithSuccess"),
      notificationResourceCreateFailTextToTranslate: MARK_AS_TRANSLATABLE("notification.notification.unableToSendRequest"),
    });
  }

  protected get _urlResource(): string {
    return ApiEnum.adminNotifications;
  }

  @OverrideDefaultAction()
  @Action(SharedNotificationAction.Create)
  create(ctx: StateContext<SharedNotificationStateModel>, action: SharedNotificationAction.Create): Observable<NotificationDlcm> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });
    const url = this._urlResource;
    return this._apiService.post<NotificationDlcm>(url, action.modelFormControlEvent.model)
      .pipe(
        tap(() => {
          ctx.dispatch(new SharedNotificationAction.CreateSuccess(action, action.modelFormControlEvent.model));
        }),
        catchError((error) => {
          ctx.dispatch(new SharedNotificationAction.CreateFail(action, error));
          throw new SolidifyStateError(this, error);
        }),
        StoreUtil.catchValidationErrors(ctx, action.modelFormControlEvent, this._notificationService),
      );
  }

  @Selector()
  static isLoading<T>(state: ResourceStateModel<T>): boolean {
    return StoreUtil.isLoadingState(state);
  }
}
