/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Portal - shared-research-domain.state.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {Injectable} from "@angular/core";
import {ApiEnum} from "@app/shared/enums/api.enum";
import {ResearchDomain} from "@models";
import {
  Action,
  Actions,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {urlSeparator} from "@shared/enums/routes.enum";
import {StateEnum} from "@shared/enums/state.enum";
import {
  SharedResearchDomainAction,
  sharedResearchDomainActionNameSpace,
} from "@shared/stores/research-domain/shared-research-domain.action";
import {Observable} from "rxjs";
import {
  catchError,
  tap,
} from "rxjs/operators";
import {
  ApiService,
  CollectionTyped,
  defaultResourceStateInitValue,
  isNullOrUndefined,
  isWhiteString,
  MappingObjectUtil,
  NotificationService,
  OverrideDefaultAction,
  QueryParametersUtil,
  ResourceState,
  ResourceStateModel,
  SolidifyHttpErrorResponseModel,
  SolidifyStateError,
  StoreUtil,
} from "solidify-frontend";
import {ApiActionNameEnum} from "../../enums/api-action-name.enum";

export interface SharedResearchDomainStateModel extends ResourceStateModel<ResearchDomain> {
  sources: string[] | undefined;
}

@Injectable()
@State<SharedResearchDomainStateModel>({
  name: StateEnum.shared_researchDomain,
  defaults: {
    ...defaultResourceStateInitValue(),
    sources: undefined,
  },
})
export class SharedResearchDomainState extends ResourceState<SharedResearchDomainStateModel, ResearchDomain> {
  constructor(protected readonly _apiService: ApiService,
              protected readonly _store: Store,
              protected readonly _notificationService: NotificationService,
              protected readonly _actions$: Actions) {
    super(_apiService, _store, _notificationService, _actions$, {
      nameSpace: sharedResearchDomainActionNameSpace,
    });
  }

  protected get _urlResource(): string {
    return ApiEnum.adminResearchDomains;
  }

  @Action(SharedResearchDomainAction.GetSource)
  getSource(ctx: StateContext<SharedResearchDomainStateModel>, action: SharedResearchDomainAction.GetSource): Observable<string[]> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });

    return this._apiService.getByIdInPath<string[]>(`${this._urlResource}/${ApiActionNameEnum.LIST_RESEARCH_DOMAIN_SOURCES}`)
      .pipe(
        tap(result => {
          ctx.dispatch(new SharedResearchDomainAction.GetSourceSuccess(action, result));
        }),
        catchError((error: SolidifyHttpErrorResponseModel) => {
          ctx.dispatch(new SharedResearchDomainAction.GetSourceFail(action));
          throw new SolidifyStateError(this, error);
        }),
      );
  }

  @Action(SharedResearchDomainAction.GetSourceSuccess)
  getSourceSuccess(ctx: StateContext<SharedResearchDomainStateModel>, action: SharedResearchDomainAction.GetSourceSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
      sources: action.sources,
    });
  }

  @Action(SharedResearchDomainAction.GetSourceFail)
  getSourceFail(ctx: StateContext<SharedResearchDomainStateModel>, action: SharedResearchDomainAction.GetSourceFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
  }

  @OverrideDefaultAction()
  @Action(SharedResearchDomainAction.GetAll)
  getAll(ctx: StateContext<SharedResearchDomainStateModel>, action: SharedResearchDomainAction.GetAll): Observable<CollectionTyped<ResearchDomain>> {
    let url = this._urlResource;
    const searchItems = QueryParametersUtil.getSearchItems(action.queryParameters);
    const nameOrCode = MappingObjectUtil.get(searchItems, "nameOrCode");
    if (!isNullOrUndefined(nameOrCode) && !isWhiteString(nameOrCode)) {
      MappingObjectUtil.set(searchItems, "search", `i-name~${nameOrCode},i-code~${nameOrCode}`);
      MappingObjectUtil.set(searchItems, "match", "any");
      MappingObjectUtil.delete(searchItems, "nameOrCode");
      url = this._urlResource + urlSeparator + ApiActionNameEnum.SEARCH;
    }
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
      queryParameters: StoreUtil.getQueryParametersToApply(action.queryParameters, ctx),
    });
    return this._apiService.get<ResearchDomain>(url, ctx.getState().queryParameters)
      .pipe(
        StoreUtil.cancelUncompleted(ctx, this._actions$, [SharedResearchDomainAction.GetAll]),
        tap((collection: CollectionTyped<ResearchDomain>) => {
          ctx.dispatch(new SharedResearchDomainAction.GetAllSuccess(action, collection));
        }),
        catchError((error: SolidifyHttpErrorResponseModel) => {
          ctx.dispatch(new SharedResearchDomainAction.GetAllFail(action));
          throw new SolidifyStateError(this, error);
        }),
      );
  }
}
