/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Portal - resource-logo.state.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {
  HttpClient,
  HttpEventType,
  HttpHeaders,
} from "@angular/common/http";
import {Person} from "@models";
import {
  Actions,
  StateContext,
  Store,
} from "@ngxs/store";
import {ResourceLogoActionHelper} from "@shared/stores/resource-logo/resource-logo-action.helper";
import {ResourceLogoNameSpace} from "@shared/stores/resource-logo/resource-logo-namespace.model";
import {ResourceLogoOptions} from "@shared/stores/resource-logo/resource-logo-options.model";
import {ResourceLogoAction} from "@shared/stores/resource-logo/resource-logo.action";
import {Observable} from "rxjs";
import {
  catchError,
  map,
  tap,
} from "rxjs/operators";
import {
  ApiService,
  BaseResourceType,
  BaseStateModel,
  defaultResourceStateInitValue,
  isNotNullNorUndefined,
  isNullOrUndefined,
  MappingObjectUtil,
  NotificationService,
  ObjectUtil,
  RegisterDefaultAction,
  ResourceState,
  SolidifyHttpErrorResponseModel,
  StringUtil,
  UploadEventModel,
} from "solidify-frontend";
import {ApiActionNameEnum} from "../../enums/api-action-name.enum";
import {ResourceLogoStateModel} from "./resource-logo-state.model";

export const defaultResourceLogoStateInitValue: <TResourceType extends BaseResourceType> () => ResourceLogoStateModel<TResourceType> = () =>
  ({
    ...defaultResourceStateInitValue(),
    logo: undefined,
    isLoadingLogo: false,
    listLogo: {},
    isLoadingAllLogo: false,
  });

export abstract class ResourceLogoState<TStateModel extends BaseStateModel, TResource extends BaseResourceType> extends ResourceState<TStateModel, TResource> {
  protected readonly _FILE_KEY: string = "file";

  protected readonly _nameSpace: ResourceLogoNameSpace;

  protected constructor(protected _apiService: ApiService,
                        protected _store: Store,
                        protected _notificationService: NotificationService,
                        protected _actions$: Actions,
                        protected _options: ResourceLogoOptions,
                        protected _httpClient: HttpClient,
                        protected _mode: ResourceLogoStateModeEnum) {
    super(_apiService, _store, _notificationService, _actions$, _options);
  }

  private _getDownloadAction(): ApiActionNameEnum {
    if (this._mode === ResourceLogoStateModeEnum.logo) {
      return ApiActionNameEnum.DOWNLOAD_LOGO;
    }
    if (this._mode === ResourceLogoStateModeEnum.avatar) {
      return ApiActionNameEnum.DOWNLOAD_AVATAR;
    }
  }

  private _getUploadAction(): ApiActionNameEnum {
    if (this._mode === ResourceLogoStateModeEnum.logo) {
      return ApiActionNameEnum.UPLOAD_LOGO;
    }
    if (this._mode === ResourceLogoStateModeEnum.avatar) {
      return ApiActionNameEnum.UPLOAD_AVATAR;
    }
  }

  private _getDeleteAction(): ApiActionNameEnum {
    if (this._mode === ResourceLogoStateModeEnum.logo) {
      return ApiActionNameEnum.DELETE_LOGO;
    }
    if (this._mode === ResourceLogoStateModeEnum.avatar) {
      return ApiActionNameEnum.DELETE_AVATAR;
    }
  }

  protected abstract get _urlLogoResource(): string;

  @RegisterDefaultAction((userAvatarNameSpace: ResourceLogoNameSpace) => userAvatarNameSpace.GetPhoto, {}, ResourceState)
  getPhoto(ctx: StateContext<ResourceLogoStateModel<TResource>>, action: ResourceLogoAction.GetPhoto): Observable<any> {
    this._isLoadingLogo(ctx, true);

    let headers = new HttpHeaders();
    headers = headers.set("Content-Disposition", "attachment; filename=" + "photo");

    return this._httpClient.get(`${this._urlLogoResource}/${action.resId}/${this._getDownloadAction()}`, {
      headers,
      responseType: "blob",
    }).pipe(
      tap((data: Blob) => {
        ctx.dispatch(ResourceLogoActionHelper.getPhotoSuccess(this._nameSpace, action, data));
      }),
      catchError((error: SolidifyHttpErrorResponseModel) => {
        ctx.dispatch(ResourceLogoActionHelper.getPhotoFail(this._nameSpace, action));
        throw error;
      }),
    );
  }

  @RegisterDefaultAction((userAvatarNameSpace: ResourceLogoNameSpace) => userAvatarNameSpace.GetPhotoSuccess, {}, ResourceState)
  getPhotoSuccess(ctx: StateContext<ResourceLogoStateModel<TResource>>, action: ResourceLogoAction.GetPhotoSuccess): void {
    ctx.patchState({
      isLoadingLogo: false,
      logo: isNullOrUndefined(action.blob) ? undefined : URL.createObjectURL(action.blob),
    });
  }

  @RegisterDefaultAction((userAvatarNameSpace: ResourceLogoNameSpace) => userAvatarNameSpace.GetPhotoFail, {}, ResourceState)
  getPhotoFail(ctx: StateContext<ResourceLogoStateModel<TResource>>, action: ResourceLogoAction.GetPhotoFail): void {
    ctx.patchState({
      isLoadingLogo: false,
      logo: undefined,
    });
  }

  @RegisterDefaultAction((userAvatarNameSpace: ResourceLogoNameSpace) => userAvatarNameSpace.GetPhotoByResId, {}, ResourceState)
  getPhotoByResId(ctx: StateContext<ResourceLogoStateModel<TResource>>, action: ResourceLogoAction.GetPhotoByResId): Observable<any> {
    ctx.patchState({
      isLoadingAllLogo: true,
    });

    let headers = new HttpHeaders();
    headers = headers.set("Content-Disposition", "attachment; filename=" + "photo");

    return this._httpClient.get(`${this._urlLogoResource}/${action.resId}/${this._getDownloadAction()}`, {
      headers,
      responseType: "blob",
    }).pipe(
      tap((data: Blob) => {
        ctx.dispatch(ResourceLogoActionHelper.getPhotoByResIdSuccess(this._nameSpace, action, data));
      }),
      catchError((error: SolidifyHttpErrorResponseModel) => {
        ctx.dispatch(ResourceLogoActionHelper.getPhotoByResIdFail(this._nameSpace, action));
        throw error;
      }),
    );
  }

  @RegisterDefaultAction((userAvatarNameSpace: ResourceLogoNameSpace) => userAvatarNameSpace.GetPhotoByResIdSuccess, {}, ResourceState)
  getPhotoByResIdSuccess(ctx: StateContext<ResourceLogoStateModel<TResource>>, action: ResourceLogoAction.GetPhotoByResIdSuccess): void {
    const logoCopy = ObjectUtil.clone(ctx.getState().listLogo);
    if (isNullOrUndefined(action.blob)) {
      MappingObjectUtil.set(logoCopy, action.parentAction.resId, StringUtil.stringEmpty);
    } else {
      MappingObjectUtil.set(logoCopy, action.parentAction.resId, URL.createObjectURL(action.blob));
    }
    ctx.patchState({
      isLoadingAllLogo: false,
      listLogo: logoCopy,
    });
  }

  @RegisterDefaultAction((userAvatarNameSpace: ResourceLogoNameSpace) => userAvatarNameSpace.GetPhotoByResIdFail, {}, ResourceState)
  getPhotoByResIdFail(ctx: StateContext<ResourceLogoStateModel<TResource>>, action: ResourceLogoAction.GetPhotoByResIdFail): void {
    const logoCopy = ObjectUtil.clone(ctx.getState().listLogo);
    MappingObjectUtil.set(logoCopy, action.parentAction.resId, StringUtil.stringEmpty);

    ctx.patchState({
      isLoadingAllLogo: false,
      listLogo: logoCopy,
    });
  }

  @RegisterDefaultAction((userAvatarNameSpace: ResourceLogoNameSpace) => userAvatarNameSpace.UploadPhoto, {}, ResourceState)
  uploadPhoto(ctx: StateContext<ResourceLogoStateModel<TResource>>, action: ResourceLogoAction.UploadPhoto): Observable<any> {
    this._isLoadingLogo(ctx, true);
    const formData = new FormData();
    formData.append(this._FILE_KEY, action.file, action.file.name);

    return this._apiService.upload(`${this._urlLogoResource}/${action.resId}/${this._getUploadAction()}`, formData)
      .pipe(
        map((event: UploadEventModel) => {
          switch (event.type) {
            case HttpEventType.DownloadProgress:
            case HttpEventType.UploadProgress:
              return;
            case HttpEventType.Response:
              if (isNotNullNorUndefined(event.body) &&
                ((this._mode === ResourceLogoStateModeEnum.avatar && isNotNullNorUndefined((event.body as Person).avatar))
                  || (this._mode === ResourceLogoStateModeEnum.logo && isNotNullNorUndefined(event.body.logo)))) {
                ctx.dispatch(ResourceLogoActionHelper.uploadPhotoSuccess(this._nameSpace, action));
              } else {
                ctx.dispatch(ResourceLogoActionHelper.getPhotoFail(this._nameSpace, action));
              }
              return;
            default:
              return;
          }
        }),
        catchError((error: SolidifyHttpErrorResponseModel) => {
          ctx.dispatch(ResourceLogoActionHelper.uploadPhotoFail(this._nameSpace, action));
          throw error;
        }),
      );
  }

  @RegisterDefaultAction((userAvatarNameSpace: ResourceLogoNameSpace) => userAvatarNameSpace.UploadPhotoSuccess, {}, ResourceState)
  uploadPhotoSuccess(ctx: StateContext<ResourceLogoStateModel<TResource>>, action: ResourceLogoAction.UploadPhotoSuccess): void {
    this._isLoadingLogo(ctx, false);
  }

  @RegisterDefaultAction((userAvatarNameSpace: ResourceLogoNameSpace) => userAvatarNameSpace.UploadPhotoFail, {}, ResourceState)
  uploadPhotoFail(ctx: StateContext<ResourceLogoStateModel<TResource>>, action: ResourceLogoAction.UploadPhotoFail): void {
    this._isLoadingLogo(ctx, false);
  }

  protected _isLoadingLogo(ctx: StateContext<ResourceLogoStateModel<TResource>>, isLoading: boolean): void {
    ctx.patchState({
      isLoadingLogo: isLoading,
    });
  }

  @RegisterDefaultAction((userAvatarNameSpace: ResourceLogoNameSpace) => userAvatarNameSpace.DeletePhoto, {}, ResourceState)
  deletePhoto(ctx: StateContext<ResourceLogoStateModel<TResource>>, action: ResourceLogoAction.DeletePhoto): Observable<any> {
    this._isLoadingLogo(ctx, true);

    return this._apiService.delete(`${this._urlLogoResource}/${action.resId}/${this._getDeleteAction()}`)
      .pipe(
        tap((result: any) => {
          ctx.dispatch(ResourceLogoActionHelper.deletePhotoSuccess(this._nameSpace, action));
        }),
        catchError((error: SolidifyHttpErrorResponseModel) => {
          ctx.dispatch(ResourceLogoActionHelper.deletePhotoSuccess(this._nameSpace, action));
          throw error;
        }),
      );
  }

  @RegisterDefaultAction((userAvatarNameSpace: ResourceLogoNameSpace) => userAvatarNameSpace.DeletePhotoSuccess, {}, ResourceState)
  deletePhotoSuccess(ctx: StateContext<ResourceLogoStateModel<TResource>>, action: ResourceLogoAction.DeletePhotoSuccess): void {
    ctx.patchState({
      isLoadingLogo: false,
      logo: undefined,
    });
  }

  @RegisterDefaultAction((userAvatarNameSpace: ResourceLogoNameSpace) => userAvatarNameSpace.DeletePhotoFail, {}, ResourceState)
  deletePhotoFail(ctx: StateContext<ResourceLogoStateModel<TResource>>, action: ResourceLogoAction.DeletePhotoFail): void {
    this._isLoadingLogo(ctx, false);
  }
}

export enum ResourceLogoStateModeEnum {
  avatar,
  logo,
  dataset,
}
