/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Portal - metadata.util.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {
  ArchiveMetadata,
  MetadataEnum,
} from "@app/shared/models/business/archive-metadata.model";
import {Enums} from "@enums";
import {
  ArchiveContributor,
  ArchiveFundingAgency,
  ArchiveInstitution,
  ArchiveLicense,
} from "@home/models/archive.model";
import {
  isFalse,
  isNotNullNorUndefinedNorWhiteString,
  isNullOrUndefined,
  isNullOrUndefinedOrWhiteString,
  isTrue,
  SOLIDIFY_CONSTANTS,
} from "solidify-frontend";
import Metadata = ArchiveMetadata.PublicMetadata;

const DATACITE_NAMESPACE: string = "http://datacite.org/schema/kernel-4";
const DLCM_URI_FINAL_ACCESS_LEVEL: string = "https://www.dlcm.ch/access-level/final";
const DLCM_URI_EMBARGO_ACCESS_LEVEL: string = "https://www.dlcm.ch/access-level/embargo";
const DLCM_URI_DATA_TAG: string = "https://www.dlcm.ch/data-tag";

export class MetadataUtil {
  private static readonly _domParser: DOMParser = new DOMParser();

  private static _parse(metadataXml: string): Document {
    return this._domParser.parseFromString(metadataXml, "text/xml");
  }

  private static _prefixNSResolver(prefix: string): string | null {
    if (prefix === "d") {
      return DATACITE_NAMESPACE;
    } else {
      return null;
    }
  }

  private static _getInformation(xpath: string, metadata: Metadata, xPathResult: number): XPathResult | undefined {
    const metadataXml: string = metadata[MetadataEnum.dataciteXml];
    if (isNullOrUndefined(metadataXml)) {
      // eslint-disable-next-line no-console
      console.error("metadataXml provided undefined");
      return undefined;
    }
    if (isNullOrUndefinedOrWhiteString(xpath)) {
      // eslint-disable-next-line no-console
      console.error("xpath provided undefined");
      return undefined;
    }
    const xmlDoc = this._parse(metadataXml);
    return xmlDoc.evaluate(xpath, xmlDoc, this._prefixNSResolver, xPathResult, undefined);
  }

  private static _getInformationString(xpath: string, metadata: Metadata): string {
    return this._getInformation(xpath, metadata, XPathResult.STRING_TYPE)?.stringValue;
  }

  static getTitle(metadata: Metadata): string {
    const xpath = "/d:resource/d:titles/d:title";
    return this._getInformationString(xpath, metadata);
  }

  static getOrganizationalUnitResId(metadata: Metadata): string {
    return metadata[MetadataEnum.aipOrganizationalUnit];
  }

  static getWithThumbnail(metadata: Metadata): boolean {
    const withThumbnail = metadata[MetadataEnum.aipThumbnail];
    return isTrue(withThumbnail);
  }

  static isCollection(metadata: Metadata): boolean {
    const aipUnit = metadata[MetadataEnum.aipUnit];
    return isFalse(aipUnit);
  }

  static getPublicationYear(metadata: Metadata): string {
    const xpath = "/d:resource/d:publicationYear";
    return this._getInformationString(xpath, metadata);
  }

  static getAccessLevel(metadata: Metadata): string {
    const xpath = `/d:resource/d:rightsList/d:rights[@rightsURI="${DLCM_URI_FINAL_ACCESS_LEVEL}"]`;
    return this._getInformationString(xpath, metadata);
  }

  static getDataSensibility(metadata: Metadata): string {
    const dataSensibility = metadata[MetadataEnum.aipDataTag];
    return isNullOrUndefined(dataSensibility) ? Enums.Deposit.DataSensitivityEnum.UNDEFINED : dataSensibility;
  }

  static getEmbargoAccessLevel(metadata: Metadata): string {
    const xpath = `/d:resource/d:rightsList/d:rights[@rightsURI="${DLCM_URI_EMBARGO_ACCESS_LEVEL}"]`;
    return this._getInformationString(xpath, metadata);
  }

  static getLicense(metadata: Metadata): ArchiveLicense {
    const xpath = `/d:resource/d:rightsList/d:rights[@rightsURI!="${DLCM_URI_FINAL_ACCESS_LEVEL}" and @rightsURI!="${DLCM_URI_EMBARGO_ACCESS_LEVEL}" and @rightsURI!="${DLCM_URI_DATA_TAG}"]`;
    const xPathResult = this._getInformation(xpath, metadata, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE);
    if (xPathResult.snapshotLength === 0) {
      return undefined;
    }
    const license = {} as ArchiveLicense;
    xPathResult.snapshotItem(0).childNodes.forEach(childNode => {
      license.name = childNode.textContent;
      license.url = childNode?.parentNode?.["attributes"]?.rightsURI?.value;
      license.identifier = childNode?.parentNode?.["attributes"]?.rightsIdentifier?.value;
      license.identifierType = childNode?.parentNode?.["attributes"]?.rightsIdentifierScheme?.value;
    });
    return license;
  }

  static getKeywords(metadata: Metadata): string[] {
    const xpath = `/d:resource/d:subjects/d:subject`;
    const keywords = this._getInformationString(xpath, metadata);
    if (isNullOrUndefinedOrWhiteString(keywords)) {
      return [];
    }
    return keywords.split(SOLIDIFY_CONSTANTS.SEMICOLON);
  }

  static getFundingAgencies(metadata: Metadata): ArchiveFundingAgency[] {
    const xpath = "/d:resource/d:fundingReferences/d:fundingReference";
    const xPathResult = this._getInformation(xpath, metadata, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE);
    const results = [];
    for (let i = 0, length = xPathResult.snapshotLength; i < length; ++i) {
      const fundingAgency = {} as ArchiveFundingAgency;
      xPathResult.snapshotItem(i).childNodes.forEach(childNode => {
        if (childNode.nodeName === "datacite:funderName") {
          fundingAgency.name = childNode.textContent;
        }
        if (childNode.nodeName === "datacite:funderIdentifier") {
          fundingAgency.identifierUrl = childNode.textContent;
          fundingAgency.identifierType = childNode?.["attributes"]?.funderIdentifierType?.value;
          fundingAgency.identifier = this._extractIdentifierFromUrl(fundingAgency.identifierType as Enums.IdentifiersEnum, fundingAgency.identifierUrl);
        }
      });
      results.push(fundingAgency);
    }
    return results;
  }

  static getDescriptionAbstract(metadata: Metadata): string {
    const xpath = `/d:resource/d:descriptions/d:description[@descriptionType="Abstract"]`;
    return this._getInformationString(xpath, metadata);
  }

  static getDescriptionOther(metadata: Metadata): string {
    const xpath = `/d:resource/d:descriptions/d:description[@descriptionType="Other"]`;
    return this._getInformationString(xpath, metadata);
  }

  static getDescriptionTechnicalInfo(metadata: Metadata): string {
    const xpath = `/d:resource/d:descriptions/d:description[@descriptionType="TechnicalInfo"]`;
    return this._getInformationString(xpath, metadata);
  }

  static getDescriptionTableOfContents(metadata: Metadata): string {
    const xpath = `/d:resource/d:descriptions/d:description[@descriptionType="TableOfContents"]`;
    return this._getInformationString(xpath, metadata);
  }

  static getDescriptionSeriesInformation(metadata: Metadata): string {
    const xpath = `/d:resource/d:descriptions/d:description[@descriptionType="SeriesInformation"]`;
    return this._getInformationString(xpath, metadata);
  }

  static getDescriptionMethods(metadata: Metadata): string {
    const xpath = `/d:resource/d:descriptions/d:description[@descriptionType="Methods"]`;
    return this._getInformationString(xpath, metadata);
  }

  static getDOI(metadata: Metadata): string {
    const xpath = "/d:resource/d:identifier[@identifierType='DOI']";
    return this._getInformationString(xpath, metadata);
  }

  static getContributors(metadata: Metadata): ArchiveContributor[] {
    const xpath = "/d:resource/d:creators/d:creator";
    const results = [];
    const xPathResult = this._getInformation(xpath, metadata, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE);
    for (let i = 0, length = xPathResult.snapshotLength; i < length; ++i) {
      const contributor = {
        affiliation: [],
      } as ArchiveContributor;
      xPathResult.snapshotItem(i).childNodes.forEach(childNode => {
        if (childNode.nodeName === "datacite:creatorName") {
          contributor.creatorName = childNode.textContent;
        }
        if (childNode.nodeName === "datacite:givenName") {
          contributor.givenName = childNode.textContent;
        }
        if (childNode.nodeName === "datacite:familyName") {
          contributor.familyName = childNode.textContent;
        }
        if (childNode.nodeName === "datacite:affiliation") {
          contributor.affiliation.push(childNode.textContent);
        }
        if (childNode.nodeName === "datacite:nameIdentifier") {
          contributor.orcid = childNode.textContent;
        }
      });
      results.push(contributor);
    }
    return results;
  }

  static getInstitutions(metadata: Metadata): ArchiveInstitution[] {
    const xpath = "/d:resource/d:contributors/d:contributor[@contributorType='ResearchGroup']/d:affiliation";
    const results = [];
    const xPathResult = this._getInformation(xpath, metadata, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE);
    for (let i = 0, length = xPathResult.snapshotLength; i < length; ++i) {
      const institution = {} as ArchiveInstitution;
      xPathResult.snapshotItem(i).childNodes.forEach(childNode => {
        institution.name = childNode.textContent;
        institution.identifierUrl = childNode?.parentNode?.["attributes"]?.affiliationIdentifier?.value;
        institution.identifierType = childNode?.parentNode?.["attributes"]?.affiliationIdentifierScheme?.value;
        institution.identifier = this._extractIdentifierFromUrl(institution.identifierType as Enums.IdentifiersEnum, institution.identifierUrl);
      });
      results.push(institution);
    }
    return results;
  }

  private static _extractIdentifierFromUrl(identifierType: Enums.IdentifiersEnum, identifierUrl: string): string | undefined {
    if (identifierType === Enums.IdentifiersEnum.ROR && isNotNullNorUndefinedNorWhiteString(identifierUrl)) {
      const lastIndexOfSeparator = identifierUrl.lastIndexOf(SOLIDIFY_CONSTANTS.URL_SEPARATOR);
      return lastIndexOfSeparator !== -1 ? identifierUrl.substring(lastIndexOfSeparator + 1) : undefined;
    }
    return undefined;
  }

  static getType(metadata: Metadata): string {
    const xpath = "/d:resource/d:resourceType";
    return this._getInformationString(xpath, metadata);
  }

  static getEmbargoEndDate(metadata: Metadata): Date | null {
    return this._getDate("Available", metadata);
  }

  static getAcceptedDate(metadata: Metadata): Date | null {
    return this._getDate("Accepted", metadata);
  }

  static getIssuedDate(metadata: Metadata): Date | null {
    return this._getDate("Issued", metadata);
  }

  static getSubmittedDate(metadata: Metadata): Date | null {
    return this._getDate("Submitted", metadata);
  }

  static getSize(metadata: Metadata): number | null {
    return metadata[MetadataEnum.aipSize];
  }

  static getFileNumber(metadata: Metadata): string | null {
    if (isNullOrUndefined(metadata[MetadataEnum.aipFiles])) {
      return null;
    }
    return metadata[MetadataEnum.aipFiles].toString();
  }

  private static _getDate(name: string, metadata: Metadata): Date | null {
    if (isNullOrUndefined(metadata)) {
      return null;
    }
    const xpath = `/d:resource/d:dates/d:date[@dateType="${name}"]`;
    const result = this._getInformationString(xpath, metadata);
    if (isNullOrUndefined(result)) {
      return null;
    }
    const date = new Date(result);
    if (date.toString() === "Invalid Date") {
      return null;
    }
    return date;
  }

  static getDatacite(metadata: Metadata): string {
    return metadata[MetadataEnum.dataciteXml];
  }
}
