/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Portal - app-carousel.state.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {
  HttpClient,
  HttpHeaders,
  HttpResponse,
} from "@angular/common/http";
import {Injectable} from "@angular/core";
import {AppState} from "@app/stores/app.state";
import {AppCarouselAction} from "@app/stores/carousel/app-carousel.action";
import {environment} from "@environments/environment";
import {
  Action,
  Actions,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {StateEnum} from "@shared/enums/state.enum";
import {Observable} from "rxjs";
import {
  catchError,
  map,
  tap,
} from "rxjs/operators";
import {
  ApiService,
  BaseStateModel,
  BasicState,
  defaultBaseStateInitValue,
  isNonEmptyString,
  isNullOrUndefined,
  MemoizedUtil,
  NotificationService,
  SolidifyError,
  SolidifyHttpErrorResponseModel,
} from "solidify-frontend";

export interface AppCarouselStateModel extends BaseStateModel {
  tiles: Node[] | undefined;
}

@Injectable()
@State<AppCarouselStateModel>({
  name: StateEnum.application_carousel,
  defaults: {
    ...defaultBaseStateInitValue(),
    tiles: undefined,
  },
})
export class AppCarouselState extends BasicState<AppCarouselStateModel> {
  constructor(protected readonly _httpClient: HttpClient,
              protected readonly _apiService: ApiService,
              protected readonly _store: Store,
              protected readonly _notificationService: NotificationService,
              protected readonly _actions$: Actions) {
    super();
  }

  @Action(AppCarouselAction.GetCarousel)
  getCarousel(ctx: StateContext<AppCarouselStateModel>, action: AppCarouselAction.GetCarousel): Observable<string> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });

    let language = MemoizedUtil.selectSnapshot(this._store, AppState, state => state.appLanguage);
    if (isNullOrUndefined(language)) {
      language = environment.defaultLanguage;
    }
    let headers = new HttpHeaders();
    headers = headers.set("Accept", ["application/xml"]);
    headers = headers.set("Content-Type", "text/xml");

    let url = `assets/themes/${environment.theme}/carousel/`;
    if (isNonEmptyString(environment.carouselUrl)) {
      url = environment.carouselUrl;
    }
    return this._httpClient.get(url + `carousel_${language}.html`, {
      headers,
      observe: "response",
      responseType: "text",
    }).pipe(
      map((response: HttpResponse<string>) => response.body),
      tap((htmlContent: string) => {
        const parser = new DOMParser();
        const parsedHtml = parser.parseFromString(htmlContent, "text/html");
        const nodeList = parsedHtml.querySelectorAll("body > ul > li");
        const listTiles = [];
        nodeList.forEach(node => {
          listTiles.push(node);
        });
        ctx.dispatch(new AppCarouselAction.GetCarouselSuccess(action, listTiles));
      }),
      catchError((error: SolidifyHttpErrorResponseModel) => {
        ctx.dispatch(new AppCarouselAction.GetCarouselFail(action));
        throw new SolidifyError("AppCarouselAction GetCarousel", error);
      }),
    );
  }

  @Action(AppCarouselAction.GetCarouselSuccess)
  getCarouselSuccess(ctx: StateContext<AppCarouselStateModel>, action: AppCarouselAction.GetCarouselSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
      tiles: action.tiles,
    });
  }

  @Action(AppCarouselAction.GetCarouselFail)
  getCarouselFail(ctx: StateContext<AppCarouselStateModel>, action: AppCarouselAction.GetCarouselFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
  }
}
