/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Portal - app-person.state.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {Injectable} from "@angular/core";
import {ApiEnum} from "@app/shared/enums/api.enum";
import {
  AppPersonAction,
  appPersonActionNameSpace,
} from "@app/stores/person/app-person.action";
import {AppPersonNotificationTypeAction} from "@app/stores/person/notification-type/app-person-notification-type.action";
import {
  AppPersonNotificationTypeState,
  AppPersonNotificationTypeStateModel,
} from "@app/stores/person/notification-type/app-person-notification-type.state";
import {Person} from "@models";
import {
  Action,
  Actions,
  Selector,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {StateEnum} from "@shared/enums/state.enum";
import {Observable} from "rxjs";
import {
  catchError,
  map,
  tap,
} from "rxjs/operators";
import {
  ApiService,
  defaultRelation2TiersStateInitValue,
  defaultResourceStateInitValue,
  isNullOrUndefined,
  NotificationService,
  ofSolidifyActionCompleted,
  OverrideDefaultAction,
  QueryParameters,
  ResourceState,
  ResourceStateModel,
  SolidifyHttpErrorResponseModel,
  SolidifyStateError,
  StoreUtil,
} from "solidify-frontend";
import {environment} from "../../../environments/environment";

export interface AppPersonStateModel extends ResourceStateModel<Person> {
  [StateEnum.application_person_notificationType]: AppPersonNotificationTypeStateModel;
}

@Injectable()
@State<AppPersonStateModel>({
  name: StateEnum.application_person,
  defaults: {
    ...defaultResourceStateInitValue(),
    [StateEnum.application_person_notificationType]: {...defaultRelation2TiersStateInitValue()},
    queryParameters: new QueryParameters(environment.defaultEnumValuePageSizeOption),
  },
  children: [
    AppPersonNotificationTypeState,
  ],
})
export class AppPersonState extends ResourceState<AppPersonStateModel, Person> {
  constructor(protected readonly _apiService: ApiService,
              protected readonly _store: Store,
              protected readonly _notificationService: NotificationService,
              protected readonly _actions$: Actions) {
    super(_apiService, _store, _notificationService, _actions$, {
      nameSpace: appPersonActionNameSpace,
      keepCurrentStateAfterUpdate: true,
    });
  }

  protected get _urlResource(): string {
    return ApiEnum.adminPeople;
  }

  @Selector()
  static isReadyToBeDisplayed(state: AppPersonStateModel): boolean {
    return this.isReadyToBeDisplayedInCreateMode
      && !isNullOrUndefined(state.current)
      && state[StateEnum.application_person_notificationType].isLoadingCounter === 0;
  }

  @Selector()
  static isReadyToBeDisplayedInCreateMode(state: AppPersonStateModel): boolean {
    return true;
  }

  // Override method here to form to use url override
  @OverrideDefaultAction()
  @Action(AppPersonAction.GetById)
  getById(ctx: StateContext<AppPersonStateModel>, action: AppPersonAction.GetById): Observable<Person> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
      current: null,
    });

    return this._apiService.getById<Person>(ApiEnum.adminPeople, action.id)
      .pipe(
        tap((model: Person) => {
          ctx.dispatch(new AppPersonAction.GetByIdSuccess(action, model));
        }),
        catchError((error: SolidifyHttpErrorResponseModel) => {
          ctx.dispatch(new AppPersonAction.GetByIdFail(action));
          throw new SolidifyStateError(this, error);
        }),
      );
  }

  @Action(AppPersonAction.AddCurrentPerson)
  getCurrentUserSuccess(ctx: StateContext<AppPersonStateModel>, action: AppPersonAction.AddCurrentPerson): void {
    ctx.patchState({
      current: action.model,
    });
  }

  @OverrideDefaultAction()
  @Action(AppPersonAction.Update)
  update(ctx: StateContext<AppPersonStateModel>, action: AppPersonAction.Update): Observable<Person> {
    return super._internalUpdate(ctx, action)
      .pipe(
        tap(model => {
          this._updateSubResource(model, action, ctx)
            .subscribe(success => {
              if (success) {
                ctx.dispatch(new AppPersonAction.UpdateSuccess(action, model));
              } else {
                ctx.dispatch(new AppPersonAction.UpdateFail(action));
              }
            });
        }),
        catchError((error: SolidifyHttpErrorResponseModel) => {
          ctx.dispatch(new AppPersonAction.UpdateFail(action));
          throw new SolidifyStateError(this, error);
        }),
      );
  }

  private _updateSubResource(model: Person, action: AppPersonAction.Create | AppPersonAction.Update, ctx: StateContext<AppPersonStateModel>): Observable<boolean> {
    const personId = model.resId;
    const newNotificationType = action.modelFormControlEvent.model.notificationTypes.map(o => o.resId);

    return StoreUtil.dispatchParallelActionAndWaitForSubActionsCompletion(ctx, [
      {
        action: new AppPersonNotificationTypeAction.Update(personId, newNotificationType),
        subActionCompletions: [
          this._actions$.pipe(ofSolidifyActionCompleted(AppPersonNotificationTypeAction.UpdateSuccess)),
          this._actions$.pipe(ofSolidifyActionCompleted(AppPersonNotificationTypeAction.UpdateFail)),
        ],
      },
    ]).pipe(
      map(result => result.success),
    );
  }
}
