/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Portal - app-system-property.state.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {Injectable} from "@angular/core";
import {ApiEnum} from "@app/shared/enums/api.enum";
import {
  AppSystemPropertyAction,
  appSystemPropertyActionNameSpace,
} from "@app/stores/system-property/app-system-property.action";
import {environment} from "@environments/environment";
import {SystemProperty} from "@models";
import {
  Action,
  Actions,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {StateEnum} from "@shared/enums/state.enum";
import {Observable} from "rxjs";
import {
  catchError,
  tap,
} from "rxjs/operators";
import {
  ApiService,
  defaultResourceStateInitValue,
  isNullOrUndefined,
  NotificationService,
  QueryParameters,
  ResourceState,
  ResourceStateModel,
  SolidifyHttpErrorResponseModel,
  SolidifyStateError,
} from "solidify-frontend";

export interface AppSystemPropertyStateModel extends ResourceStateModel<SystemProperty> {
}

@Injectable()
@State<AppSystemPropertyStateModel>({
  name: StateEnum.application_systemProperty,
  defaults: {
    ...defaultResourceStateInitValue(),
    queryParameters: new QueryParameters(environment.defaultEnumValuePageSizeOption),
  },
})
export class AppSystemPropertyState extends ResourceState<AppSystemPropertyStateModel, SystemProperty> {
  constructor(protected readonly _apiService: ApiService,
              protected readonly _store: Store,
              protected readonly _notificationService: NotificationService,
              protected readonly _actions$: Actions) {
    super(_apiService, _store, _notificationService, _actions$, {
      nameSpace: appSystemPropertyActionNameSpace,
    });
  }

  protected get _urlResource(): string {
    return ApiEnum.adminSystemProperties;
  }

  @Action(AppSystemPropertyAction.GetSystemProperties)
  getSystemProperty(ctx: StateContext<AppSystemPropertyStateModel>, action: AppSystemPropertyAction.GetSystemProperties): Observable<SystemProperty> {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    });

    const url = ApiEnum.adminSystemProperties;
    if (isNullOrUndefined(environment.allowedUrls)) {
      environment.allowedUrls = [];
    }
    environment.allowedUrls.push(url);
    return this._apiService.getByIdInPath<SystemProperty>(url)
      .pipe(
        tap((systemProperty: SystemProperty) => {
          ctx.dispatch(new AppSystemPropertyAction.GetSystemPropertiesSuccess(action, systemProperty));
        }),
        catchError((error: SolidifyHttpErrorResponseModel) => {
          ctx.dispatch(new AppSystemPropertyAction.GetSystemPropertiesFail(action));
          throw new SolidifyStateError(this, error);
        }),
      );
  }

  @Action(AppSystemPropertyAction.GetSystemPropertiesSuccess)
  getSystemPropertiesSuccess(ctx: StateContext<AppSystemPropertyStateModel>, action: AppSystemPropertyAction.GetSystemPropertiesSuccess): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
      current: action.systemProperty,
    });
  }

  @Action(AppSystemPropertyAction.GetSystemPropertiesFail)
  getSystemPropertiesFail(ctx: StateContext<AppSystemPropertyStateModel>, action: AppSystemPropertyAction.GetSystemPropertiesFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
  }
}
