/*-
 * %%----------------------------------------------------------------------------------------------
 * DLCM Technology - DLCM Portal - app-user.state.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2022 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import {HttpClient} from "@angular/common/http";
import {Injectable} from "@angular/core";
import {ApiEnum} from "@app/shared/enums/api.enum";
import {AppPersonAction} from "@app/stores/person/app-person.action";
import {
  AppUserAction,
  appUserActionNameSpace,
} from "@app/stores/user/app-user.action";
import {ApmService} from "@elastic/apm-rum-angular";
import {environment} from "@environments/environment";
import {User} from "@models";
import {
  Action,
  Actions,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {ApiActionNameEnum} from "@shared/enums/api-action-name.enum";
import {urlSeparator} from "@shared/enums/routes.enum";
import {StateEnum} from "@shared/enums/state.enum";
import {AppTourService} from "@shared/services/app-tour.service";
import {ResourceLogoStateModel} from "@shared/stores/resource-logo/resource-logo-state.model";
import {
  defaultResourceLogoStateInitValue,
  ResourceLogoState,
  ResourceLogoStateModeEnum,
} from "@shared/stores/resource-logo/resource-logo.state";
import {Observable} from "rxjs";
import {
  catchError,
  tap,
} from "rxjs/operators";
import {
  ApiService,
  CollectionTyped,
  isNotNullNorUndefined,
  isNullOrUndefined,
  NotificationService,
  QueryParameters,
  SolidifyHttpErrorResponseModel,
  SolidifyStateError,
} from "solidify-frontend";

export const defaultAppUserStateModel: () => AppUserStateModel = () =>
  ({
    ...defaultResourceLogoStateInitValue(),
    queryParameters: new QueryParameters(environment.defaultEnumValuePageSizeOption),
  });

export interface AppUserStateModel extends ResourceLogoStateModel<User> {
}

@Injectable()
@State<AppUserStateModel>({
  name: StateEnum.application_user,
  defaults: {
    ...defaultAppUserStateModel(),
  },
})
export class AppUserState extends ResourceLogoState<AppUserStateModel, User> {
  constructor(protected readonly _apiService: ApiService,
              protected readonly _store: Store,
              protected readonly _notificationService: NotificationService,
              protected readonly _actions$: Actions,
              protected readonly _httpClient: HttpClient,
              protected readonly _appTourService: AppTourService,
              protected readonly _apmService: ApmService) {
    super(_apiService, _store, _notificationService, _actions$, {
      nameSpace: appUserActionNameSpace,
    }, _httpClient, ResourceLogoStateModeEnum.avatar);
  }

  protected get _urlResource(): string {
    return ApiEnum.adminUsers;
  }

  protected get _urlLogoResource(): string {
    return ApiEnum.adminPeople;
  }

  @Action(AppUserAction.GetCurrentUser)
  getCurrentUser(ctx: StateContext<AppUserStateModel>, action: AppUserAction.GetCurrentUser): Observable<CollectionTyped<User>> {
    return this._apiService.get<User>(ApiEnum.adminUsers + urlSeparator + ApiActionNameEnum.AUTHENTICATED, null)
      .pipe(
        tap((user: User | any) => {
          ctx.dispatch(new AppUserAction.GetCurrentUserSuccess(action, user));
        }),
        catchError((error: SolidifyHttpErrorResponseModel) => {
          ctx.dispatch(new AppUserAction.GetCurrentUserFail(action));
          throw new SolidifyStateError(this, error);
        }),
      );
  }

  @Action(AppUserAction.GetCurrentUserSuccess)
  getCurrentUserSuccess(ctx: StateContext<AppUserStateModel>, action: AppUserAction.GetCurrentUserSuccess): void {
    ctx.patchState({
      current: action.user,
    });

    const apm = this._apmService.apm;
    if (isNotNullNorUndefined(apm)) {
      apm.setUserContext({
        id: action.user.resId,
        email: action.user.email,
        username: action.user.firstName + " " + action.user.lastName,
      });
    }

    ctx.dispatch(new AppPersonAction.AddCurrentPerson(action.user.person));

    if (isNotNullNorUndefined(action.user.person?.avatar)) {
      ctx.dispatch(new AppUserAction.GetPhoto(action.user.person.resId));
    }

    if (isNullOrUndefined(action.user.lastLoginTime)) {
      this._appTourService.runFullTour();
    }
  }
}
